/*
 * This code was created by Jeff Molofee '99
 * (ported to Linux by Ti Leggett '01)
 * (ported to i.mx51, i.mx31 and x11 by Freescale '10)
 * If you've found this code useful, please let him know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <assert.h>
#include <math.h>
#include <GLU3/glu3.h>]

#ifndef USE_MX31
#include "GLES/gl.h"
#include "GLES/glext.h"
#ifndef ANDROID
#include "EGL/egl.h"
#endif
#else

#include <GLES/gl.h>
#include <GLES/egl.h>
#endif

#ifdef EGL_USE_X11
#include <X11/X.h>
#include <X11/Xlib.h>
#endif 

#ifdef ANDROID
#include <jni.h>
#endif

#ifndef ANDROID
EGLDisplay			egldisplay;
EGLConfig			eglconfig;
EGLSurface			eglsurface;
EGLContext			eglcontext;
int				currentFrame = 0;
#endif

#ifdef EGL_USE_X11
Display 			*display;
#endif


/* function to release/destroy our resources and restoring the old desktop */
void Cleanup()
{
    
}

/* general OpenGL initialization function */
int init(void)
{
#ifndef ANDROID
    static const EGLint s_configAttribs[] =
    {
	EGL_RED_SIZE,		5,
	EGL_GREEN_SIZE, 	6,
	EGL_BLUE_SIZE,		5,
	EGL_ALPHA_SIZE, 	0,
      	EGL_SAMPLES, 		0,
      	EGL_NONE
    };

    EGLint numconfigs;

#ifdef EGL_USE_X11
    //Open X display
    printf("XOpenDisplay \n");
    display = XOpenDisplay(NULL);
    assert(display != NULL);	

    //Get the default screen from display
    int screen;
    printf("DefaultScreen [");
    screen = DefaultScreen(display);
    printf("%d]\n", screen);

    //get root window
    Window window, rootwindow; 
    printf("RootWindow \n");
    rootwindow = RootWindow(display,screen); 
	
    //get egldisplay from X display
    printf("eglGetDisplay (%d)\n", (int)display);
    egldisplay = eglGetDisplay ( display);
    assert(eglGetError() == EGL_SUCCESS);

    printf("Disp = %d \n",(int) egldisplay);

    //initializa egl display
    printf("eglInitialize \n");
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);

    //tell the driver we are using OpenGL ES
    printf("eglBindAPI \n");
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    printf("eglChooseConfig \n");
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    assert(eglGetError() == EGL_SUCCESS);

    assert(numconfigs == 1);

    //Create and x window
    printf("XCreateSimpleWindow \n");
    window = XCreateSimpleWindow(display, rootwindow, 0, 0, 400, 533, 0, 0, WhitePixel (display, screen));

    XMapWindow(display, window);

    //get an eglsurface from the display using our egl configuration
    printf("*eglCreateWindowSurface \n");
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, window, NULL);
    assert(eglGetError() == EGL_SUCCESS);

#else

    //get egl display
    egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    //Initialize egl
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);
    //tell the driver we are using OpenGL ES
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    printf("chooseconfig, \n");
    assert(eglGetError() == EGL_SUCCESS);
    assert(numconfigs == 1);

    //You must pass in the file system handle to the linux framebuffer when creating a window
#ifndef USE_MX31
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, open("/dev/fb0", O_RDWR), NULL);
#else
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, (NativeWindowType)0, NULL);
#endif
    printf("createwindow, \n");
    assert(eglGetError() == EGL_SUCCESS);

#endif

    //create the egl graphics context
    eglcontext = eglCreateContext(egldisplay, eglconfig, NULL, NULL);
    printf("creatcontext, \n");
    assert(eglGetError() == EGL_SUCCESS);

    //make the context current
    eglMakeCurrent(egldisplay, eglsurface, eglsurface, eglcontext);
    printf("makecurrent, \n");
    assert(eglGetError() == EGL_SUCCESS);
#endif
    /* Enable smooth shading */
    glShadeModel(GL_SMOOTH);

    /* Set the background black */
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

    /* Depth buffer setup */
    glClearDepthf(1.0f);

    /* Enables Depth Testing */
    glEnable(GL_DEPTH_TEST);

    /* The Type Of Depth Test To Do */
    glDepthFunc(GL_LEQUAL);

    /* Really Nice Perspective Calculations */
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
#ifndef ANDROID
    /*get width and height from egl*/
    EGLint h, w;
    eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &w);
    eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &h);

    /*change to projection matrix*/
    glMatrixMode(GL_PROJECTION);
    /*reset the projection matrix*/
    glLoadIdentity();
    /*set the viewport*/
    glViewport(0, 0, w, h);

    GLUmat4 perspective;
    /*use glu to set perspective*/
    gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
    glMultMatrixf(&perspective.col[0].values[0]);

    /*get back to model view matrix*/
    glMatrixMode(GL_MODELVIEW);
    /*reset modevl view matrix*/
    glLoadIdentity();
#endif
    return 1;
}

/* Here goes our drawing code */
void render()
{

    GLfloat colors[4][4];
    GLfloat vertices[4][3];

    /* Clear The Screen And The Depth Buffer */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* Enable COLOR and VERTEX arrays */
    glEnableClientState(GL_COLOR_ARRAY);
    glEnableClientState(GL_VERTEX_ARRAY);

    /* Setup pointers to COLOR and VERTEX arrays */
    glColorPointer(4, GL_FLOAT, 0, colors);
    glVertexPointer(3, GL_FLOAT, 0, vertices);

    /* Move Left 1.5 Units And Into The Screen 6.0 */
    glLoadIdentity();
    glTranslatef(-1.5f, 0.0f, -6.0f);

    /* Red */
    colors[0][0]=1.0f; colors[0][1]=0.0f; colors[0][2]=0.0f; colors[0][3]=1.0f;
    /* Top Of Triangle */
    vertices[0][0]=0.0f; vertices[0][1]=1.0f; vertices[0][2]=0.0f;
    /* Green */
    colors[1][0]=0.0f; colors[1][1]=1.0f; colors[1][2]=0.0f; colors[1][3]=1.0f;
    /* Left Of Triangle */
    vertices[1][0]=-1.0f; vertices[1][1]=-1.0f; vertices[1][2]=0.0f;
    /* Blue */
    colors[2][0]=0.0f; colors[2][1]=0.0f; colors[2][2]=1.0f; colors[2][3]=1.0f;
    /* Right Of Triangle */
    vertices[2][0]=1.0f; vertices[2][1]=-1.0f; vertices[2][2]=0.0f;

    /* Drawing Using Triangles, draw triangles using 3 vertices */
    glDrawArrays(GL_TRIANGLES, 0, 3);

    /* Move Right 3 Units */
    glLoadIdentity();
    glTranslatef(1.5f, 0.0f, -6.0f);

    /* Set The Color To Blue One Time Only */
    glColor4f(0.5f, 0.5f, 1.0f, 1.0f);

    /* Disable vertex color array, use global color */
    glDisableClientState(GL_COLOR_ARRAY);

    /* Top Right Of The Quad    */
    vertices[0][0]=1.0f;  vertices[0][1]=1.0f;  vertices[0][2]=0.0f;
    /* Top Left Of The Quad     */
    vertices[1][0]=-1.0f; vertices[1][1]=1.0f;  vertices[1][2]=0.0f;
    /* Bottom Left Of The Quad  */
    vertices[2][0]=1.0f;  vertices[2][1]=-1.0f; vertices[2][2]=0.0f;
    /* Bottom Right Of The Quad */
    vertices[3][0]=-1.0f; vertices[3][1]=-1.0f; vertices[3][2]=0.0f;

    /* Drawing using triangle strips, draw triangles using 4 vertices */
    glDrawArrays(GL_TRIANGLE_STRIP, 0, 4);

    /* Disable vertex array */
    glDisableClientState(GL_VERTEX_ARRAY);

    /* Flush all drawings */
    glFinish();
}

void resize(int w, int h)
{
	
	/*change to projection matrix*/
    	glMatrixMode(GL_PROJECTION);
	/*reset the projection matrix*/
    	glLoadIdentity();
	/*set the viewport*/
    	glViewport(0, 0, w, h);

	GLUmat4 perspective;
	/*use glu to set perspective*/
    	gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
	glMultMatrixf(&perspective.col[0].values[0]);
	
	/*get back to model view matrix*/
    	glMatrixMode(GL_MODELVIEW);
	/*reset modevl view matrix*/
    	glLoadIdentity();


}

#ifdef ANDROID
void Java_com_freescale_lesson03_Lesson03Renderer_nativeInit( JNIEnv*  env ){
	init();
}

void Java_com_freescale_lesson03_Lesson03Renderer_nativeRender(JNIEnv*  env ){
	render();
}

void Java_com_freescale_lesson03_Lesson03Renderer_nativeResize(JNIEnv*  env, jobject thiz, jint w, jint h ){
	resize(w, h);
}
#endif

#ifndef ANDROID
void deinit(void)
{
	//call clean up to release any memory allocated
	Cleanup();
	//Make a empty surface current
	eglMakeCurrent(egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	assert(eglGetError() == EGL_SUCCESS);
	//destroy egl desplay
	eglTerminate(egldisplay);
	assert(eglGetError() == EGL_SUCCESS);
	//end egl thread
	eglReleaseThread();
#ifdef EGL_USE_X11
	//close x display	
	XCloseDisplay(display);
#endif
}


int main(int argc, char** argv)
{
    assert( init() );

	while (currentFrame < 1000)
	{
		EGLint width = 0;
		EGLint height = 0;
		eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &height);
		//render(width, height);
		render();
		currentFrame++;
		eglSwapBuffers(egldisplay, eglsurface);
	}
	
    deinit();
    return 1;
}
#endif
