/*
 * This code was created by Jeff Molofee '99
 * (ported to Linux by Ti Leggett '01)
 * (ported to i.mx51, i.mx31 and x11 by Freescale '10)
 * If you've found this code useful, please let him know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <assert.h>
#include <math.h>
#include <GLU3/glu3.h>
#include <FSL/fslutil.h>

#ifndef USE_MX31
#include "GLES/gl.h"
#include "GLES/glext.h"
#ifndef ANDROID
#include "EGL/egl.h"
#endif

#else
#include <GLES/gl.h>
#include <GLES/egl.h>
#endif

#ifdef EGL_USE_X11
#include <X11/X.h>
#include <X11/Xlib.h>
#endif

#ifdef ANDROID
#include <jni.h>
#endif

#ifndef ANDROID
EGLDisplay			egldisplay;
EGLConfig			eglconfig;
EGLSurface			eglsurface;
EGLContext			eglcontext;
int				currentFrame = 0;
#endif

#ifdef EGL_USE_X11
Display 			*display;
#endif


/* Number of stars */
#define NUM 50

/* Twinkling stars */
int  twinkle=1;

/* Define the star structure */
typedef struct
{
    int r, g, b;   /* Stars Color */
    GLfloat dist;  /* Stars Distance From Center */
    GLfloat angle; /* Stars Current Angle */
} star;

star stars[NUM];       /* Make an array of size 'NUM' of stars */

GLfloat zoom=-15.0f;   /* Viewing Distance Away From Stars */
GLfloat tilt=90.0f;    /* Tilt The View */

GLuint loop;           /* General Loop Variable */
GLuint texture[1];     /* Storage For One Texture */

/* function to load in bitmap as a GL texture */
int LoadGLTextures()
{
    Image *image1;

    // allocate space for texture we will use
    image1 = (Image *) malloc(sizeof(Image));
    if (image1 == NULL) 
    {
	printf("Error allocating space for image");
	return 0;
    }	

#ifdef ANDROID
    if (ImageLoad( "/data/app/lesson09_imx.bmp", image1))
#else
    if (ImageLoad( "lesson09_imx.bmp", image1))
#endif   
    {
        /* Create The Textures */
        glGenTextures(1, &texture[0]);

        /* Load in texture 1 */
        /* Typical Texture Generation Using Data From The Bitmap */
        glBindTexture(GL_TEXTURE_2D, texture[0]);

	/* Linear Filtering */
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

        /* Generate The Texture */
        glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,image1->sizeX,image1->sizeY,0,GL_RGB,GL_UNSIGNED_BYTE,image1->data);
        
    }else return 0;

    /* Free up any memory we may have used */
    if (image1 != NULL)
    {
        free(image1);
    }

    return 1;
}

/* function to release/destroy our resources and restoring the old desktop */
void Cleanup()
{

}

/* general OpenGL initialization function */
int init(void)
{
#ifndef ANDROID
    static const EGLint s_configAttribs[] =
    {
	EGL_RED_SIZE,		5,
	EGL_GREEN_SIZE, 	6,
	EGL_BLUE_SIZE,		5,
	EGL_ALPHA_SIZE, 	0,
      	EGL_SAMPLES, 		0,
      	EGL_NONE
    };

    EGLint numconfigs;

#ifdef EGL_USE_X11
    //Open X display
    printf("XOpenDisplay \n");
    display = XOpenDisplay(NULL);
    assert(display != NULL);	

    //Get the default screen from display
    int screen;
    printf("DefaultScreen [");
    screen = DefaultScreen(display);
    printf("%d]\n", screen);

    //get root window
    Window window, rootwindow; 
    printf("RootWindow \n");
    rootwindow = RootWindow(display,screen); 
	
    //get egldisplay from X display
    printf("eglGetDisplay (%d)\n", (int)display);
    egldisplay = eglGetDisplay ( display);
    assert(eglGetError() == EGL_SUCCESS);

    printf("Disp = %d \n",(int) egldisplay);

    //initializa egl display
    printf("eglInitialize \n");
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);

    //tell the driver we are using OpenGL ES
    printf("eglBindAPI \n");
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    printf("eglChooseConfig \n");
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    assert(eglGetError() == EGL_SUCCESS);

    assert(numconfigs == 1);

    //Create and x window
    printf("XCreateSimpleWindow \n");
    window = XCreateSimpleWindow(display, rootwindow, 0, 0, 400, 533, 0, 0, WhitePixel (display, screen));

    XMapWindow(display, window);

    //get an eglsurface from the display using our egl configuration
    printf("*eglCreateWindowSurface \n");
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, window, NULL);
    assert(eglGetError() == EGL_SUCCESS);

#else

    //get egl display
    egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    //Initialize egl
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);
    //tell the driver we are using OpenGL ES
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    printf("chooseconfig, \n");
    assert(eglGetError() == EGL_SUCCESS);
    assert(numconfigs == 1);

    //You must pass in the file system handle to the linux framebuffer when creating a window
#ifndef USE_MX31
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, open("/dev/fb0", O_RDWR), NULL);
#else
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, (NativeWindowType)0, NULL);
#endif
    printf("createwindow, \n");
    assert(eglGetError() == EGL_SUCCESS);

    #endif

    //create the egl graphics context
    eglcontext = eglCreateContext(egldisplay, eglconfig, NULL, NULL);
    printf("creatcontext, \n");
    assert(eglGetError() == EGL_SUCCESS);

    //make the context current
    eglMakeCurrent(egldisplay, eglsurface, eglsurface, eglcontext);
    printf("makecurrent, \n");
    assert(eglGetError() == EGL_SUCCESS);
#endif
    /* Load in the texture */
    if (!LoadGLTextures())
    {
        return 0;
    }

    /* Enable smooth shading */
    glShadeModel(GL_SMOOTH);

    /* Set the background black */
    glClearColor(0.0f, 0.0f, 0.0f, 0.0f);

    /* Depth buffer setup */
    glClearDepthf(1.0f);

    /* Really Nice Perspective Calculations */
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    /* Enable Blending */
    glEnable(GL_BLEND);

    /* Set The Blending Function For Translucency */
    glBlendFunc(GL_SRC_ALPHA, GL_ONE);

    /* Create A Loop That Goes Through All The Stars */
    for (loop=0; loop<NUM; loop++)
    {
        /* Start All The Stars At Angle Zero */
        stars[loop].angle=0.0f;

        /* Calculate Distance From The Center */
        stars[loop].dist=((float)loop/NUM)*5.0f;
        /* Give star[loop] A Random Red Intensity */
        stars[loop].r=rand() % 256;
        /* Give star[loop] A Random Green Intensity */
        stars[loop].g=rand() % 256;
        /* Give star[loop] A Random Blue Intensity */
        stars[loop].b=rand() % 256;
    }

    /* Enable Texture Mapping */
    glEnable(GL_TEXTURE_2D);
#ifndef ANDROID
    /*set perspective*/
    EGLint h, w;
    /*get width and height from egl*/
    eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &w);
    eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &h);

    /*change to projection matrix*/
    glMatrixMode(GL_PROJECTION);
    /*reset the projection matrix*/
    glLoadIdentity();
    /*set the viewport*/
    glViewport(0, 0, w, h);

    GLUmat4 perspective;
    /*use glu to set perspective*/
    gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
    glMultMatrixf(&perspective.col[0].values[0]);

    /*get back to model view matrix*/
    glMatrixMode(GL_MODELVIEW);
    /*reset modevl view matrix*/
    glLoadIdentity();
#endif
    return 1;
}

/* Here goes our drawing code */
int render()
{
    static int spin=0;

    GLfloat texcoords[4][2];
    GLfloat vertices[4][3];
    GLubyte indices[4]={1, 0, 2, 3}; /* QUAD to TRIANGLE_STRIP conversion; */

    /* Set pointers to vertices and texcoords */
    glVertexPointer(3, GL_FLOAT, 0, vertices);
    glTexCoordPointer(2, GL_FLOAT, 0, texcoords);

    /* Enable vertices and texcoords arrays */
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);

    /* Clear The Screen And The Depth Buffer */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    /* Select Our Texture */
    glBindTexture(GL_TEXTURE_2D, texture[0]);
    glLoadIdentity();

    /* Loop Through All The Stars */
    for (loop=0; loop<NUM; loop++)
    {
        /* Reset The View Before We Draw Each Star */
        glLoadIdentity();
        /* Zoom Into The Screen (Using The Value In 'zoom') */
        glTranslatef(0.0f, 0.0f, zoom);

        /* Tilt The View (Using The Value In 'tilt') */
        glRotatef(tilt, 1.0f, 0.0f, 0.0f);
        /* Rotate To The Current Stars Angle */
        glRotatef(stars[loop].angle, 0.0f, 1.0f, 0.0f);

        /* Move Forward On The X Plane */
        glTranslatef(stars[loop].dist, 0.0f, 0.0f);

        /* Cancel The Current Stars Angle */
        glRotatef(-stars[loop].angle, 0.0f, 1.0f, 0.0f);
        /* Cancel The Screen Tilt */
        glRotatef(-tilt, 1.0f, 0.0f, 0.0f);

        /* Twinkling Stars Enabled */
        if (twinkle)
        {
            /* Assign A Color Using Bytes */
            glColor4f((GLfloat)stars[(NUM-loop)-1].r/255.0f,
                      (GLfloat)stars[(NUM-loop)-1].g/255.0f,
                      (GLfloat)stars[(NUM-loop)-1].b/255.0f, 1.0f);

            /* Fill texture and vertex arrays */
            texcoords[0][0]=0.0f; texcoords[0][1]=0.0f;
            vertices[0][0]=-1.0f; vertices[0][1]=-1.0f; vertices[0][2]=0.0f;
            texcoords[1][0]=1.0f; texcoords[1][1]=0.0f;
            vertices[1][0]=1.0f; vertices[1][1]=-1.0f; vertices[1][2]=0.0f;
            texcoords[2][0]=1.0f; texcoords[2][1]=1.0f;
            vertices[2][0]=1.0f; vertices[2][1]=1.0f; vertices[2][2]=0.0f;
            texcoords[3][0]=0.0f; texcoords[3][1]=1.0f;
            vertices[3][0]=-1.0f; vertices[3][1]=1.0f; vertices[3][2]=0.0f;

            /* Draw one textured plane using two stripped triangles */
            glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

        }

        /* Rotate The Star On The Z Axis */
        glRotatef(spin, 0.0f, 0.0f, 1.0f);

        /* Assign A Color Using Bytes */
        glColor4f((GLfloat)stars[loop].r/255.0f,
                  (GLfloat)stars[loop].g/255.0f,
                  (GLfloat)stars[loop].b/255.0f, 1.0f);

        /* Fill texture and vertex arrays */
        texcoords[0][0]=0.0f; texcoords[0][1]=0.0f;
        vertices[0][0]=-1.0f; vertices[0][1]=-1.0f; vertices[0][2]=0.0f;
        texcoords[1][0]=1.0f; texcoords[1][1]=0.0f;
        vertices[1][0]=1.0f; vertices[1][1]=-1.0f; vertices[1][2]=0.0f;
        texcoords[2][0]=1.0f; texcoords[2][1]=1.0f;
        vertices[2][0]=1.0f; vertices[2][1]=1.0f; vertices[2][2]=0.0f;
        texcoords[3][0]=0.0f; texcoords[3][1]=1.0f;
        vertices[3][0]=-1.0f; vertices[3][1]=1.0f; vertices[3][2]=0.0f;

        /* Draw one textured plane using two stripped triangles */
        glDrawElements(GL_TRIANGLE_STRIP, 4, GL_UNSIGNED_BYTE, indices);

        /* Used To Spin The Stars */
        spin+=0.01f;
        /* Changes The Angle Of A Star */
        stars[loop].angle+=(float)loop/NUM;
        /* Changes The Distance Of A Star */
        stars[loop].dist-=0.01f;

        /* Is The Star In The Middle Yet */
        if (stars[loop].dist<0.0f)
        {
            /* Move The Star 5 Units From The Center */
            stars[loop].dist+=5.0f;
            /* Give It A New Red Value */
            stars[loop].r=rand()%256;
            /* Give It A New Green Value */
            stars[loop].g=rand()%256;
            /* Give It A New Blue Value */
            stars[loop].b=rand()%256;
        }
    }

    /* Disable texcoords and vertices arrays */
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);
    glDisableClientState(GL_VERTEX_ARRAY);

    glFinish();

    return 1;
}

void resize(int w, int h)
{
	
	/*change to projection matrix*/
    	glMatrixMode(GL_PROJECTION);
	/*reset the projection matrix*/
    	glLoadIdentity();
	/*set the viewport*/
    	glViewport(0, 0, w, h);

	GLUmat4 perspective;
	/*use glu to set perspective*/
    	gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
	glMultMatrixf(&perspective.col[0].values[0]);
	
	/*get back to model view matrix*/
    	glMatrixMode(GL_MODELVIEW);
	/*reset modevl view matrix*/
    	glLoadIdentity();


}

#ifdef ANDROID
void Java_com_freescale_lesson09_Lesson09Renderer_nativeInit( JNIEnv*  env ){
	init();
}

void Java_com_freescale_lesson09_Lesson09Renderer_nativeRender(JNIEnv*  env ){
	render();
}

void Java_com_freescale_lesson09_Lesson09Renderer_nativeResize(JNIEnv*  env, jobject thiz, jint w, jint h ){
	resize(w, h);
}
#endif

#ifndef ANDROID
void deinit(void)
{
	//call clean up to release any memory allocated
	Cleanup();
	//Make a empty surface current
	eglMakeCurrent(egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	assert(eglGetError() == EGL_SUCCESS);
	//destroy egl desplay
	eglTerminate(egldisplay);
	assert(eglGetError() == EGL_SUCCESS);
	//end egl thread
	eglReleaseThread();
	#ifdef EGL_USE_X11
	//close x display	
	XCloseDisplay(display);
	#endif
}

int main(int argc, char** argv)
{
    assert( init() );

	while (currentFrame < 1000)
	{
		EGLint width = 0;
		EGLint height = 0;
		eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &height);
		render();
		currentFrame++;
		eglSwapBuffers(egldisplay, eglsurface);
	}
	
    deinit();
    return 1;
}
#endif
