/*
 * This code was created by Jeff Molofee '99
 * (ported to Linux by Ti Leggett '01)
 * (ported to i.mx51, i.mx31 and x11 by Freescale '10)
 * If you've found this code useful, please let him know.
 *
 * Visit Jeff at http://nehe.gamedev.net/
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <FSL/fslutil.h>
#include <GLU3/glu3.h>

#ifndef USE_MX31
#include "GLES/gl.h"
#include "GLES/glext.h"
#ifndef ANDROID
#include "EGL/egl.h"
#endif

#else
#include <GLES/gl.h>
#include <GLES/egl.h>
#endif

#ifdef EGL_USE_X11
#include <X11/X.h>
#include <X11/Xlib.h>
#endif

#ifdef ANDROID
#include <jni.h>
#endif

#define MIN_DIVS 1
#define MAX_DIVS 64


#ifndef ANDROID
EGLDisplay			egldisplay;
EGLConfig			eglconfig;
EGLSurface			eglsurface;
EGLContext			eglcontext;
int				currentFrame = 0;
#endif

#ifdef EGL_USE_X11
Display 			*display;
#endif


/*  Structure for a 3-dimensional point (NEW) */
typedef struct point_3d
{
    float x, y, z;
} POINT_3D;

/*  Structure for a 3rd degree bezier patch (NEW) */
typedef struct bpatch
{
    POINT_3D    anchors[4][4];    /*  4x4 grid of anchor points */
    GLfloat     vertices[(MAX_DIVS)*(MAX_DIVS+1)*2][3];
    GLfloat     texcoords[(MAX_DIVS)*(MAX_DIVS+1)*2][2];
    GLuint      texture;          /*  Texture for the patch */
} BEZIER_PATCH;

/*  Rotation about the Z axis */
GLfloat      rotz=0.0f;
/*  The bezier patch we're going to use (NEW) */
BEZIER_PATCH mybezier;
/*  Toggles displaying the control point grid (NEW) */
int     showCPoints=1;
/*  Number of intrapolations (conrols poly resolution) (NEW) */
int          divs=7;

GLuint texture[1];       /* Storage For One Texture */

/*  Adds 2 points. Don't just use '+' ;) */
POINT_3D pointAdd(POINT_3D p, POINT_3D q)
{
    p.x+=q.x;
    p.y+=q.y;
    p.z+=q.z;

    return p;
}

/*  Multiplies a point and a constant. Don't just use '*' */
POINT_3D pointTimes(float c, POINT_3D p)
{
    p.x*=c;
    p.y*=c;
    p.z*=c;

    return p;
}

/*  Function for quick point creation */
POINT_3D makePoint(float a, float b, float c)
{
    POINT_3D p;

    p.x=a;
    p.y=b;
    p.z=c;

    return p;
}

/*  Calculates 3rd degree polynomial based on array of 4 points */
/*  and a single variable (u) which is generally between 0 and 1 */
POINT_3D Bernstein(float u, POINT_3D* p)
{
    POINT_3D a, b, c, d, r;

    a=pointTimes(pow(u,3), p[0]);
    b=pointTimes(3*pow(u,2)*(1-u), p[1]);
    c=pointTimes(3*u*pow((1-u),2), p[2]);
    d=pointTimes(pow((1-u),3), p[3]);

    r=pointAdd(pointAdd(a, b), pointAdd(c, d));

    return r;
}

/*  Generates a arrays based on the data in the patch */
/*  and the number of divisions */
void genBezier(BEZIER_PATCH* patch, int divs)
{
    int u=0;
    int v;
    float py, px, pyold; 
    /*  make the arrays */
    POINT_3D temp[4];
    /*  array of points to mark the first line of polys */
    POINT_3D* last=(POINT_3D*)malloc(sizeof(POINT_3D)*(divs+1));

    /*  the first derived curve (along x axis) */
    temp[0]=patch->anchors[0][3];
    temp[1]=patch->anchors[1][3];
    temp[2]=patch->anchors[2][3];
    temp[3]=patch->anchors[3][3];

    /*  create the first line of points */
    for (v=0; v<=divs; v++)
    {
        /*  percent along y axis */
        px=((float)v)/((float)divs);
        /*  use the 4 points from the derives curve to calculate the points along that curve */
        last[v]=Bernstein(px, temp);
    }

    for (u=1; u<=divs; u++)
    {
        /*  Percent along Y axis */
        py=((float)u)/((float)divs);
        /*  Percent along old Y axis */
        pyold=((float)u-1.0f)/((float)divs);

        /*  Calculate new bezier points */
        temp[0]=Bernstein(py, patch->anchors[0]);
        temp[1]=Bernstein(py, patch->anchors[1]);
        temp[2]=Bernstein(py, patch->anchors[2]);
        temp[3]=Bernstein(py, patch->anchors[3]);

        /*  Begin a new triangle strip */
        for (v=0; v<=divs; v++)
        {
            /*  Percent along the X axis */
            px=((float)v)/((float)divs);

            /*  Apply the old texture coords */
            patch->texcoords[(u-1)*2*(divs+1)+v*2][0]=pyold;
            patch->texcoords[(u-1)*2*(divs+1)+v*2][1]=1.0f-px;
            /*  Old Point */
            patch->vertices[(u-1)*2*(divs+1)+v*2][0]=last[v].x;
            patch->vertices[(u-1)*2*(divs+1)+v*2][1]=last[v].y;
            patch->vertices[(u-1)*2*(divs+1)+v*2][2]=last[v].z;

            /*  Generate new point */
            last[v]=Bernstein(px, temp);
            /*  Apply the new texture coords */
            patch->texcoords[(u-1)*2*(divs+1)+v*2+1][0]=py;
            patch->texcoords[(u-1)*2*(divs+1)+v*2+1][1]=1.0f-px;
            /*  New Point */
            patch->vertices[(u-1)*2*(divs+1)+v*2+1][0]=last[v].x;
            patch->vertices[(u-1)*2*(divs+1)+v*2+1][1]=last[v].y;
            patch->vertices[(u-1)*2*(divs+1)+v*2+1][2]=last[v].z;
        }
    }

    /*  Free the old vertices array */
    free(last);

    /*  Return the array */
    return;
}

/****************************************************************************/
void initBezier(void)
{
    /*  set the bezier vertices */
    mybezier.anchors[0][0]=makePoint(-0.75, -0.75, -0.5);
    mybezier.anchors[0][1]=makePoint(-0.25, -0.75,  0.0);
    mybezier.anchors[0][2]=makePoint( 0.25, -0.75,  0.0);
    mybezier.anchors[0][3]=makePoint( 0.75, -0.75, -0.5);
    mybezier.anchors[1][0]=makePoint(-0.75, -0.25, -0.75);
    mybezier.anchors[1][1]=makePoint(-0.25, -0.25,  0.5);
    mybezier.anchors[1][2]=makePoint( 0.25, -0.25,  0.5);
    mybezier.anchors[1][3]=makePoint( 0.75, -0.25, -0.75);
    mybezier.anchors[2][0]=makePoint(-0.75,  0.25,  0.0);
    mybezier.anchors[2][1]=makePoint(-0.25,  0.25, -0.5);
    mybezier.anchors[2][2]=makePoint( 0.25,  0.25, -0.5);
    mybezier.anchors[2][3]=makePoint( 0.75,  0.25,  0.0);
    mybezier.anchors[3][0]=makePoint(-0.75,  0.75, -0.5);
    mybezier.anchors[3][1]=makePoint(-0.25,  0.75, -1.0);
    mybezier.anchors[3][2]=makePoint( 0.25,  0.75, -1.0);
    mybezier.anchors[3][3]=makePoint( 0.75,  0.75, -0.5);
}

/* function to load in bitmap as a GL texture */
int LoadGLTextures()
{
    Image *image1;

    // allocate space for texture we will use
    image1 = (Image *) malloc(sizeof(Image));
    if (image1 == NULL) 
    {
	printf("Error allocating space for image");
	return 0;
    }	

#ifdef ANDROID
    if (ImageLoad( "/data/app/lesson28_imx.bmp", image1))
#else
    if (ImageLoad( "lesson28_imx.bmp", image1))
#endif   
    {
        /* Create The Textures */
        glGenTextures(1, &texture[0]);

        /* Load in texture 1 */
        /* Typical Texture Generation Using Data From The Bitmap */
        glBindTexture(GL_TEXTURE_2D, texture[0]);

	/* Linear Filtering */
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
        glTexParameterx(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);

        /* Generate The Texture */
        glTexImage2D(GL_TEXTURE_2D,0,GL_RGB,image1->sizeX,image1->sizeY,0,GL_RGB,GL_UNSIGNED_BYTE,image1->data);
        
    }else return 0;

    /* Free up any memory we may have used */
    if (image1 != NULL)
    {
        free(image1);
    }

    return 1;
}

/* function to release/destroy our resources and restoring the old desktop */
void Cleanup()
{

}

/* general OpenGL initialization function */
int init(void)
{
#ifndef ANDROID
    static const EGLint s_configAttribs[] =
    {
	EGL_RED_SIZE,		5,
	EGL_GREEN_SIZE, 	6,
	EGL_BLUE_SIZE,		5,
	EGL_ALPHA_SIZE, 	0,
      	EGL_SAMPLES, 		0,
      	EGL_NONE
    };

    EGLint numconfigs;

#ifdef EGL_USE_X11
    //Open X display
    printf("XOpenDisplay \n");
    display = XOpenDisplay(NULL);
    assert(display != NULL);	

    //Get the default screen from display
    int screen;
    printf("DefaultScreen [");
    screen = DefaultScreen(display);
    printf("%d]\n", screen);

    //get root window
    Window window, rootwindow; 
    printf("RootWindow \n");
    rootwindow = RootWindow(display,screen); 
	
    //get egldisplay from X display
    printf("eglGetDisplay (%d)\n", (int)display);
    egldisplay = eglGetDisplay ( display);
    assert(eglGetError() == EGL_SUCCESS);

    printf("Disp = %d \n",(int) egldisplay);

    //initializa egl display
    printf("eglInitialize \n");
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);

    //tell the driver we are using OpenGL ES
    printf("eglBindAPI \n");
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    printf("eglChooseConfig \n");
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    assert(eglGetError() == EGL_SUCCESS);

    assert(numconfigs == 1);

    //Create and x window
    printf("XCreateSimpleWindow \n");
    window = XCreateSimpleWindow(display, rootwindow, 0, 0, 400, 533, 0, 0, WhitePixel (display, screen));

    XMapWindow(display, window);

    //get an eglsurface from the display using our egl configuration
    printf("*eglCreateWindowSurface \n");
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, window, NULL);
    assert(eglGetError() == EGL_SUCCESS);

#else

    //get egl display
    egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    //Initialize egl
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);
    //tell the driver we are using OpenGL ES
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    printf("chooseconfig, \n");
    assert(eglGetError() == EGL_SUCCESS);
    assert(numconfigs == 1);

    //You must pass in the file system handle to the linux framebuffer when creating a window
#ifndef USE_MX31
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, open("/dev/fb0", O_RDWR), NULL);
#else
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, (NativeWindowType)0, NULL);
#endif
    printf("createwindow, \n");
    assert(eglGetError() == EGL_SUCCESS);

#endif

    //create the egl graphics context
    eglcontext = eglCreateContext(egldisplay, eglconfig, NULL, NULL);
    printf("creatcontext, \n");
    assert(eglGetError() == EGL_SUCCESS);

    //make the context current
    eglMakeCurrent(egldisplay, eglsurface, eglsurface, eglcontext);
    printf("makecurrent, \n");
    assert(eglGetError() == EGL_SUCCESS);
#endif
    /*  Enable Texture Mapping */
    glEnable(GL_TEXTURE_2D);
    /*  Enable Smooth Shading */
    glShadeModel(GL_SMOOTH);
    /*  Black Background */
    glClearColor(0.25f, 0.25f, 0.25f, 0.5f);
    /*  Depth Buffer Setup */
    glClearDepthf(1.0f);
    /*  Enables Depth Testing */
    glEnable(GL_DEPTH_TEST);
    /*  The Type Of Depth Testing To Do */
    glDepthFunc(GL_LEQUAL);
    /*enable cullface*/
    glEnable( GL_CULL_FACE );
    glCullFace( GL_BACK );
    /*  Really Nice Perspective Calculations */
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

    /*  Initialize the Bezier's control grid */
    initBezier();

    /* Load in the texture */
    if (!LoadGLTextures())
    {
        return 0;
    }

    /*  Generate the patch */
    genBezier(&mybezier, divs);
#ifndef ANDROID
    /*set perspective*/
    EGLint h, w;
    /*get width and height from egl*/
    eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &w);
    eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &h);

    /*change to projection matrix*/
    glMatrixMode(GL_PROJECTION);
    /*reset the projection matrix*/
    glLoadIdentity();
    /*set the viewport*/
    glViewport(0, 0, w, h);

    GLUmat4 perspective;
    /*use glu to set perspective*/
    gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
    glMultMatrixf(&perspective.col[0].values[0]);

    /*get back to model view matrix*/
    glMatrixMode(GL_MODELVIEW);
    /*reset modevl view matrix*/
    glLoadIdentity();
#endif
    return 1;
}

/* Here goes our drawing code */
int render()
{
 
    int i, j;
    GLfloat baseverts[4][3];

    /*  Clear Screen And Depth Buffer */
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    /*  Reset The Current Modelview Matrix */
    glLoadIdentity();
    /*  Move Left 1.5 Units And Into The Screen 6.0 */
    glTranslatef(0.0f, 0.0f, -4.0f);
    glRotatef(-75.0f, 1.0f, 0.0f, 0.0f);
    /*  Rotate The Triangle On The Z axis ( NEW ) */
    glRotatef(rotz, 0.0f, 0.0f, 1.0f);

    /*  Bind the texture */
    glBindTexture(GL_TEXTURE_2D, texture[0]);

    /*  Draw the Bezier's array */
    /*  this need only be updated when the patch changes */

    /* Set pointers to vertices and texcoords */
    glVertexPointer(3, GL_FLOAT, 0, &mybezier.vertices[0][0]);
    glTexCoordPointer(2, GL_FLOAT, 0, &mybezier.texcoords);

    /* Enable vertices and texcoords arrays */
    glEnableClientState(GL_VERTEX_ARRAY);
    glEnableClientState(GL_TEXTURE_COORD_ARRAY);

    /* Draw bezier patch */
    for (i=0; i<divs; i++)
    {
        glDrawArrays(GL_TRIANGLE_STRIP, i*(divs+1)*2, (divs+1)*2);
    }

    /* Disable vertices and texcoords arrays */
    glDisableClientState(GL_VERTEX_ARRAY);
    glDisableClientState(GL_TEXTURE_COORD_ARRAY);

    /* Set pointers to vertices and texcoords */
    glVertexPointer(3, GL_FLOAT, 0, baseverts);

    /* Enable vertices array */
    glEnableClientState(GL_VERTEX_ARRAY);

    /*  If drawing the grid is toggled on */
    if (showCPoints)
    {
        glDisable(GL_TEXTURE_2D);

        /* Set color to Red for control points */
        glColor4f(1.0f, 0.0f, 0.0f, 1.0f);

        /*  draw the horizontal lines */
        for(i=0; i<4; i++)
        {
            for(j=0; j<4; j++)
            {
                baseverts[j][0]=mybezier.anchors[i][j].x;
                baseverts[j][1]=mybezier.anchors[i][j].y;
                baseverts[j][2]=mybezier.anchors[i][j].z;
            }
            glDrawArrays(GL_LINE_STRIP, 0, 4);
        }

        /*  draw the vertical lines */
        for(i=0; i<4; i++)
        {
            for(j=0; j<4; j++)
            {
                baseverts[j][0]=mybezier.anchors[j][i].x;
                baseverts[j][1]=mybezier.anchors[j][i].y;
                baseverts[j][2]=mybezier.anchors[j][i].z;
            }
            glDrawArrays(GL_LINE_STRIP, 0, 4);
        }

        /* Restore default color to avoid texture color modulation */
        glColor4f(1.0f, 1.0f, 1.0f, 1.0f);
        glEnable(GL_TEXTURE_2D);
    }

    /* Disable vertices array */
    glDisableClientState(GL_VERTEX_ARRAY);

    glFinish();

    return 1;
}

void resize(int w, int h)
{
	
	/*change to projection matrix*/
    	glMatrixMode(GL_PROJECTION);
	/*reset the projection matrix*/
    	glLoadIdentity();
	/*set the viewport*/
    	glViewport(0, 0, w, h);

	GLUmat4 perspective;
	/*use glu to set perspective*/
    	gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
	glMultMatrixf(&perspective.col[0].values[0]);
	
	/*get back to model view matrix*/
    	glMatrixMode(GL_MODELVIEW);
	/*reset modevl view matrix*/
    	glLoadIdentity();


}

#ifdef ANDROID
void Java_com_freescale_lesson28_Lesson28Renderer_nativeInit( JNIEnv*  env ){
	init();
}

void Java_com_freescale_lesson28_Lesson28Renderer_nativeRender(JNIEnv*  env ){
	render();
}

void Java_com_freescale_lesson28_Lesson28Renderer_nativeResize(JNIEnv*  env, jobject thiz, jint w, jint h ){
	resize(w, h);
}
#endif

#ifndef ANDROID
void deinit(void)
{
	//call clean up to release any memory allocated
	Cleanup();
	//Make a empty surface current
	eglMakeCurrent(egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	assert(eglGetError() == EGL_SUCCESS);
	//destroy egl desplay
	eglTerminate(egldisplay);
	assert(eglGetError() == EGL_SUCCESS);
	//end egl thread
	eglReleaseThread();
	#ifdef EGL_USE_X11
	//close x display	
	XCloseDisplay(display);
	#endif
}

int main(int argc, char** argv)
{
    assert( init() );

	while (currentFrame < 1000)
	{
		EGLint width = 0;
		EGLint height = 0;
		eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &height);
		render();
		currentFrame++;
		eglSwapBuffers(egldisplay, eglsurface);
	}
	
    deinit();
    return 1;
}
#endif
