
/* File: fslcard.h
 *
 * Copyright � 2009 Freescale Semiconductor
 */

#ifndef _FSLCARD_H_
#define _FSLCARD_H_

#include <stdio.h>
#include <EGL/egl.h>
#include <GLES2/gl2.h>
#include <FSL/fslutil.h>

//These are all pretty arbitrary and 'off the cuff'
#define MAX_PRE_LENGTH 750
#define MAX_POST_LENGTH 750
#define MAX_INIT_CONFIGS 10
#define MAX_PHASE_ARRAY_SIZE 20

enum fslPhaseType 
{
	FSL_PHASE_UNSPECIFIED		 	= 0,
	FSL_PHASE_PRERENDER_VG		 	= 1000,  //implcitly read framebuffer only;  write is to offscreen buffers only
	FSL_PHASE_PRERENDER			 	= 2000,  //should not affect FB
	FSL_PHASE_RENDER_WRITEONLY 		= 3000,  //For items without alpha
	FSL_PHASE_RENDER_READWRITE 		= 4000,  //For items w/ alpha
	FSL_PHASE_POSTRENDER		 	= 5000,  //Render Cleanup
	FSL_PHASETYPE_END	 			= 0xFFFFFFFF,
};

typedef struct fslPhaseElementType_s
{
	unsigned int 	phaseID;
	fslPhaseType 	type;
} fslPhaseElementType;

enum fslInitStageType 
{
	FSL_INIT_UNSPECIFIED		 	= 0,
	FSL_INIT_COMPILE			 	= 1000,
	FSL_INIT_LINK				 	= 2000,
	FSL_INIT_END	 				= 0xFFFFFFFF,
};

class fslCard
{
protected:
	void* parentScene_;
	unsigned long long cardID_;
	float currentTime_;
	float tickTime_;
	char pixelPreString_[MAX_PRE_LENGTH], 
		pixelPostString_[MAX_POST_LENGTH], 
		vertexPreString_[MAX_PRE_LENGTH], 
		vertexPostString_[MAX_POST_LENGTH];

	//Scene may need multiple shader versions (like for reflection).. need to make array
	GLuint shaderProgram_[MAX_INIT_CONFIGS];
	unsigned int numShaderAttributes_;
	unsigned int baseEventID_;
	EGLDisplay dpy_;
	
public:

	void setEGLDisplay(EGLDisplay dpy)
	{
		dpy_ = dpy;
	}

	EGLDisplay getEGLDisplay()
	{
		return dpy_;
	}

	void setParentScene( void* parent )
	{
		parentScene_ = parent;
	}
	
	//Scenes call this for esablishing a 'namespace' of event identifies coming from a card
	//Cards use this as minimum ID value for calling receiveCardEvent in the event in needs to 
	//  signal its parent scene (i.e. if a user hits a button that will affect another card)
	void setBaseEventID( unsigned int baseEventID )  
	{
		baseEventID_ = baseEventID;
	}

	unsigned int getBaseEventID( )
	{
		return baseEventID_;
	}
	
	// Should this be in the base or derived classes?
	fslStatus setVertexShaderAppendStrings( char* prestring, char* poststring)
	{ 
		//Probably needs some error checking
		sprintf(vertexPreString_, "%s", prestring);
		sprintf(vertexPostString_, "%s", poststring);
		return FSL_STATUS_SUCCESS;
	}
	
	fslStatus setPixelShaderAppendStrings( char* prestring, char* poststring)
	{
		//Probably needs some error checking
		sprintf(pixelPreString_, "%s", prestring);
		sprintf(pixelPostString_, "%s", poststring);
		return FSL_STATUS_SUCCESS;
	}
	
	unsigned long long getCardID()
	{
		return cardID_;
	}
	
	GLuint getShaderProgram(unsigned int index)
	{
		return shaderProgram_[index];
	}
	
	fslStatus setCardID(unsigned long long id)
	{
		//Probably needs error checking
		cardID_ = id;
		return FSL_STATUS_SUCCESS;
	}

	unsigned int getNumShaderAttributes( )
	{
		return numShaderAttributes_;
	}

	//Virtual functions
	virtual fslStatus initCard( fslInitStageType init, unsigned int config)=0; //initilizes GL state vars, loads textures, compiles Textures, etc.
	virtual fslStatus shutdownCard()=0; //Releases dynamic resources without destroying card (renamed to avoid system conflict)
	virtual fslStatus resumeCard()=0;   //Reallocates resources for card; probably the same as initCard for most cards (renamed due to system conflict)
	virtual fslStatus animTick(void)=0; //adjusts state by the time tick set in resolution
	virtual fslStatus setTickRes(float time) {tickTime_ = time; return FSL_STATUS_SUCCESS;}
	virtual fslStatus setTime(float time) { currentTime_ = time; return FSL_STATUS_SUCCESS;}
	virtual float getTime() {return currentTime_;}
	virtual fslStatus renderCard(unsigned int phaseID, unsigned int config)=0;  //Broken into phases so master render function doesn't get things out of order (alpha, etc.)
	virtual fslStatus preRenderCard(unsigned int phaseID, unsigned int config){return FSL_STATUS_SUCCESS;} //for setting up prestate like glUseProgram
	virtual void getRenderPhases(fslPhaseElementType* phaseArray) //assumed fixed size of MAX_PHASE_ARRAY_SIZE
		{
			if(phaseArray == NULL)
				return;
			
			phaseArray[0].type = FSL_PHASE_UNSPECIFIED;
			phaseArray[0].phaseID = 0;
		} 
	virtual float getAspectRatio()  //The card itslef just renders to Z axis aligned points, need the aspect ratio for world transform / projection matrix
		{return (4.0f/3.0f);}
	virtual fslStatus handleInputEvent( fslInputEventType event)=0;  //These events are ONLY card relevent;  All coords are in 0-1, 0-1, card space (lower left origin)
	//Reflection functs removed as they are redundant with new shader functs

};

#endif //_FSLCARD_H_
