
/* File: fslscene.h
 *
 * Copyright � 2009 Freescale Semiconductor
 */
 
#ifndef _FSLSCENE_H_
#define _FSLSCENE_H_

#include <CARD/fslcard.h>
#include <FSL/fslutil.h>

#define MAX_CARDS_IN_SCENE 20
#define MAX_SCENES_IN_SCENE 20

enum fslTransitionType
{
	FSL_TRANSITION_UNSPECIFIED		=0001,
	FSL_TRANSITION_ALPHA_FADE		=1000,
	FSL_TRANSITION_FLY_STAGE_LEFT	=2000,
	FSL_TRANSITION_FLY_STAGE_RIGHT	=3000,
	FSL_TRANSITION_SCATTER			=4000,
	FSL_TRANSITION_ZOOM_IN			=5000,
	FSL_TRANSITION_ZOOM_OUT			=6000,
	FSL_TRANSITIONTYPE_END			=0xFFFFFFFF,
};
		
enum fslOrientationType
{
	FSL_LANDSCAPE					=1000,  //Should this be changed to 0,90,180,270 rotation instead?
	FSL_PORTRAIT					=2000,
	FSL_ORIENTATIONTYPE_END			=0xFFFFFFFF,
};


class fslScene
{
protected:
	fslCard* cardList_[MAX_CARDS_IN_SCENE];  //Scenes can have cards and other scenes within them
	unsigned int numCards_;  //Number of cards attached to scene.  may be better to have linked list
	fslScene* sceneList_[MAX_SCENES_IN_SCENE];
	unsigned int numScenes_;
	fslScene* parentScene_;  //Points back to the parent; lets you embedded scenes in a master scene
	unsigned long long newCardID_;
	float aspectRatio_;
	float PM_[16]; //projection matrix
	fslOrientationType orientation_;
        EGLDisplay dpy_;

	unsigned int phases_[MAX_PHASE_ARRAY_SIZE];

	fslStatus addCard_(fslCard* newCard)
	{
		cardList_[numCards_] = newCard;  //Should probably be made into linked list
		numCards_ ++;
		return FSL_STATUS_SUCCESS;
	}
	
	fslStatus addScene_(fslScene* newScene)
	{
		sceneList_[numScenes_] = newScene;  //Should probably be made into linked list
		numScenes_ ++;
		return FSL_STATUS_SUCCESS;		
	}
	
public:
	
	fslScene() 
	{
		parentScene_ = NULL;
	}  
	
	void setEGLDisplay(EGLDisplay dpy)
	{
		dpy_ = dpy;
	}

	EGLDisplay getEGLDisplay(EGLDisplay * dpy)
	{
		return dpy_;
	}

	unsigned long long getCardUniqueID() //Used to create new cards
	{
		if(parentScene_ == NULL)
			return newCardID_++;
		else
			return parentScene_->getCardUniqueID();
	}
	
	//Virtual Functions
	virtual fslStatus setAspectRatio( float aspectRatio ) // this is virtual because changing the Aspect Ratio may break pre-rendered content
	{
		aspectRatio_ = aspectRatio;
		return FSL_STATUS_SUCCESS;
	}
	
	virtual fslStatus setOrientation( fslOrientationType orientation ) // this is virtual because changing the Aspect Ratio may break pre-rendered content		
	{
		orientation_ = orientation;
		return FSL_STATUS_SUCCESS;
	}

	virtual fslStatus handleInputEvent(fslInputEventType event)
	{
			return FSL_STATUS_SUCCESS;
	}
	
	virtual fslStatus transitionOnStage( fslTransitionType transistion ) {return FSL_STATUS_SUCCESS;}
	virtual fslStatus transitionOffStage( fslTransitionType transistion ) {return FSL_STATUS_SUCCESS;}
	virtual fslBool isOnStage() {return FSL_FALSE;}  //Test for scene visibilty
	virtual fslStatus animResolve( float deltaTime ) {return FSL_STATUS_SUCCESS;}  //Pass in the time since the last animResolve call
	
	virtual fslStatus renderScene( fslPhaseType onlyThese ) {return FSL_STATUS_SUCCESS;}  //UNSPECIFIED for all
	virtual fslStatus initScene() {return FSL_STATUS_SUCCESS;}  //GL state inititization
	virtual void receiveCardEvent( unsigned int eventID, void* data) { }  //cards call this function of parent scene; eventID base is set by scene by call card
};

#endif //_FSLSCENE_H_
