
/* File: fslvideobrowserscene.h
 *
 * Copyright � 2009 Freescale Semiconductor
 */
 
#ifndef _FSLVIDEOBROWSERSCENE_H_
#define _FSLVIDEOBROWSERSCENE_H_
 
#include <CARD/fslcard.h>
#include <CARD/fslscene.h>
#include <CARD/fslvideorendercard.h>
#include <CARD/fslvideochaptercard.h>
#include <CARD/fslplaybackcontrolcard.h>
#include <CARD/fslfilebrowsercard.h>
#include <FSL/fslutil.h>
 
#define FSLVIDEOBROWSERSCENE_NUM_SHADER_CONFIGS 10
 
 
enum fslVideoBrowserStateType 
{
	FSL_STATE_UNKNOWN			 	= 0,
	FSL_STATE_VIDEO_PLAYING			= 2000,  
	FSL_STATE_VIDEO_CHAPTERS 		= 3000,  
	FSL_STATE_VIDEO_SELECT	 		= 4000,  
	FSL_VIDEOBROWSERSTATETYPE_END	= 0xFFFFFFFF,
};
 
class fslVideoBrowserScene: public fslScene
{
protected:
	fslVideoBrowserStateType currentState_;
	fslVideoBrowserStateType nextState_;
	fslBool isChangingState_;
	fslBool isTransitionOffStage_;
	fslBool isTransitionOnStage_;
	
	fslBool pointerDownVideo_;
	long pointerDownVideoTime_;
	unsigned int pointerDownVideoX_;
	unsigned int pointerDownVideoY_;
	
	float videoRenderPosition_;
	float videoChapterPosition_;
	float playbackControlPosition_;
	float fileSelectionPosition_;
	float stageTransitionPosition_;
	
	fslVideoRenderCard myVideoCard_;
	fslVideoChapterCard myChapterCard_;
	fslPlaybackControlCard myPlaybackCard_;
	fslFileBrowserCard myFileBrowserCard_;
	
	fslBool isVisible_;
	GLuint projMatrixLoc_[FSLVIDEOBROWSERSCENE_NUM_SHADER_CONFIGS];
	GLuint modelMatrixLoc_[FSLVIDEOBROWSERSCENE_NUM_SHADER_CONFIGS];
	GLuint shaderProgram_[FSLVIDEOBROWSERSCENE_NUM_SHADER_CONFIGS];
	
	float projMatrix_[16];
	float videoInvProjMatrix_[16];
	float chapterInvProjMatrix_[16];
	fslBool projMatrixDirty_;
	
	float modelMatrixVideo_[16];
	fslBool modelMatrixVideoDirty_;
	float modelMatrixControl_[16];
	fslBool modelMatrixControlDirty_;
	float modelMatrixChapter_[16];
	fslBool modelMatrixChapterDirty_;
	float modelMatrixSelect_[16];
	fslBool modelMatrixSelectDirty_;	
	
	//float currentInvProjMatrix3_[16];
	
	bool isRunning_;	
		
	void changeState(fslVideoBrowserStateType newState);
	void changeStateOnPositionCheck(void);
	void updateModelMatrices(void);
	
public:
	fslVideoBrowserScene( fslScene* parentscene );
	~fslVideoBrowserScene();

	fslStatus setAspectRatio( float aspectRatio ); 
	fslStatus setOrientation( fslOrientationType orientation ); 	
	fslStatus transitionOnStage( fslTransitionType transition);
	fslStatus transitionOffStage( fslTransitionType transition);
	fslBool isOnStage();
	fslStatus animResolve( float deltaTime );	
	fslStatus renderScene( fslPhaseType onlyThese);  
	fslStatus handleInputEvent( fslInputEventType event);
	fslStatus initScene(); 
	void receiveCardEvent( unsigned int eventID, void* data);
	bool isRunning(){return isRunning_;}

	void changeStateWithTransition(fslVideoBrowserStateType newState);
	
};

#endif //_FSLVIDEOBROWSERSCENE_H_

