
#include <GLES2/gl2.h>
#include <EGL/egl.h>
#include <CARD/fslexamplecard.h>
#include <CARD/plane.h>
#include <FSL/fslutil.h>

#include <stdio.h>

Plane *bgPlane = 0;
float g_animAngle = 0;
int g_direction = 1;
//shader sources, we need to implement a parser to load this from a text file
static const char* g_strEnd =
    "}										\n";

static const char* g_strVSProgramA = 
    "uniform   mat4 g_matModelView;						\n"
    "										\n"
    "attribute vec4 g_vVertex;							\n"
    "attribute vec4 g_vTexcoord;						\n"
    "varying   vec4 g_vVSTexcoord;						\n"
    "										\n";

static const char* g_strVSProgramB = 
    "void main()								\n"
    "{										\n"	
    "    vec4 cardPosition = g_matModelView * g_vVertex;   			\n"
    "    g_vVSTexcoord = g_vTexcoord;						\n";

static const char* g_strFSProgramA = 
    "#ifdef GL_FRAGMENT_PRECISION_HIGH						\n"
    "   precision highp float;							\n"
    "#else									\n"
    "   precision mediump float;						\n"
    "#endif									\n"
    "										\n"
    "uniform   sampler2D g_sImageTexture;					\n"
    "varying   vec4      g_vVSTexcoord;						\n"
    "										\n";

static const char* g_strFSProgramB = 
    "void main()								\n"
    "{										\n"
    "    vec4 cardColor;							\n"
    "    cardColor.r = 0.0;							\n"
    "    cardColor.g = 0.5;							\n"
    "    cardColor.b = 0.3;							\n"
    "    cardColor.a = 1.0;							\n"
    "    cardColor = texture2D(g_sImageTexture, g_vVSTexcoord.xy);		\n";


fslExampleCard::fslExampleCard( )
{
	parentScene_ = NULL;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	fslLoadIdentityMatrix4x4(MV_);
	return;
}

fslExampleCard::fslExampleCard( void* parentScene )
{
	parentScene_ = parentScene;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	fslLoadIdentityMatrix4x4(MV_);
	return;
}

fslExampleCard::~fslExampleCard()
{
	return;
}

//This should probably be made to work with 'n' shaders
fslStatus fslExampleCard::initCard( fslInitStageType init, unsigned int config) 
{
	static GLuint hVertexShader;
	static GLuint hFragmentShader;
	bgPlane = new Plane();
	bgPlane->createPlane(0.0,0.0,1.0,1.0);
	//load identity in modelView matrix
	fslLoadIdentityMatrix4x4(MV_);
	fslLoadIdentityMatrix4x4(MV_);
	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
			{
				GLint nCompileResult;
				char vs[MAX_SHADER_LENGTH];
				char ps[MAX_SHADER_LENGTH];
				const char* pvs = (char*)(vs); // This is kind of silly and I'm not sure why this is being required in C++ and not C
				const char* pps = (char*)(ps); //  may also be solved with &(const_cast<char*>(vs))

				//vs = new char[MAX_SHADER_LENGTH];
				//ps = new char[MAX_SHADER_LENGTH];

				printf("Starting Initialization \n");

				//This would probably be better handled with string objects
				sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
				sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				//Could probably add more code to make this less intensive on subsequent inits 
	      printf("About to load texture! \n");
		
				if(!fslInit2DCTESTextureGL( "media/textures/Background.CTES", &backgroundTextureHandle_[config] ))
					return FSL_STATUS_GENERAL_ERROR;

				if(!fslInit2DCTESTextureGL( "media/textures/frame256x256.CTES", &backgroundTextureHandle_[1] ))
					return FSL_STATUS_GENERAL_ERROR;
		
				printf("Vertex Shader = \n %s \n", vs);
				printf("Pixel Shader = \n %s \n", ps);

					// Compile the shaders
				printf("Compiling Shaders... \n");
				hVertexShader = glCreateShader( GL_VERTEX_SHADER );
				glShaderSource( hVertexShader, 1, &pvs, NULL );
				glCompileShader( hVertexShader );

				hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
				glShaderSource( hFragmentShader, 1, &pps, NULL );
				glCompileShader( hFragmentShader );

				// Check for compile success
				nCompileResult = 0;
				glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					printf("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					return FSL_STATUS_GENERAL_ERROR;
				}

				// Attach the individual shaders to the common shader program
				shaderProgram_[config] = glCreateProgram();
				glAttachShader( shaderProgram_[config], hVertexShader );
				glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
				glBindAttribLocation(shaderProgram_[config], vertexLoc_,   "g_vVertex");
				glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "g_vTexcoord");
				
				numShaderAttributes_ = 2; //The scene can use attributes higher than this

			}
 		
			break;
		case FSL_INIT_LINK:
			{
				GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
				printf("Linking Shaders... \n");
				glLinkProgram( shaderProgram_[config] );

				// Check for link success
				nLinkResult = 0;
				glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);
				if (!nLinkResult)
				{
					char Log[1024];
					GLint nLength;
					printf("Error linking shader! \n");
					glGetProgramInfoLog(shaderProgram_[config], 1024, &nLength, Log);
					return FSL_STATUS_GENERAL_ERROR;
				}

				//projMatrixLoc = glGetUniformLocation( g_hshaderProgram_, "g_matProj" );
				modelMatrixLoc_[config] = glGetUniformLocation( shaderProgram_[config], "g_matModelView" );

				glDeleteShader( hVertexShader );
				glDeleteShader( hFragmentShader );
				//printf("Finished Initialization... \n");	
			}			
			break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::shutdownCard() //Releases dynamic resources without destroying card
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::resumeCard()   //Reallocates resources for card; probably the same as initCard for most cards
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::animTick(void) //adjusts state by the time tick set in resolution
{
	switch (g_direction)
	{
		case 1:
			if(g_animAngle >1.0)
				g_direction=g_direction*-1;
			else
				g_animAngle += 0.01;  //Something is wrong with the time function, this is temporary
		break;
		case -1:
			if(g_animAngle <-1.0)
				g_direction=g_direction*-1;
			else
				g_animAngle+=-0.01;
		break;		
	}
		//Rotation
		//fslRotateMatrix4x4(MV_,g_animAngle,FSL_Y_AXIS);
		
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::setTickRes(float time)
{
	//This card currently has no animation
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::setTime(float time)
{
	//This card currently has no animation
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

float fslExampleCard::getTime()
{
	return currentTime_;
}

fslStatus fslExampleCard::renderCard( unsigned int phaseID, unsigned int config )  
{
	//Broken into phases so master render function doesn't get things out of order (alpha, etc.)
	//PhaseID are set by the card in the getRenderPhases
	//Scenes call getRenderPhases to get stages needed by card
	//Scenes then call renderCard with the IDs they got from getRenderPhases grouped with other
	//  cards by the type
	switch(phaseID)
	{
		case 1:
			{

			glUseProgram( shaderProgram_[config] );
			glUniformMatrix4fv( modelMatrixLoc_[config],1, 0, MV_ );
			
			
		  drawPlane(bgPlane->vertex,bgPlane->vertexTexCoords,backgroundTextureHandle_[0]);
			}

			break;

		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}


void fslExampleCard::getRenderPhases(fslPhaseElementType* phaseArray) 
{
	if(phaseArray == NULL)
		return;
		
	//phaseArray is assumed fixed size of MAX_PHASE_ARRAY_SIZE
	//This card only has one stage
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_WRITEONLY;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[1].phaseID = 0;

	return;
}

float fslExampleCard::getAspectRatio()
{
	//This is only a hard-coded suggested value as the scene can really use whatever it wants
	return 4.0f/3.0f; 
}

//These events are ONLY card relevent;  All coords are in 0-1, 0-1, card space (lower left origin)
fslStatus fslExampleCard::handleInputEvent( fslInputEventType event)
{
	if(event.input == FSL_INPUT_POINTERDOWN) 
	{
		printf( "Card Recieved Pointer Click @ [%u,%u]\n", event.signalA, event.signalB);
	}
	
	printf("input!!!");
	return FSL_STATUS_SUCCESS;
}


fslStatus fslExampleCard::drawPlane( float VertexPos[] , float TexCoords[], GLuint texture)
{
			
				
				glBindTexture( GL_TEXTURE_2D, texture );

				glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPos );
				glEnableVertexAttribArray( vertexLoc_ );

				glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, TexCoords);
				glEnableVertexAttribArray( texcoordLoc_ );

				//printf("Drawing...\n");
				glDrawArrays( GL_TRIANGLES, 0, 6 );

				glDisableVertexAttribArray( vertexLoc_ );
				glDisableVertexAttribArray( texcoordLoc_ );
	return FSL_STATUS_SUCCESS;
}











