//
// C++ Implementation: GUIScene
//
// Description: 
//
//
// Author: FCSZ <root@ubuntu-desktop>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <CARD/fslscene.h>
#include <CARD/fslGUIScene.h>

static char* g_strVSPre =  "	uniform   mat4 g_matProj;				\n";
static char* g_strVSPost = "    gl_Position  = g_matProj * cardPosition;		\n";
static char* g_strPSPre =  "								\n";
static char* g_strPSPost = "    gl_FragColor = cardColor;				\n";

fslGUIScene::fslGUIScene( fslScene* parentScene )
{
	parentScene_ = parentScene;

	for(int i=0; i < MAX_CARDS_IN_SCENE; i++)
		cardList_[i] = NULL;

	for(int i=0; i < MAX_SCENES_IN_SCENE; i++)
		sceneList_[i] = NULL;

	numCards_ = 2;
	numScenes_ = 0;

	newCardID_ = 0;
	orientation_ = FSL_LANDSCAPE;
	isVisible_ = FSL_FALSE;

	myMusicTileCard_[0].setParentScene(this);
	myMusicTileCard_[1].setParentScene(this);
	//base value to keep track of events from card
	// i.e if between 1000 - 1999, then it is from this card
	//Only really valuable when tracking multiple cards
	myMusicTileCard_[0].setBaseEventID( 1000 );
	myMusicTileCard_[1].setBaseEventID( 2000 );
	isRunning_ = true;
	return;
}

fslGUIScene::~fslGUIScene()
{
	return;
}
	
fslStatus fslGUIScene::setAspectRatio( float aspect )
{
	//There is no big impact to this card for changing the aspect ratio
	aspectRatio_ = aspect;
	return FSL_STATUS_SUCCESS;
}
	
fslStatus fslGUIScene::setOrientation( fslOrientationType orientation )
{
	//There is no big impact to this card for changing the orientation
	orientation_ = orientation;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslGUIScene::transitionOnStage( fslTransitionType transition)
{
	//Nothing fancy for the example
	//This should probably trigger an animation state change in a real scene
	isVisible_ = FSL_TRUE;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslGUIScene::transitionOffStage( fslTransitionType transition)
{
	isVisible_ = FSL_FALSE;
	return FSL_STATUS_SUCCESS;
}


fslStatus fslGUIScene::renderScene( fslPhaseType onlyThese)
{
		
	if(isVisible_ != FSL_TRUE)
		return FSL_STATUS_SUCCESS;

	if(onlyThese == FSL_PHASE_RENDER_WRITEONLY || onlyThese == FSL_PHASE_UNSPECIFIED)
	{
		//memcpy(currentProjMatrix_, PM_, (sizeof(float) * 16));
		fslInvertMatrix4x4( PM_, currentInvProjMatrix_);

		//Master Scene (or application) handles the clears
		//Scene handles other state setup such as:
		//glDisable(GL_DEPTH_TEST); //because we'll start using it :)
	
		//set attribs and uniforms
		//myMusicTileCard_[0].preRenderCard(phases_[0], 0);
		
		glUniformMatrix4fv( projMatrixLoc_[0], 1, 0, PM_ );//pass matrix to shader uniform
		glUniformMatrix4fv( projMatrixLoc_[1], 1, 0, PM_ );//pass matrix to shader uniform
		
		myMusicTileCard_[0].renderCard(phases_[0], 0);
		myMusicTileCard_[0].renderCard(phases_[1], 0);

		myMusicTileCard_[1].renderCard(phases_[0], 0);
		myMusicTileCard_[1].renderCard(phases_[1], 0);

		
		return FSL_STATUS_SUCCESS;
		
	}
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslGUIScene::handleInputEvent( fslInputEventType event)
{


	float vecA[4] = {0};
	float vecC[4] = {0};
	float vecA3[4] = {0};
	float vecC3[4] = {0};
	
	//This switch is for catching other event types
	/*
	printf("Scene Event:");
	switch(event.input)
	{
		case FSL_INPUT_KEYPRESS: printf("Key Press"); break;
		case FSL_INPUT_KEYRELEASE: printf("Key Release"); break;
		case FSL_INPUT_POINTERDOWN: printf("Pointer Down"); break;
		case FSL_INPUT_POINTERUP: printf("Pointer Up"); break;
		case FSL_INPUT_POINTERMOVE: printf("Pointer Move"); break;
		default: printf("UNHANDLED\n");
	}
	*/
	
	//Convert click coords to -1 to 1 X,Y 3D coords
	vecA[0] = vecA3[0] = ((float) event.signalA * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[1] = vecA3[1] = ((float) event.signalB * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	
	vecA3[1] -= 0.8f;

	//convert from screen space to card space
	fslMultMatrix4x4Vec4x1 ( currentInvProjMatrix_, vecA, vecC );

	
	//Debug Info
	//printf(" [%u,%u] \n",event.signalA,event.signalB);
//	printf(" pretransformed vec = [%f,%f,%f,%f] \n",vecA3[0],vecA3[1],vecA3[2],vecA3[3]);		
//	printf(" invtransformed vec = [%f,%f,%f,%f] \n",vecC3[0],vecC3[1],vecC3[2],vecC3[3]);	
	
	//Make sure the click falls in the bounds of the card
	if( (event.input == FSL_INPUT_POINTERDOWN) 
		&& (vecC[0] <= 1.0) && (vecC[0] >= -1.0)
		&& (vecC[1] <= 1.0) && (vecC[1] >= -1.0))
	{
		//Convert from -1 to 1 float to 0 to 1 uint
		event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
		event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX);  
		
		//printf( "Passing [%u,%u] to Card\n", event.signalA, event.signalB);
		
// 		myExampleCard_.handleInputEvent( event );
	}

	//Make sure the click falls in the bounds of the card
	if( ((event.input == FSL_INPUT_POINTERDOWN) || 
		(event.input == FSL_INPUT_POINTERUP)) 
		&& (vecC3[0] <= 1.0) && (vecC3[0] >= -1.0)
		&& (vecC3[1] <= 1.0) && (vecC3[1] >= -1.0))
	{
		//Convert from -1 to 1 float to 0 to 1 uint
		event.signalA = (unsigned int)(((vecC3[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
		event.signalB = (unsigned int)(((vecC3[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX);  


	}	

	if (event.input == FSL_INPUT_CLOSEWINDOW)
	{
		isRunning_ = false;
	}

	return FSL_STATUS_SUCCESS;
}

fslStatus fslGUIScene::initScene()  //GL state inititization
{
	fslPhaseElementType phaseArray[MAX_PHASE_ARRAY_SIZE];

	myMusicTileCard_[0].getRenderPhases( phaseArray );

 	//Set projection matrix to Identity Matrix
	fslLoadIdentityMatrix4x4(PM_);

	//set perspective projection matrix
	//should be aspectRatio_, not sure if that works, so hard coded in the meantime
	fslPerspectiveMatrix4x4(PM_, 50.0f, 1.3333f, 0.001f, 100.0f);

	//I know there is only one phase for this card
	//More dynamic scenes would need to parse the whole array
	phases_[0] = phaseArray[0].phaseID;
	phases_[1] = phaseArray[1].phaseID;

	//Set our shader contibution

	myMusicTileCard_[0].setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myMusicTileCard_[0].setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	myMusicTileCard_[1].setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myMusicTileCard_[1].setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );


	printf("about to compile musicTileCard shader \n");
	if( myMusicTileCard_[0].initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	printf("about to compile musicTileCard shader \n");
	if( myMusicTileCard_[1].initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	shaderProgram_[1] = myMusicTileCard_[1].getShaderProgram(0);
	shaderProgram_[0] = myMusicTileCard_[0].getShaderProgram(0);
	//Set any attribute handles needed
	// There are none for this scene
	

	if( myMusicTileCard_[0].initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	if( myMusicTileCard_[1].initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	//Get uniform handles
	projMatrixLoc_[0] = glGetUniformLocation( shaderProgram_[0], "g_matProj" );
	projMatrixLoc_[1] = glGetUniformLocation( shaderProgram_[1], "g_matProj" );
	
	//Rinse, repeat for extra stages, reflections, etc.  Make sure and increase the config parameter for init
	
	return FSL_STATUS_SUCCESS;
}

fslBool fslGUIScene::isOnStage()
{
	return isVisible_;
}

fslStatus fslGUIScene::animResolve( float deltaTime )
{
	//he scene calls the animTick functions of the cards

	myMusicTileCard_[0].animTick();
	myMusicTileCard_[1].animTick();

	fslTranslateMatrix4x4(myMusicTileCard_[0].getMVMatrix(),-1.f,0,-3);
	fslScaleMatrix4x4(myMusicTileCard_[1].getMVMatrix(), 0.5f, 0.5f, 0.0f);

	fslTranslateMatrix4x4(myMusicTileCard_[1].getMVMatrix(),1.f,0,-3);

	return FSL_STATUS_SUCCESS;
}

void fslGUIScene::receiveCardEvent( unsigned int eventID, void* data )
{
	//Child Cards call this function on events

	fslInputEventType event;
	if (eventID == FSL_INPUT_CLOSEWINDOW)
	{
		
		event.input = FSL_INPUT_CLOSEWINDOW;
// 		myExampleCard_.handleInputEvent( event );
		isRunning_ = false;

	}

}

