//
// C++ Implementation: fslShell
//
// Description:A shell to take care of EGL.
//
//
// Author: FCSZ <b20593>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include <FSL/fslShell.h>

fslShell::fslShell()
{

}

fslShell::~fslShell()
{

}

void fslShell::cleanupEGL(XWindow* xwin)
{
	printf("Cleaning Up EGL and Main Window\n");
	eglMakeCurrent(_egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglTerminate(_egldisplay);
#ifdef EGL_USE_X11	
	 XCloseDisplay(xwin->display);
#endif	
	eglReleaseThread();
}

void fslShell::cleanupGL()
{
	printf("Cleaning Up GL\n");
	//Nothing yet to see... move along
}

bool fslShell::handleEvents(XWindow *xMainWindow, XEvent event, fslInputEventType inputEvent, EGLint width, EGLint height, fslScene* _myScene)
{
#ifdef EGL_USE_X11	
	
	//[FCSZ] register closing window (WM_DELETE_WINDOW) event.
	Atom wmDelete = XInternAtom(xMainWindow->display,"WM_DELETE_WINDOW",True);
	XSetWMProtocols(xMainWindow->display, xMainWindow->window, &wmDelete, 1);
	
		while(XPending(xMainWindow->display))
		{	
			XNextEvent(xMainWindow->display,&event);
			switch(event.type)
			{
			case ButtonPress:
				//printf("Event:ButtonPress [%d, %d]\n", event.xbutton.x, event.xbutton.y);
				inputEvent.input = FSL_INPUT_POINTERDOWN;
				inputEvent.signalA = (unsigned int)(FSL_POINTER_MAX * ((float) event.xbutton.x / (float)width));
				inputEvent.signalB = (unsigned int)(FSL_POINTER_MAX * (((float)height - (float) event.xbutton.y) / (float)height));
				if (event.xbutton.x > 250)
				{
				inputEvent.signalC = 0;
				} else {inputEvent.signalC = 1;}
				inputEvent.time = event.xbutton.time;
				_myScene->handleInputEvent( inputEvent );
				break;
			case ButtonRelease:
				//printf("Event:ButtonRelease [%d, %d]\n", event.xbutton.x, event.xbutton.y);
				inputEvent.input = FSL_INPUT_POINTERUP;
				inputEvent.signalA = FSL_POINTER_MAX * ((float) event.xbutton.x / (float)width);
				inputEvent.signalB = FSL_POINTER_MAX * (((float)height - (float) event.xbutton.y) / (float)height);
				inputEvent.signalC = 0;
				inputEvent.time = event.xbutton.time;			
				_myScene->handleInputEvent( inputEvent );
				break;
			case KeyPress:
				//printf("Event:KeyPress [%d, %d]\n", event.xkey.x, event.xkey.y);
				inputEvent.input = FSL_INPUT_KEYPRESS;
				inputEvent.signalA = event.xkey.keycode;
				inputEvent.signalB = 0;
				inputEvent.signalC = 0;
				inputEvent.time = event.xkey.time;
				_myScene->handleInputEvent( inputEvent );
				break;
			case KeyRelease:
				//printf("Event:KeyRelease [%d, %d]\n", event.xkey.x, event.xkey.y); 
				inputEvent.input = FSL_INPUT_KEYRELEASE;
				inputEvent.signalA = event.xkey.keycode;
				inputEvent.signalB = 0;
				inputEvent.signalC = 0;
				inputEvent.time = event.xkey.time;
				_myScene->handleInputEvent( inputEvent );
				break;
			case MotionNotify:
				//printf("Event:MotionNotify [%d, %d]\n", event.xmotion.x, event.xmotion.y); 
				inputEvent.input = FSL_INPUT_POINTERMOVE;
				inputEvent.signalA = FSL_POINTER_MAX * ((float) event.xmotion.x / (float)width);
				inputEvent.signalB = FSL_POINTER_MAX * (((float) height - (float) event.xmotion.y) / (float)height);
				inputEvent.signalC = 0;
				inputEvent.time = event.xmotion.time;
				_myScene->handleInputEvent( inputEvent );
				break;
			case ClientMessage:
				inputEvent.input = FSL_INPUT_CLOSEWINDOW;
				_myScene->handleInputEvent( inputEvent );
				break;
			default:
				break;
			}
			
		}
#else
		//printf("1\n");
		FD_SET(fd, &fds);
		ret = select(fd + 1, &fds, NULL, NULL, &zeroTime);
		//printf("s=%d\n",ret);
		if(ret)
		{
			//printf("2\n");
			rd = read(fd, ev, sizeof(struct input_event) * 64);
			//printf("3\n");

			if (rd >= (int) sizeof(struct input_event)) 
			{
				
				for (unsigned int i = 0; i < rd / sizeof(struct input_event); i++)
				{ //printf("3\n");
					switch(ev[i].type)
					{
						case EV_SYN: //printf("Sync\n");
							if(pointerDown == FSL_TRUE) 
							{
								if( (currentX == 0) && (currentY == 0) )
								{//printf("Pointer UP [%d,%d]\n");
									//Pointer Up Event
									pointerDown = FSL_FALSE;
									inputEvent.input = FSL_INPUT_POINTERUP;
									inputEvent.signalA = lastX * 10;
									inputEvent.signalB = lastY * 10;
									inputEvent.signalC = 0;
									inputEvent.time = 0;
									_myScene->handleInputEvent( inputEvent );
								}
								else
								{
									//Pointer Move Event
									lastX = currentX;
									lastY = currentY;

									inputEvent.input = FSL_INPUT_POINTERMOVE;
									inputEvent.signalA = lastX * 10;
									inputEvent.signalB = lastY * 10;
									inputEvent.signalC = 0;
									inputEvent.time = 0;
									_myScene->handleInputEvent( inputEvent );

								}
							}
							else
							{
								//Pointer Down Event
								pointerDown = FSL_TRUE;
								lastX = currentX;
								lastY = currentY;
								inputEvent.input = FSL_INPUT_POINTERDOWN;
								inputEvent.signalA = lastX * 10;
								inputEvent.signalB = lastY * 10;
								inputEvent.signalC = 0;
								inputEvent.time = 0;
								_myScene->handleInputEvent( inputEvent );
								//printf("Pointer Down [%d,%d]\n", lastX, lastY);
							}
							break;
						case 3:
							if(ev[i].code == 0)//X
							{
								currentX = ev[i].value;
							}
							else if(ev[i].code == 1)//Y
							{
								currentY = ev[i].value;
							}
							break;
					}
				}
			}
		}

#endif		
	return true;
}

bool fslShell::initEGL(XWindow* xwin)
{
	static const EGLint configAttribs16[] =
	{
			EGL_RED_SIZE,           5,
			EGL_GREEN_SIZE,         6,
			EGL_BLUE_SIZE,          5,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,
			EGL_SAMPLES,            0,
			EGL_NONE
	};
	EGLint	numconfigs 	= 0;
	
#ifdef EGL_USE_X11
	static const EGLint configAttribs24[] =
	{
			EGL_RED_SIZE,           8,
			EGL_GREEN_SIZE,         8,
			EGL_BLUE_SIZE,          8,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,			
			EGL_SAMPLES,            0,
			EGL_NONE
	};


	unsigned int	width		= 1024;
	unsigned int	height		= 768;

	unsigned int 	dummy, depth;
	int 		x;
	Window 		rootwindow;

	printf("+Starting Main Window X Initialization:\n");

	printf("\tXOpenDisplay [");
	xwin->display = XOpenDisplay(NULL);
	printf("%d]\n", (int)xwin->display);
	assert(xwin->display != NULL);

	printf("\tDefaultScreen [");
	xwin->screen = DefaultScreen(xwin->display);
	printf("%d]\n", xwin->screen);

	rootwindow = RootWindow(xwin->display,xwin->screen);

	printf("\tXCreateSimpleWindow \n");
	xwin->window = XCreateSimpleWindow(xwin->display, rootwindow, 0, 0, width, height, 0, 0, WhitePixel (xwin->display, xwin->screen));
	
	XMapWindow(xwin->display, xwin->window);
	
	XStoreName(xwin->display,xwin->window,"Test Application for FileBrowserCard");

	{
		XSizeHints Hints;
		Hints.flags=PSize|PMinSize|PMaxSize;
		Hints.min_width=Hints.max_width=Hints.base_width=width;
		Hints.min_height=Hints.max_height=Hints.base_height=height;
		XSetWMNormalHints(xwin->display,xwin->window,&Hints);
	}	
	
	if(!XGetGeometry(xwin->display, xwin->window, &rootwindow, &x, &x, &width, &height, &dummy, &depth) || (depth == 0))
	{
		printf("\tERROR: X11 Window Creation Error\n");
		return false;
	}
	
	printf("-Finished Main Window X Initialization:\n");
	
	printf("+Starting EGL Initialization:\n");

	//Initialize EGL
	printf("\teglGetDisplay \n");
	_egldisplay = eglGetDisplay ( xwin->display );
	fslEGLCheck(true);

	printf("\teglInitialize \n");
	eglInitialize(_egldisplay, NULL, NULL);
	fslEGLCheck(true);

	printf("\teglBindAPI \n");
	eglBindAPI(EGL_OPENGL_ES_API);

	printf("\teglChooseConfig \n");
	if(depth == 16)
		eglChooseConfig(_egldisplay, configAttribs16, &_eglconfig, 1, &numconfigs);
	else if(depth == 24)
		eglChooseConfig(_egldisplay, configAttribs24, &_eglconfig, 1, &numconfigs);
	else
	{
		printf("\tERROR: Only 16b and 24b windows supported \n");
		return false;
	}
	
	fslEGLCheck(true);

	if(numconfigs != 1)
	{
		printf("\tERROR: No EGL suitable configurations found. \n");
		return false;
	}

	printf("\teglCreateWindowSurface \n");
	_eglsurface = eglCreateWindowSurface(_egldisplay, _eglconfig, xwin->window, NULL);
	fslEGLCheck(true);
	
#else 
	_egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);

    printf("*eglInitialize \n");
	eglInitialize(_egldisplay, NULL, NULL);
	fslEGLCheck(true);;

	printf("*eglChooseConfig \n");
	eglChooseConfig(_egldisplay, configAttribs16, &_eglconfig, 1, &numconfigs);
	fslEGLCheck(true);
	if(numconfigs != 1)
	{
		printf("\tERROR: No EGL suitable configurations found. \n");
		return false;
	}

	printf("*eglCreateWindowSurface \n");
	_eglsurface = eglCreateWindowSurface(_egldisplay, _eglconfig, open("/dev/fb0", O_RDWR), NULL);
	fslEGLCheck(true);	
#endif


	printf("\tglCreateContext \n");
	_eglcontext = eglCreateContext(_egldisplay, _eglconfig, NULL, NULL);
	fslEGLCheck(true);

	printf("\teglMakeCurrent \n");
	eglMakeCurrent(_egldisplay, _eglsurface, _eglsurface, _eglcontext);
	fslEGLCheck(true);
	
	printf("-Finished EGL Initialization:\n");
	
	return true;
}

bool fslShell::initGL(fslScene *_myScene )
{
	printf("+Starting GL Initialization: \n");

	//There is no master scene in this example, so we are handling the init
	
	if( _myScene->initScene() != FSL_STATUS_SUCCESS )
		return false;
	
	_myScene->transitionOnStage(FSL_TRANSITION_UNSPECIFIED);
	
	printf("-Finished GL Initialization\n");

	return true;
}

void fslShell::printFramerate(unsigned int &startTime, unsigned int &endTime, EGLint width, EGLint height)
{
#ifdef PRINT_FRAMERATE
		// Printing FPS metric
		if(_currentFrame % 100 == 0)
		{
			endTime = fslGetTickCount();
			if(startTime && startTime < endTime)
			{
				
				printf("\t%f FPS, %f ms/frame [%d] (%d x %d)\n", 
					100 * 1000.f / (endTime - startTime),  
					(float)(endTime - startTime)/(float)100, 
					_currentFrame, width, height);
			
			}
			startTime = fslGetTickCount();
		}
#endif	
}

void fslShell::render(fslScene *_myScene, int width, int height)
{
	//glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glEnable(GL_DEPTH_TEST);

	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	

	_myScene->setAspectRatio( width / height );
	_myScene->renderScene( FSL_PHASE_UNSPECIFIED );
	
	//printf("Swapping..\n");
	eglSwapBuffers(_egldisplay, _eglsurface);
}

void fslShell::selectInputType(XWindow* xMainWindow)
{
#ifdef EGL_USE_X11		
	XSelectInput(xMainWindow->display,xMainWindow->window,
	       ExposureMask|KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask);
#else
	int fd,rd;
	struct input_event ev[64];	
	unsigned int currentX = 0;
	unsigned int currentY = 0;
	unsigned int lastX = 0;
	unsigned int lastY = 0;	
	fslBool pointerDown = FSL_FALSE;
	fd_set fds;
	struct timeval zeroTime;
	int ret;
	
	zeroTime.tv_sec = 5;
	zeroTime.tv_usec = 0;
	
	fd = open("/dev/input/event1", O_RDONLY|O_NONBLOCK);
	if(fd < 0)
	{
		printf("ERROR: Cannot open touchscreen input\n");
		return 0;
	}
	FD_ZERO(&fds);
	FD_SET(fd, &fds);
	
#endif
}

