/* File: fslfilebrowsercard.cpp
 * 
 * Desc: This card provides a simple file selection interfacet
 * The current implmentation generates a series of vertical textured buttons
 * If a button is detected a pressed, a string is passed to the parent
 * 			
 * Copyright 2009 Freescale Semiconductor
 */


static const char* g_strEnd = (char*)
    "}													\n";

//Simple shader for simple texture application
static const char* g_strVSProgramA = (char*)
	"uniform   mat4 g_matModelView;						\n"
    "													\n"
    "attribute vec4 g_vVertex;							\n"
    "attribute vec4 g_vTexcoord;						\n"
    "													\n"
    "varying   vec4 g_vVSTexcoord;						\n"
    "													\n";

static const char* g_strVSProgramB = (char*)
    "void main()										\n"
    "{													\n"	
	"    vec4 cardPosition = g_matModelView * g_vVertex;\n"
    "    g_vVSTexcoord = g_vTexcoord;					\n";


static const char* g_strFSProgramA = (char*)
    "#ifdef GL_FRAGMENT_PRECISION_HIGH									\n"
    "   precision highp float;											\n"
    "#else																\n"
    "   precision mediump float;										\n"
    "#endif																\n"
    "																	\n"
    "uniform   sampler2D g_sImageTexture;								\n"
    "varying   vec4      g_vVSTexcoord;									\n"
    "																	\n";

//Just sets up the sampled button texture and hands it to the scene post-string
static const char* g_strFSProgramB = (char*)
    "void main()														\n"
    "{																	\n"
	"    vec4 cardColor;	\n"		
	"    cardColor = texture2D(g_sImageTexture, g_vVSTexcoord.xy);	\n";	
    
#include <GLES2/gl2.h>
#include <EGL/egl.h>
#include <CARD/fslcard.h>
#include <CARD/fslscene.h>
#include <CARD/fslfilebrowsercard.h>
#include <FSL/fslutil.h>

#include <stdio.h>
#include <cstdio>
#include <iostream>

#ifdef DBGPRINT
#undef DBGPRINT
#endif

#if 1
/*#define PREPRINT std::cout << "FILEBROWSER: [" << __FILE__ << ":" << __LINE__ << "] ";*/
#define PREPRINT std::cout << "FILEBROWSER: ";
#define DBGPRINT(...) PREPRINT printf(__VA_ARGS__);
#else
#define DBGPRINT(fmt,...)
#endif

fslFileBrowserCard::fslFileBrowserCard( )
{
	parentScene_ = NULL;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	
	//Simple bounds for the card input detection
	buttonLimits_[0] = 0.0f;
	buttonLimits_[1] = 0.25f;
	buttonLimits_[2] = 0.5f;
	buttonLimits_[3] = 0.75f;
	buttonLimits_[4] = 1.0f;

	//Simple button state
	for(int i=0;  i < FSL_MAX_FILE_BUTTONS; i++)
	{
		buttonList_[i].loaded = FSL_FALSE;
		buttonList_[i].isPressed = FSL_FALSE;
	}
		
	//Strings returned to parent upon press detection
	sprintf(buttonList_[0].filename, "media/video/movie1.mp4");
	sprintf(buttonList_[1].filename, "media/video/movie2.mp4");
	sprintf(buttonList_[2].filename, "media/video/movie3.mp4");
	sprintf(buttonList_[3].filename, "media/video/movie4.mp4");
	
}


fslFileBrowserCard::fslFileBrowserCard( void* parentScene )
{
	parentScene_ = parentScene;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	
	
	//Simple bounds for the card input detection
	buttonLimits_[0] = 0.0f;
	buttonLimits_[1] = 0.25f;
	buttonLimits_[2] = 0.5f;
	buttonLimits_[3] = 0.75f;
	buttonLimits_[4] = 1.0f;

	//Simple button state
	for(int i=0;  i < FSL_MAX_FILE_BUTTONS; i++)
	{
		buttonList_[i].loaded = FSL_FALSE;
		buttonList_[i].isPressed = FSL_FALSE;
	}
		
	//Strings returned to parent upon press detection
	sprintf(buttonList_[0].filename, "media/video/movie1.mp4");
	sprintf(buttonList_[1].filename, "media/video/movie2.mp4");
	sprintf(buttonList_[2].filename, "media/video/movie3.mp4");
	sprintf(buttonList_[3].filename, "media/video/movie4.mp4");
	
}

fslFileBrowserCard::~fslFileBrowserCard()
{
	//Nothing dynamically allocated in this card
}

fslStatus fslFileBrowserCard::initCard( fslInitStageType init, unsigned int config) 
{
	static GLuint hVertexShader;
	static GLuint hFragmentShader;
	
	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
			{
				GLint nCompileResult;
				char vs[MAX_SHADER_LENGTH];
				char ps[MAX_SHADER_LENGTH];
				const char* pvs = (char*)(vs);
				const char* pps = (char*)(ps);

				DBGPRINT("Starting Initialization \n");

				//Tack the scene contribution to the shders onto ours
				sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
				sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				//Load the textures for the buttons
				for(int i=0; i < FSL_MAX_FILE_BUTTONS; i++)
					if(buttonList_[i].loaded != FSL_TRUE){
						switch(i){
						case 0:	if(!fslInit2DCTESTextureGL( "media/textures/button1.CTES", &buttonList_[i].textureHandle ))
									return FSL_STATUS_GENERAL_ERROR;
									break;
						case 1:	if(!fslInit2DCTESTextureGL( "media/textures/button2.CTES", &buttonList_[i].textureHandle ))
									return FSL_STATUS_GENERAL_ERROR;
									break;
						case 2:	if(!fslInit2DCTESTextureGL( "media/textures/button3.CTES", &buttonList_[i].textureHandle ))
									return FSL_STATUS_GENERAL_ERROR;
									break;
						case 3:	if(!fslInit2DCTESTextureGL( "media/textures/button4.CTES", &buttonList_[i].textureHandle ))
									return FSL_STATUS_GENERAL_ERROR;
									break;
						
						//Set state to reflect the texture was loaded for the button
						buttonList_[i].loaded = FSL_TRUE;
					}
				}				
				DBGPRINT("Vertex Shader = \n %s \n", vs);
				DBGPRINT("Pixel Shader = \n %s \n", ps);

				// Compile the shaders
				DBGPRINT("Compiling Shaders... \n");
				hVertexShader = glCreateShader( GL_VERTEX_SHADER );
				glShaderSource( hVertexShader, 1, &pvs, NULL );
				glCompileShader( hVertexShader );

				hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
				glShaderSource( hFragmentShader, 1, &pps, NULL );
				glCompileShader( hFragmentShader );

				// Check for compile success
				nCompileResult = 0;
				glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					return FSL_STATUS_GENERAL_ERROR;
				}

				// Attach the individual shaders to the common shader program
				shaderProgram_[config] = glCreateProgram();
				glAttachShader( shaderProgram_[config], hVertexShader );
				glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
				glBindAttribLocation(shaderProgram_[config], vertexLoc_,   "g_vVertex");
				glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "g_vTexcoord");
				
				numShaderAttributes_ = 2; //The scene can use attributes higher than this

			}
 		
			break;
		case FSL_INIT_LINK:
			{
				GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
				DBGPRINT("Linking Shaders... \n");
				glLinkProgram( shaderProgram_[config] );

				// Check for link success
				nLinkResult = 0;
				glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);
				if (!nLinkResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error linking shader! \n");
					glGetProgramInfoLog(shaderProgram_[config], 1024, &nLength, Log);
					return FSL_STATUS_GENERAL_ERROR;
				}

				modelMatrixLoc_[config] = glGetUniformLocation( shaderProgram_[config], "g_matModelView" );

				glDeleteShader( hVertexShader );
				glDeleteShader( hFragmentShader );
			}			
			break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslFileBrowserCard::shutdownCard() //Releases dynamic resources without destroying card
{
	//No dynamic resources in this card
	//Textures may be added here in the future
	return FSL_STATUS_SUCCESS;
}

fslStatus fslFileBrowserCard::resumeCard()   //Reallocates resources for card
{
	//No dynamic resources in this card
	//Textures may be added here in the future
	return FSL_STATUS_SUCCESS;
}

fslStatus fslFileBrowserCard::animTick(void) //adjusts state by the time tick set in resolution
{
	//This card currently has no animation
	return FSL_STATUS_SUCCESS;
}

fslStatus fslFileBrowserCard::setTickRes(float time)
{
	//This card currently has no animation
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslFileBrowserCard::setTime(float time)
{
	//This card currently has no animation
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

float fslFileBrowserCard::getTime()
{
	return currentTime_;
}

fslStatus fslFileBrowserCard::preRenderCard(unsigned int phaseID, unsigned int config)
{
	if(phaseID == 1)
	{
		// Set the shader program and the texture
		glUseProgram( shaderProgram_[config] );
	}

	return FSL_STATUS_SUCCESS;
}

fslStatus fslFileBrowserCard::renderCard( unsigned int phaseID, unsigned int config )  
{
	//This card is simple and only has one render phase
	switch(phaseID)
	{
		case 1:
			{
				float matModelView[16] = {0};

				for(int i = 0; i < FSL_MAX_FILE_BUTTONS; i++ )
				{

					float VertexPositions[] =
										{
											-1.0,	-1.0, 0.0f, 1.0f,
											1.0f,	-1.0, 0.0f, 1.0f,
											-1.0,	1.0f, 0.0f, 1.0f,
											1.0f,	1.0f, 0.0f, 1.0f
										};
										
					float VertexTexcoord[] =
										{
											0.0f,	1.0f,
											1.0f,	1.0f,
											0.0f,	0.0f,
											1.0f,	0.0f
										};
					
					//Scale the button smaller if being hit
					if(buttonList_[i].isPressed == FSL_TRUE)
					{
						matModelView[ 0] = 0.60f;
						matModelView[ 5] = 0.15f;
					}
					else
					{
						matModelView[ 0] = 0.80f;
						matModelView[ 5] = 0.20f;
					}
										
					matModelView[10] = 1.0f;
					//Place the buttons equally spaced vertically
					matModelView[13] = ((float)i/2.0f) - 1.0f + 0.145f;
					matModelView[15] = 1.0f;
					

					glUniformMatrix4fv( modelMatrixLoc_[config],1, 0, matModelView );
					
					glBindTexture( GL_TEXTURE_2D, buttonList_[i].textureHandle );

					glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
					glEnableVertexAttribArray( vertexLoc_ );

					glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexcoord);
					glEnableVertexAttribArray( texcoordLoc_ );

					//DBGPRINT("Drawing...\n");
					glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );

					glDisableVertexAttribArray( vertexLoc_ );
					glDisableVertexAttribArray( texcoordLoc_ );
				}
			}

			break;

		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}


void fslFileBrowserCard::getRenderPhases(fslPhaseElementType* phaseArray) 
{
	if(phaseArray == NULL)
		return;
		
	//phaseArray is assumed fixed size of MAX_PHASE_ARRAY_SIZE
	//This card is simple and only has one render stage
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_WRITEONLY;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[1].phaseID = 0;

	return;
}


float fslFileBrowserCard::getAspectRatio()
{
	//This is only a hard-coded suggested value as the scene can really use whatever it wants
	return 1.0f; 
}

//These events are ONLY card relevent;  All coords are in 0-1, 0-1, card space (lower left origin)
fslStatus fslFileBrowserCard::handleInputEvent( fslInputEventType event)
{
	if(event.input == FSL_INPUT_POINTERDOWN) 
	{
		//get the reletive position coord
		float ypos = (float) event.signalB / (float) FSL_POINTER_MAX;
		
		for(int i = 1; i < 5; i++)
		{
			//test against the buttons bounds to see what got hit
			if( (ypos >= buttonLimits_[i-1]) && (ypos <= buttonLimits_[i]) )
			{
				currentButtonDown_ = i;
				//Send event to the parent based on the button that got hit
				((fslScene*) parentScene_)->receiveCardEvent( baseEventID_ + 1, buttonList_[i-1].filename);
				buttonList_[i-1].isPressed = FSL_TRUE;
				
			}
		}
	} else 	if(event.input == FSL_INPUT_POINTERUP) 
	{
		//Reset all the buttons on a pointer up
		for(int i = 0; i < FSL_MAX_FILE_BUTTONS; i++ )
		{
			buttonList_[i].isPressed = FSL_FALSE;
		}
	}
	
	return FSL_STATUS_SUCCESS;
}



#undef DBGPRINT


