
/* File: cardsample.c
 * 
 * Desc: Mary had a little lamb.  Its fleece was scratchy and uncomfortable.
 * 			
 * Copyright  2009 Freescale Semiconductor
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <fcntl.h> 
#include <malloc.h> 
#include <math.h> 
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <limits.h>
#include <sys/select.h>

 
#ifndef BOOL
#define BOOL int
#define TRUE 1
#define FALSE !TRUE
#endif

#include <GLES2/gl2.h>
#define GL_GLEXT_PROTOTYPES 1
#include <GLES2/gl2ext.h>
#include <GLES2/gl2amdext.h>
#include <EGL/egl.h>
#define EGL_EGLEXT_PROTOTYPES 1
#include <EGL/eglext.h>

#ifdef EGL_USE_X11

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/render.h>
#include <X11/extensions/Xrender.h>

#endif

#include <FSL/fslutil.h>
 
#include <CARD/fslscene.h>
#include "testscene_fslfilebrowsercard.h"


//--------------------------------------------------------------------------------------
// Types
//--------------------------------------------------------------------------------------

#ifdef EGL_USE_X11
typedef struct XWindow_s {
	Display *display;
	Window window;
	int screen;
	Visual *visual;
	GC gc;
	Colormap colormap;
	int depth;
	int pixelsize;
	XFontStruct *font_info;
	XWindowAttributes attr;
	Picture picture;
	Pixmap pixmap;
} XWindow;
#else
typedef int XWindow;
#endif

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

EGLDisplay		g_egldisplay;
EGLConfig		g_eglconfig;
EGLSurface		g_eglsurface;
EGLContext		g_eglcontext;

unsigned int	g_currentFrame = 0;

fslExampleScene g_myScene( NULL );

//--------------------------------------------------------------------------------------
// Macro constants
//--------------------------------------------------------------------------------------

#define PRINT_FRAMERATE 1

//--------------------------------------------------------------------------------------
// Name: initGL
// Desc: Sets up GL variables and state before rendering.  Sets up EGLImage from X11 pixmap
//--------------------------------------------------------------------------------------
BOOL initGL( )
{
	printf("+Starting GL Initialization: \n");

	//There is no master scene in this example, so we are handling the init
	
	if( g_myScene.initScene() != FSL_STATUS_SUCCESS )
		return FALSE;
	
	g_myScene.transitionOnStage(FSL_TRANSITION_UNSPECIFIED);
	
	printf("-Finished GL Initialization\n");

	return TRUE;
}

//--------------------------------------------------------------------------------------
// Name: cleanupGL()
// Desc: cleanup any objects created in initGL
//--------------------------------------------------------------------------------------
void cleanupGL( )
{
	printf("Cleaning Up GL\n");
	//Nothing yet to see... move along
}

//--------------------------------------------------------------------------------------
// Name: render
// Desc: Render the scene
//--------------------------------------------------------------------------------------
void render(int width, int height)
{
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	//printf("Clearing..\n");
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	
	
	g_myScene.setAspectRatio( width / height );
	g_myScene.renderScene( FSL_PHASE_UNSPECIFIED );
	
	//printf("Swapping..\n");
	eglSwapBuffers(g_egldisplay, g_eglsurface);
}

//--------------------------------------------------------------------------------------
// Name: initEGL()
// Desc: Initialize the X window and EGL objects / state need draw to screen
//--------------------------------------------------------------------------------------

BOOL initEGL(XWindow* xwin)
{
	static const EGLint configAttribs16[] =
	{
			EGL_RED_SIZE,           5,
			EGL_GREEN_SIZE,         6,
			EGL_BLUE_SIZE,          5,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,
			EGL_SAMPLES,            0,
			EGL_NONE
	};
	EGLint	numconfigs 	= 0;
	
#ifdef EGL_USE_X11
	static const EGLint configAttribs24[] =
	{
			EGL_RED_SIZE,           8,
			EGL_GREEN_SIZE,         8,
			EGL_BLUE_SIZE,          8,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,			
			EGL_SAMPLES,            0,
			EGL_NONE
	};


	unsigned int	width		= 500;
	unsigned int	height		= 350;

	unsigned int 	dummy, depth;
	int 		x;
	Window 		rootwindow;

	printf("+Starting Main Window X Initialization:\n");

	printf("\tXOpenDisplay [");
	xwin->display = XOpenDisplay(NULL);
	printf("%d]\n", (int)xwin->display);
	assert(xwin->display != NULL);

	printf("\tDefaultScreen [");
	xwin->screen = DefaultScreen(xwin->display);
	printf("%d]\n", xwin->screen);

	rootwindow = RootWindow(xwin->display,xwin->screen);

	printf("\tXCreateSimpleWindow \n");
	xwin->window = XCreateSimpleWindow(xwin->display, rootwindow, 0, 0, width, height, 0, 0, WhitePixel (xwin->display, xwin->screen));
	
	XMapWindow(xwin->display, xwin->window);
	
	XStoreName(xwin->display,xwin->window,"Test Application for FileBrowserCard");

	{
		XSizeHints Hints;
		Hints.flags=PSize|PMinSize|PMaxSize;
		Hints.min_width=Hints.max_width=Hints.base_width=width;
		Hints.min_height=Hints.max_height=Hints.base_height=height;
		XSetWMNormalHints(xwin->display,xwin->window,&Hints);
	}	
	
	if(!XGetGeometry(xwin->display, xwin->window, &rootwindow, &x, &x, &width, &height, &dummy, &depth) || (depth == 0))
	{
		printf("\tERROR: X11 Window Creation Error\n");
		return FALSE;
	}
	
	printf("-Finished Main Window X Initialization:\n");
	
	printf("+Starting EGL Initialization:\n");

	//Initialize EGL
	printf("\teglGetDisplay \n");
	g_egldisplay = eglGetDisplay ( xwin->display );
	fslEGLCheck(TRUE);

	printf("\teglInitialize \n");
	eglInitialize(g_egldisplay, NULL, NULL);
	fslEGLCheck(TRUE);

	printf("\teglBindAPI \n");
	eglBindAPI(EGL_OPENGL_ES_API);

	printf("\teglChooseConfig \n");
	if(depth == 16)
		eglChooseConfig(g_egldisplay, configAttribs16, &g_eglconfig, 1, &numconfigs);
	else if(depth == 24)
		eglChooseConfig(g_egldisplay, configAttribs24, &g_eglconfig, 1, &numconfigs);
	else
	{
		printf("\tERROR: Only 16b and 24b windows supported \n");
		return FALSE;
	}
	
	fslEGLCheck(TRUE);

	if(numconfigs != 1)
	{
		printf("\tERROR: No EGL suitable configurations found. \n");
		return FALSE;
	}

	printf("\teglCreateWindowSurface \n");
	g_eglsurface = eglCreateWindowSurface(g_egldisplay, g_eglconfig, xwin->window, NULL);
	fslEGLCheck(TRUE);
	
#else 
	g_egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);

    printf("*eglInitialize \n");
	eglInitialize(g_egldisplay, NULL, NULL);
	fslEGLCheck(TRUE);;

	printf("*eglChooseConfig \n");
	eglChooseConfig(g_egldisplay, configAttribs16, &g_eglconfig, 1, &numconfigs);
	fslEGLCheck(TRUE);
	if(numconfigs != 1)
	{
		printf("\tERROR: No EGL suitable configurations found. \n");
		return FALSE;
	}

	printf("*eglCreateWindowSurface \n");
	g_eglsurface = eglCreateWindowSurface(g_egldisplay, g_eglconfig, open("/dev/fb0", O_RDWR), NULL);
	fslEGLCheck(TRUE);	
#endif


	printf("\tglCreateContext \n");
	g_eglcontext = eglCreateContext(g_egldisplay, g_eglconfig, NULL, NULL);
	fslEGLCheck(TRUE);

	printf("\teglMakeCurrent \n");
	eglMakeCurrent(g_egldisplay, g_eglsurface, g_eglsurface, g_eglcontext);
	fslEGLCheck(TRUE);
	
	printf("-Finished EGL Initialization:\n");
	
	return TRUE;
}

//--------------------------------------------------------------------------------------
// Name: cleanupEGL
// Desc: Cleanup any objects created in initEGL
//--------------------------------------------------------------------------------------

void cleanupEGL(XWindow* xwin)
{
	printf("Cleaning Up EGL and Main Window\n");
	eglMakeCurrent(g_egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglTerminate(g_egldisplay);
#ifdef EGL_USE_X11	
    XCloseDisplay(xwin->display);
#endif	
	eglReleaseThread();
}

//--------------------------------------------------------------------------------------
// Name: handleEvents
// Desc: Handle events from X: Needs to use OpenKODE
//--------------------------------------------------------------------------------------
BOOL handleEvents(XWindow *xwin)
{
	//TBD
	return TRUE;
}



//--------------------------------------------------------------------------------------
// Name: main
// Desc: main
//--------------------------------------------------------------------------------------
int main (void)
{
	XWindow	xMainWindow;
#ifdef EGL_USE_X11
	XEvent 	event;
#endif	
	EGLint 	width = 0, height = 0;
	float	animStartTime = 0, animEndTime = 0;
	fslInputEventType inputEvent;
	
#ifdef PRINT_FRAMERATE	
	unsigned int	startTime = 0, endTime = 0;
#endif	

	
	printf("Freescale Sample Card Demo\n");
	printf("++Starting Initialization\n");
	
	//Set up error handler here
	
	assert( initEGL( &xMainWindow ) );

	assert( initGL( ) );
	
	eglQuerySurface(g_egldisplay, g_eglsurface, EGL_WIDTH, &width);
	eglQuerySurface(g_egldisplay, g_eglsurface, EGL_HEIGHT, &height);
	printf("--Finished Initializing (%d x %d).  Now Rendering. \n",width, height);

	animStartTime = ((float) fslGetTickCount() / 1000.f);

#ifdef EGL_USE_X11		
	XSelectInput(xMainWindow.display,xMainWindow.window,
	       ExposureMask|KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask);
#else
	int fd,rd;
	struct input_event ev[64];	
	unsigned int currentX = 0;
	unsigned int currentY = 0;
	unsigned int lastX = 0;
	unsigned int lastY = 0;	
	fslBool pointerDown = FSL_FALSE;
	fd_set fds;
	struct timeval zeroTime;
	int ret;
	
	zeroTime.tv_sec = 5;
	zeroTime.tv_usec = 0;
	
	fd = open("/dev/input/event1", O_RDONLY|O_NONBLOCK);
	if(fd < 0)
	{
		printf("ERROR: Cannot open touchscreen input\n");
		return 0;
	}
	FD_ZERO(&fds);
	FD_SET(fd, &fds);
	
#endif
		   
	while (g_currentFrame < 100000)
	{
		eglQuerySurface(g_egldisplay, g_eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(g_egldisplay, g_eglsurface, EGL_HEIGHT, &height);

		//Check for new events here
		//Some KODE or X calls
#ifdef EGL_USE_X11		
		while(XPending(xMainWindow.display))
		{	
			XNextEvent(xMainWindow.display,&event);
			switch(event.type)
			{
			case ButtonPress:
				//printf("Event:ButtonPress [%d, %d]\n", event.xbutton.x, event.xbutton.y);
				inputEvent.input = FSL_INPUT_POINTERDOWN;
				inputEvent.signalA = (unsigned int)(FSL_POINTER_MAX * ((float) event.xbutton.x / (float)width));
				inputEvent.signalB = (unsigned int)(FSL_POINTER_MAX * (((float)height - (float) event.xbutton.y) / (float)height));
				inputEvent.signalC = 0;
				inputEvent.time = event.xbutton.time;
				g_myScene.handleInputEvent( inputEvent );
				break;
			case ButtonRelease:
				//printf("Event:ButtonRelease [%d, %d]\n", event.xbutton.x, event.xbutton.y);
				inputEvent.input = FSL_INPUT_POINTERUP;
				inputEvent.signalA = FSL_POINTER_MAX * ((float) event.xbutton.x / (float)width);
				inputEvent.signalB = FSL_POINTER_MAX * (((float)height - (float) event.xbutton.y) / (float)height);
				inputEvent.signalC = 0;
				inputEvent.time = event.xbutton.time;			
				g_myScene.handleInputEvent( inputEvent );
				break;
			case KeyPress:
				//printf("Event:KeyPress [%d, %d]\n", event.xkey.x, event.xkey.y);
				inputEvent.input = FSL_INPUT_KEYPRESS;
				inputEvent.signalA = event.xkey.keycode;
				inputEvent.signalB = 0;
				inputEvent.signalC = 0;
				inputEvent.time = event.xkey.time;
				g_myScene.handleInputEvent( inputEvent );
				break;
			case KeyRelease:
				//printf("Event:KeyRelease [%d, %d]\n", event.xkey.x, event.xkey.y); 
				inputEvent.input = FSL_INPUT_KEYRELEASE;
				inputEvent.signalA = event.xkey.keycode;
				inputEvent.signalB = 0;
				inputEvent.signalC = 0;
				inputEvent.time = event.xkey.time;
				g_myScene.handleInputEvent( inputEvent );
				break;
			case MotionNotify:
				//printf("Event:MotionNotify [%d, %d]\n", event.xmotion.x, event.xmotion.y); 
				inputEvent.input = FSL_INPUT_POINTERMOVE;
				inputEvent.signalA = FSL_POINTER_MAX * ((float) event.xmotion.x / (float)width);
				inputEvent.signalB = FSL_POINTER_MAX * (((float) height - (float) event.xmotion.y) / (float)height);
				inputEvent.signalC = 0;
				inputEvent.time = event.xmotion.time;
				g_myScene.handleInputEvent( inputEvent );
				break;
			}
			
		}
#else
		//printf("1\n");
		FD_SET(fd, &fds);
		ret = select(fd + 1, &fds, NULL, NULL, &zeroTime);
		//printf("s=%d\n",ret);
		if(ret)
		{
			//printf("2\n");
			rd = read(fd, ev, sizeof(struct input_event) * 64);
			//printf("3\n");

			if (rd >= (int) sizeof(struct input_event)) 
			{
				
				for (unsigned int i = 0; i < rd / sizeof(struct input_event); i++)
				{ //printf("3\n");
					switch(ev[i].type)
					{
						case EV_SYN: //printf("Sync\n");
							if(pointerDown == FSL_TRUE) 
							{
								if( (currentX == 0) && (currentY == 0) )
								{//printf("Pointer UP [%d,%d]\n");
									//Pointer Up Event
									pointerDown = FSL_FALSE;
									inputEvent.input = FSL_INPUT_POINTERUP;
									inputEvent.signalA = lastX * 10;
									inputEvent.signalB = lastY * 10;
									inputEvent.signalC = 0;
									inputEvent.time = 0;
									g_myScene.handleInputEvent( inputEvent );
								}
								else
								{
									//Pointer Move Event
									lastX = currentX;
									lastY = currentY;

									inputEvent.input = FSL_INPUT_POINTERMOVE;
									inputEvent.signalA = lastX * 10;
									inputEvent.signalB = lastY * 10;
									inputEvent.signalC = 0;
									inputEvent.time = 0;
									g_myScene.handleInputEvent( inputEvent );

								}
							}
							else
							{
								//Pointer Down Event
								pointerDown = FSL_TRUE;
								lastX = currentX;
								lastY = currentY;
								inputEvent.input = FSL_INPUT_POINTERDOWN;
								inputEvent.signalA = lastX * 10;
								inputEvent.signalB = lastY * 10;
								inputEvent.signalC = 0;
								inputEvent.time = 0;
								g_myScene.handleInputEvent( inputEvent );
								//printf("Pointer Down [%d,%d]\n", lastX, lastY);
							}
							break;
						case 3:
							if(ev[i].code == 0)//X
							{
								currentX = ev[i].value;
							}
							else if(ev[i].code == 1)//Y
							{
								currentY = ev[i].value;
							}
							break;
					}
				}
			}
		}

#endif		
		animEndTime = ( (float) fslGetTickCount() / 1000.0f);
		g_myScene.animResolve(animEndTime - animStartTime);
		animStartTime = animEndTime;
		
		render(width, height);

		g_currentFrame++;

#ifdef PRINT_FRAMERATE
		// Printing FPS metric
		if(g_currentFrame % 100 == 0)
		{
			endTime = fslGetTickCount();
			if(startTime && startTime < endTime)
			{
				
				printf("\t%f FPS, %f ms/frame [%d] (%d x %d)\n", 
					100 * 1000.f / (endTime - startTime),  
					(float)(endTime - startTime)/(float)100, 
					g_currentFrame, width, height);
			
			}
			startTime = fslGetTickCount();
		}
#endif		
							
	}
	cleanupGL( );
	cleanupEGL( &xMainWindow );

	return 0;
}





