
#include <CARD/fslscene.h>
#include "testscene_fslfilebrowsercard.h"


fslExampleScene::fslExampleScene( fslScene* parentScene )
{
	parentScene_ = parentScene;

	for(int i=0; i < MAX_CARDS_IN_SCENE; i++)
		cardList_[i] = NULL;

	for(int i=0; i < MAX_SCENES_IN_SCENE; i++)
		sceneList_[i] = NULL;

	numCards_ = 0; 
	numScenes_ = 0;
	
	newCardID_ = 0;
	orientation_ = FSL_LANDSCAPE;
	isVisible_ = FSL_FALSE;

	myCard_.setParentScene(this);
	
	//base value to keep track of events from card
	// i.e if between 1000 - 1999, then it is from this card
	//Only really valuable when tracking multiple cards
	myCard_.setBaseEventID( 1000 ); 
	
	return;
}

fslExampleScene::~fslExampleScene()
{
	return;
}
	
fslStatus fslExampleScene::setAspectRatio( float aspect )
{
	//There is no big impact to this card for changing the aspect ratio
	aspectRatio_ = aspect;
	return FSL_STATUS_SUCCESS;
}
	
fslStatus fslExampleScene::setOrientation( fslOrientationType orientation )
{
	//There is no big impact to this card for changing the orientation
	orientation_ = orientation;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleScene::transitionOnStage( fslTransitionType transition)
{
	//Nothing fancy for the example
	//This should probably trigger an animation state change in a real scene
	isVisible_ = FSL_TRUE;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleScene::transitionOffStage( fslTransitionType transition)
{
	isVisible_ = FSL_FALSE;
	return FSL_STATUS_SUCCESS;
}

float animAngle = 0.0f;

fslStatus fslExampleScene::renderScene( fslPhaseType onlyThese)
{
	float matProjMatrix[16] = {0};
	/*
	float matA[16];
	float vecA[4];
	float vecB[4];
	float vecC[4];
	*/
	
	if(isVisible_ != FSL_TRUE)
		return FSL_STATUS_SUCCESS;

		
	if(onlyThese == FSL_PHASE_RENDER_WRITEONLY || onlyThese == FSL_PHASE_UNSPECIFIED)
	{
		
		//Set Identity Matrix
		matProjMatrix[ 0] = 1.0f;
		matProjMatrix[ 5] = 1.0f;
		matProjMatrix[10] = 1.0f;
		matProjMatrix[15] = 1.0f;	
		
		//Rotation
		matProjMatrix[ 0] = +cos( animAngle );
        matProjMatrix[ 3] = +sin( animAngle );
        matProjMatrix[ 8] = -sin( animAngle );
        matProjMatrix[10] = +cos( animAngle );
		
		//memcpy(currentProjMatrix_, matProjMatrix, (sizeof(float) * 16));
		fslInvertMatrix4x4( matProjMatrix, currentInvProjMatrix_);
		
		/*
		vecA [0] = 0.2;
		vecA [1] = 0.4;
		vecA [2] = 0.0;
		vecA [3] = 0.0;
		
		fslMultMatrix4x4Vec4x1 ( matProjMatrix, vecA, vecB );
		
		printf(" transformed vec = [%f,%f,%f,%f] \n",vecB[0],vecB[1],vecB[2],vecB[3]);
		
		fslInvertMatrix4x4( matProjMatrix, matA);
		
		fslMultMatrix4x4Vec4x1 ( matA, vecB, vecC );
		
		printf(" invtransformed vec = [%f,%f,%f,%f] \n",vecC[0],vecC[1],vecC[2],vecC[3]);		
		*/
		//Master Scene (or application) handles the clears
		//Scene handles other state setup such as:
		glDisable(GL_DEPTH_TEST);
	
		myCard_.preRenderCard(examplePhase_, 0);
	
		//set attribs and uniforms
		glUniformMatrix4fv( projMatrixLoc_[0], 1, 0, matProjMatrix );
	
		//Render the card
		//This needs more structure in a real scene
		//As this is a simple scene and a simple card, we card just render it without
		//  parsing the phase list, etc.  More complex cards will need to be broken up and render phases batched.
		//printf("Calling Render Card..\n");
		return myCard_.renderCard(examplePhase_, 0);
	}
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleScene::handleInputEvent( fslInputEventType event)
{
	float vecA[4] = {0};
	float vecC[4] = {0};
	
	//This switch is for catching other event types
	/*
	printf("Scene Event:");
	switch(event.input)
	{
		case FSL_INPUT_KEYPRESS: printf("Key Press"); break;
		case FSL_INPUT_KEYRELEASE: printf("Key Release"); break;
		case FSL_INPUT_POINTERDOWN: printf("Pointer Down"); break;
		case FSL_INPUT_POINTERUP: printf("Pointer Up"); break;
		case FSL_INPUT_POINTERMOVE: printf("Pointer Move"); break;
		default: printf("UNHANDLED\n");
	}
	*/
	
	//Convert click coords to -1 to 1 X,Y 3D coords
	vecA[0] = ((float) event.signalA * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[1] = ((float) event.signalB * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	
	//convert from screen space to card space
	fslMultMatrix4x4Vec4x1 ( currentInvProjMatrix_, vecA, vecC );
	
	//Debug Info
	//printf(" [%u,%u] \n",event.signalA,event.signalB);
	//printf(" pretransformed vec = [%f,%f,%f,%f] \n",vecA[0],vecA[1],vecA[2],vecA[3]);		
	//printf(" invtransformed vec = [%f,%f,%f,%f] \n",vecC[0],vecC[1],vecC[2],vecC[3]);	
	
	//Make sure the click falls in the bounds of the card
	//(event.input == FSL_INPUT_POINTERDOWN) && 
	if( (vecC[0] <= 1.0) && (vecC[0] >= -1.0)
		&& (vecC[1] <= 1.0) && (vecC[1] >= -1.0))
	{
		//Convert from -1 to 1 float to 0 to 1 uint
		event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
		event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX);  
		
		//printf( "Passing [%u,%u] to Card\n", event.signalA, event.signalB);
		
		myCard_.handleInputEvent( event );  
	 	
	}
	
	return FSL_STATUS_SUCCESS;
}

static char* g_strVSPre = "uniform   mat4 g_matProj;							\n";

static char* g_strVSPost = "    gl_Position  = g_matProj * cardPosition;		\n";

static char* g_strPSPre = "												 \n";

static char* g_strPSPost = "    gl_FragColor = cardColor;	\n";

fslStatus fslExampleScene::initScene()  //GL state inititization
{
	fslPhaseElementType phaseArray[MAX_PHASE_ARRAY_SIZE];

	myCard_.getRenderPhases( phaseArray );

	//I know there is only one phase for this card
	//More dynamic scenes would need to parse the whole array
	examplePhase_ = phaseArray[0].phaseID; 

	//Set our shader contibution
	myCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	
	//Compile
	if( myCard_.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	shaderProgram_[0] = myCard_.getShaderProgram(0);
	
	//Set any attribute handles needed
	// There are none for this scene
	
	//Link
	if( myCard_.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	//Get uniform handles
	projMatrixLoc_[0] = glGetUniformLocation( shaderProgram_[0], "g_matProj" );
	
	//Rinse, repeat for extra stages, reflections, etc.  Make sure and increase the config parameter for init
	
	return FSL_STATUS_SUCCESS;
}

fslBool fslExampleScene::isOnStage()
{
	return isVisible_;
}

fslStatus fslExampleScene::animResolve( float deltaTime )
{
	//animAngle += ( 0.5 * deltaTime );
	animAngle += 0.01;  //Something is wrong with the time function, this is temporary
	
	//No animation in this example scene
	//Normally, the scene would then call the animTick functions of the cards
	return FSL_STATUS_SUCCESS;
}

void fslExampleScene::receiveCardEvent( unsigned int eventID, void* data )
{
	if(eventID == 1001)
		printf("Card requests file open : %s\n",(char*) data);
	//Child Cards call this function on events
}

