
/* File: fslmusiccontainerscene.cpp
 *
 * Desc: This scene contains the music player. It is composed by the following:
 *	fslMusicTileCard[4] - One for each cd album that form the carrousel
 *	fslPlaybackControlCard - The buttons to control playback
 *	fslMusicVisualizerCard - The particle based animation being displayed
 *				 in the background
 *	
 * Copyright © 2010 Freescale Semiconductor
 */


#include <CARD/fslmusiccontainerscene.h>

//Pre and post strings with shader code unique to this scene to be appended to
//this scene's cards

static char* g_strVSPre =  
		"	uniform   mat4 g_matProj;			\n";
static char* g_strVSPost = 
		"	gl_Position  = g_matProj * cardPosition;	\n";

static char* g_strVSPrePc =  
		"	uniform   mat4 g_matProj;			\n"
		"	uniform   mat4 gc_matModelView;			\n";
static char* g_strVSPostPc = 
		"	cardPosition = gc_matModelView * cardPosition;	\n"
		"	gl_Position  = g_matProj * cardPosition;	\n";
		
static char* g_strPSPre =  "						\n";
static char* g_strPSPost = "    gl_FragColor = cardColor;		\n";

int currentCase = 0;
int nextCase = 0;
int savedCase = 0;

//--------------------------------------------------------------------------------------
// Name: fslMusicContainerScene
// Desc: Constructor; initializes cards, matrices and some flag variables used to
// 	 track the state of the player.
//--------------------------------------------------------------------------------------
fslMusicContainerScene::fslMusicContainerScene( fslScene* parentScene )
{
	right = left = false;
	parentScene_ = parentScene;

	for(int i=0; i < MAX_CARDS_IN_SCENE; i++)
		cardList_[i] = NULL;

	for(int i=0; i < MAX_SCENES_IN_SCENE; i++)
		sceneList_[i] = NULL;

	numCards_ = 0; 
	numScenes_ = 0;

	newCardID_ = 0;
	orientation_ = FSL_LANDSCAPE;
	isVisible_ = FSL_FALSE;

	cdCases_[0].setParentScene(this);
	cdCases_[1].setParentScene(this);
	cdCases_[2].setParentScene(this);
	cdCases_[3].setParentScene(this);

	myPlaybackCard_.setParentScene(this);

	visualizerCard_.setParentScene(this);
	
	//base value to keep track of events from card
	// i.e if between 1000 - 1999, then it is from this card
	//Only really valuable when tracking multiple cards
	cdCases_[0].setBaseEventID( 1000 ); 
	cdCases_[1].setBaseEventID( 2000 ); 
	cdCases_[2].setBaseEventID( 3000 ); 
	cdCases_[3].setBaseEventID( 4000 ); 

	myPlaybackCard_.setBaseEventID( 5000 );
	visualizerCard_.setBaseEventID( 6000 );
	
	modelMatrixcd0Dirty_ = FSL_TRUE;
	modelMatrixcd1Dirty_ = FSL_TRUE;
	modelMatrixcd2Dirty_ = FSL_TRUE;
	modelMatrixcd3Dirty_ = FSL_TRUE;
	modelMatrixControlDirty_ = FSL_TRUE;
	modelVisualizerDirty_    = FSL_TRUE;
	
	fslLoadIdentityMatrix4x4(modelMatrixcd0_);
	fslLoadIdentityMatrix4x4(modelMatrixcd1_);
	fslLoadIdentityMatrix4x4(modelMatrixcd2_);
	fslLoadIdentityMatrix4x4(modelMatrixcd3_);
	fslLoadIdentityMatrix4x4(modelMatrixcd3_);

	fslLoadIdentityMatrix4x4(modelMatrixControl_);

	fslLoadIdentityMatrix4x4(modelVisualizer_);
	
	fslLoadIdentityMatrix4x4(rotationM_);

	isRunning_ = true;
	return;
}


//--------------------------------------------------------------------------------------
// Name: ~fslMusicContainerScene
// Desc: Destructor; in case some resources are assigned dynamically, release them here.
//--------------------------------------------------------------------------------------
fslMusicContainerScene::~fslMusicContainerScene()
{
	return;
}


//--------------------------------------------------------------------------------------
// Name: setAspectRatio
// Desc: accessor method to se the scene's aspect ratio.
//--------------------------------------------------------------------------------------	
fslStatus fslMusicContainerScene::setAspectRatio( float aspect )
{
	//There is no big impact to this card for changing the aspect ratio
	aspectRatio_ = aspect;
	return FSL_STATUS_SUCCESS;
}


//--------------------------------------------------------------------------------------
// Name: setOrientation
// Desc: accessor method to se the scene's orientation.
//--------------------------------------------------------------------------------------	
fslStatus fslMusicContainerScene::setOrientation( fslOrientationType orientation )
{
	//There is no big impact to this card for changing the orientation
	orientation_ = orientation;
	return FSL_STATUS_SUCCESS;
}


//--------------------------------------------------------------------------------------
// Name: transitionOnStage
// Desc: called whenever the scene enters the application space.
//--------------------------------------------------------------------------------------
fslStatus fslMusicContainerScene::transitionOnStage( fslTransitionType transition)
{
	tileTransitionOnStage(transition);
	visualizerCard_.resetVisualizer();
	
	return FSL_STATUS_SUCCESS;
}


//--------------------------------------------------------------------------------------
// Name: tileTransitionOnStage
// Desc: called whenever the tiles enter the application space.
//	 (called by transitionOnStage)
//--------------------------------------------------------------------------------------
void fslMusicContainerScene::tileTransitionOnStage( fslTransitionType transition)
{
	goingAway_ = false;
	isVisible_ = true;
	if (sceneHidden_)
	{
	 	float backSpeed = 0.1f;	
	 	static float transCounterBack = 0.0f;
	 
	 	fslTranslateMatrix4x4(modelMatrixControl_,0,backSpeed,0);
	 
	 	fslTranslateMatrix4x4(modelMatrixcd0_,0,-backSpeed*2,0);
	 	fslTranslateMatrix4x4(modelMatrixcd1_,0,-backSpeed*2,0);
	 	fslTranslateMatrix4x4(modelMatrixcd2_,0,-backSpeed*2,0);
	 	fslTranslateMatrix4x4(modelMatrixcd3_,0,-backSpeed*2,0);
	 
	 	transCounterBack += 0.1f;
	 	if(transCounterBack > 3.7f)
	 	{
// 	 		currentCase = savedCase;
			transCounterBack = 0.0f;
	 		showScene_ = true;
	 		sceneHidden_ = false;
			startTimeOnScreen_ = fslGetTickCount();
	 	}
	}
}


//--------------------------------------------------------------------------------------
// Name: transitionOffStage
// Desc: called whenever the scene leaves the application space.
//--------------------------------------------------------------------------------------
fslStatus fslMusicContainerScene::transitionOffStage( fslTransitionType transition)
{
	tileTransitionOffStage(transition);
	visualizerCard_.setPipelineState(5);

}


//--------------------------------------------------------------------------------------
// Name: tileTransitionOffStage
// Desc: called whenever the tiles leave the application space 
//	 (called by transitionOffStage)
//--------------------------------------------------------------------------------------
void fslMusicContainerScene::tileTransitionOffStage( fslTransitionType transition)
{
	goingAway_ = true;
	timeDifference_ = 4000;
	if(!sceneHidden_)
	{
		float awaySpeed = 0.1f;	
		static float transCounter = 0.0f;
	// 	savedCase = currentCase;
	// 	currentCase = 40;
	
		fslTranslateMatrix4x4(modelMatrixControl_,0,-awaySpeed,0);
	
		fslTranslateMatrix4x4(modelMatrixcd0_,0,awaySpeed*2,0);
		fslTranslateMatrix4x4(modelMatrixcd1_,0,awaySpeed*2,0);
		fslTranslateMatrix4x4(modelMatrixcd2_,0,awaySpeed*2,0);
		fslTranslateMatrix4x4(modelMatrixcd3_,0,awaySpeed*2,0);
	
		transCounter += 0.1f;
		if(transCounter > 3.7f)
		{
			transCounter = 0.0f;
			showScene_ = false;
			sceneHidden_ = true;
			//isVisible_ = false;
		}
	
	}
}


//--------------------------------------------------------------------------------------
// Name: renderScene
// Desc: calls each card's render method in the correct drawing order depending on the
//	 scene's current state.
//--------------------------------------------------------------------------------------
fslStatus fslMusicContainerScene::renderScene( fslPhaseType onlyThese)
{
	if(isVisible_ != FSL_TRUE)
		return FSL_STATUS_SUCCESS;
	if(showScene_)
	{
		tileTransitionOnStage(FSL_TRANSITION_UNSPECIFIED);
		timeDifference_ = fslGetTickCount() - startTimeOnScreen_;
		if (timeDifference_ > 8000) //3 secs
		{
		tileTransitionOffStage(FSL_TRANSITION_UNSPECIFIED);
		}
	}
	if(!sceneHidden_ && !showScene_)
	{
		tileTransitionOffStage(FSL_TRANSITION_UNSPECIFIED);
		
	}


	if(onlyThese == FSL_PHASE_RENDER_WRITEONLY || onlyThese == FSL_PHASE_UNSPECIFIED)
	{
		glUseProgram( shaderProgram_[5] );
			glUniformMatrix4fv( projMatrixLoc_[5], 1, 0, projMatrix_ );
			glUniformMatrix4fv( modelMatrixLoc_[5], 1, 0, modelVisualizer_ );
			visualizerCard_.renderCard(phases_[0], 5);

		//set attribs and uniforms
		glUseProgram( shaderProgram_[0] );

			glUniformMatrix4fv( rotationMatrixLoc_, 1, 0, rotationM_ );

			for (int i = 0;i<4 ;i++)
			{
			glUniformMatrix4fv( projMatrixLoc_[drawOrder_[i]], 1, 0, projMatrix_ );
			glUniformMatrix4fv( modelMatrixLoc_[drawOrder_[i]], 1, 0, cdModelViews_[drawOrder_[i]] );
			cdCases_[drawOrder_[i]].renderCard(phases_[0], drawOrder_[i]);
			cdCases_[drawOrder_[i]].renderCard(phases_[1], drawOrder_[i]);
			}
		glUseProgram( shaderProgram_[4] );

			glUniformMatrix4fv( projMatrixLoc_[4], 1, 0, projMatrix_ );
			glUniformMatrix4fv( modelMatrixLoc_[4], 1, 0, modelMatrixControl_ );
			myPlaybackCard_.renderCard(phases_[0], 4);
		return FSL_STATUS_SUCCESS;
		
	}

	return FSL_STATUS_SUCCESS;
}


//--------------------------------------------------------------------------------------
// Name: initScene
// Desc: called once after the constructor from the parent scene. Initialices its cards,
//	 attributes, builds and compiles the card's shaders and prepares the card's
//	 matrices by giving them a starting location.
//--------------------------------------------------------------------------------------
fslStatus fslMusicContainerScene::initScene()  //GL state inititization
{
	goingAway_ = false;
	showScene_ = false;
	sceneHidden_ = false;
	timeDifference_ = 4000;
	startTimeOnScreen_ = 0;
	fslPhaseElementType phaseArray[MAX_PHASE_ARRAY_SIZE];
	cdCases_[0].getRenderPhases( phaseArray );
 	//myPlaybackCard_.getRenderPhases( phaseArray ); ///this should use a separate array
	
	//Set projection matrix to Identity Matrix
	fslLoadIdentityMatrix4x4(projMatrix_);

	cdCases_[0].setTextureFileName("media/textures/cdCase.CTES",26);
	cdCases_[1].setTextureFileName("media/textures/case1_512.CTES",29);
	cdCases_[2].setTextureFileName("media/textures/case2_512.CTES",29);
	cdCases_[3].setTextureFileName("media/textures/case3_512.CTES",29);

	drawOrder_[0] = 0;
	drawOrder_[1] = 2;
	drawOrder_[2] = 1;
	drawOrder_[3] = 3;

	cdModelViews_[0] = modelMatrixcd0_;
	cdModelViews_[1] = modelMatrixcd1_;
	cdModelViews_[2] = modelMatrixcd2_;
	cdModelViews_[3] = modelMatrixcd3_;

	cdCases_[0].setSongNumber(0);
	cdCases_[1].setSongNumber(1);
	cdCases_[2].setSongNumber(2);
	cdCases_[3].setSongNumber(3);

	//set perspective projection matrix
	//should be aspectRatio_, not sure if that works, so hard coded in the meantime
	fslPerspectiveMatrix4x4(projMatrix_, 50.0f, 1.3333f, 1.f, 100.0f);
	//I know there is only one phase for this card
	//More dynamic scenes would need to parse the whole array
	phases_[0] = phaseArray[0].phaseID;
	phases_[1] = phaseArray[1].phaseID;

	//Set our shader contibution
	cdCases_[0].setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	cdCases_[0].setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	cdCases_[1].setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	cdCases_[1].setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	cdCases_[2].setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	cdCases_[2].setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	cdCases_[3].setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	cdCases_[3].setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	
	myPlaybackCard_.setVertexShaderAppendStrings( g_strVSPrePc, g_strVSPostPc );
	myPlaybackCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	visualizerCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	visualizerCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	//Compile
	printf("about to compile case0 shader \n");
	if( cdCases_[0].initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	//Compile
	printf("about to compile case1 shader \n");
	if( cdCases_[1].initCard(FSL_INIT_COMPILE, 1) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	//Compile
	printf("about to compile case2 shader \n");
	if( cdCases_[2].initCard(FSL_INIT_COMPILE, 2) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	//Compile
	printf("about to compile case3 shader \n");
	if( cdCases_[3].initCard(FSL_INIT_COMPILE, 3) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
		//Compile
	printf("about to compile playbackcard shader \n");
	if( myPlaybackCard_.initCard(FSL_INIT_COMPILE, 4) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	printf("about to compile visualizercard shader \n");
	if( visualizerCard_.initCard(FSL_INIT_COMPILE, 5) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
		
	
	//Set any attribute handles needed
	// There are none for this scene
	
	//Link
	if( cdCases_[0].initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	if( cdCases_[1].initCard(FSL_INIT_LINK, 1) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	if( cdCases_[2].initCard(FSL_INIT_LINK, 2) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	if( cdCases_[3].initCard(FSL_INIT_LINK, 3) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	if( myPlaybackCard_.initCard(FSL_INIT_LINK, 4) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	if( visualizerCard_.initCard(FSL_INIT_LINK, 5) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	shaderProgram_[0] = cdCases_[0].getShaderProgram(0);
	shaderProgram_[1] = cdCases_[1].getShaderProgram(1);
	shaderProgram_[2] = cdCases_[2].getShaderProgram(2);
	shaderProgram_[3] = cdCases_[3].getShaderProgram(3);

	shaderProgram_[4] = myPlaybackCard_.getShaderProgram(4);

	shaderProgram_[5] = visualizerCard_.getShaderProgram(5);

	//Get uniform handles
	projMatrixLoc_[0] = glGetUniformLocation( shaderProgram_[0], "g_matProj" );
	projMatrixLoc_[1] = glGetUniformLocation( shaderProgram_[1], "g_matProj" );
	projMatrixLoc_[2] = glGetUniformLocation( shaderProgram_[2], "g_matProj" );
	projMatrixLoc_[3] = glGetUniformLocation( shaderProgram_[3], "g_matProj" );
	projMatrixLoc_[4] = glGetUniformLocation( shaderProgram_[4], "g_matProj" );
	projMatrixLoc_[5] = glGetUniformLocation( shaderProgram_[5], "g_matProj" );


	modelMatrixLoc_[0] = glGetUniformLocation( shaderProgram_[0], "gc_matModelView" );
	modelMatrixLoc_[1] = glGetUniformLocation( shaderProgram_[1], "gc_matModelView" );
	modelMatrixLoc_[2] = glGetUniformLocation( shaderProgram_[2], "gc_matModelView" );
	modelMatrixLoc_[3] = glGetUniformLocation( shaderProgram_[3], "gc_matModelView" );
	modelMatrixLoc_[4] = glGetUniformLocation( shaderProgram_[4], "gc_matModelView" );
	modelMatrixLoc_[5] = glGetUniformLocation( shaderProgram_[5], "gc_matModelView" );

	rotationMatrixLoc_ = glGetUniformLocation( shaderProgram_[0], "gc_matRotation" );

	cdCases_[0].EnableReflection();
	cdCases_[1].EnableReflection();
	cdCases_[2].EnableReflection();
	cdCases_[3].EnableReflection();


	fslTranslateMatrix4x4(modelMatrixControl_,0,-.35f,-1.f);
	fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
	fslTranslateMatrix4x4(modelVisualizer_,0.5f,1.0f,-9.0f);

	fslTranslateMatrix4x4(modelMatrixcd0_,-0,-.5f,-3.f);
	fslTranslateMatrix4x4(modelMatrixcd1_,-1.5f,-.5f,-4.5f);
	fslTranslateMatrix4x4(modelMatrixcd2_,1.5f,-.5f,-4.5f);
	fslTranslateMatrix4x4(modelMatrixcd3_,0,-.5f,-6.f);

	//Rinse, repeat for extra stages, reflections, etc.  Make sure and increase the config parameter for init
	//initial state change for animation
	return FSL_STATUS_SUCCESS;
}


//--------------------------------------------------------------------------------------
// Name: isOnStage
// Desc: returns true if the scene is currently showing, false otherwise.
//--------------------------------------------------------------------------------------
fslBool fslMusicContainerScene::isOnStage()
{
	if(!showScene_ && sceneHidden_)
		return FSL_TRUE;
	else
		return FSL_FALSE;
}


//--------------------------------------------------------------------------------------
// Name: animResolve
// Desc: Rotates the cdCards (tiles) according to the current state of the carrousel
//	 and assigns new values to the tile's drawing order, and depending on the 
//	 direction clocked.
//--------------------------------------------------------------------------------------
fslStatus fslMusicContainerScene::animResolve( float deltaTime )
{
	float speed = .1f;
	static float z = 0.f;
	static float w = -1.5f;
	if (showScene_ && !sceneHidden_ && timeDifference_<3000)
	{
	switch (currentCase)
	{
		case 0:
				fslLoadIdentityMatrix4x4(modelMatrixcd0_);//case0
				fslTranslateMatrix4x4(modelMatrixcd0_,0,-.5f,-3.f);
				fslLoadIdentityMatrix4x4(modelMatrixcd1_);
				fslTranslateMatrix4x4(modelMatrixcd1_,-1.5f,-.5f,-4.5f);
				fslLoadIdentityMatrix4x4(modelMatrixcd2_);
				fslTranslateMatrix4x4(modelMatrixcd2_,1.5f,-.5f,-4.5f);
				fslLoadIdentityMatrix4x4(modelMatrixcd3_);
				fslTranslateMatrix4x4(modelMatrixcd3_,0,-.5f,-6.f);
			if (right)
			{
				
				fslTranslateMatrix4x4(modelMatrixcd0_,sqrt((-(z*z))-(3.0f*z)),0,z);
				fslTranslateMatrix4x4(modelMatrixcd1_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd3_,-sqrt((-(z*z))-(3.0f*z)),0,-z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {z = 0; w = -1.5f; right = false; currentCase = nextCase;//case1
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,0,-.5f,-3.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,0,-.5f,-6.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,-1.5f,-.5f,-4.5f);
						drawOrder_[0] = 1;
						drawOrder_[1] = 0;
						drawOrder_[2] = 3;
						drawOrder_[3] = 2;
				}
			}
			
			if (left)
			{

				fslTranslateMatrix4x4(modelMatrixcd0_,-sqrt((-(z*z))-(3.0f*z)),0,z);
				fslTranslateMatrix4x4(modelMatrixcd1_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd3_,sqrt((-(z*z))-(3.0f*z)),0,-z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {
					z = 0; w=-1.5f; left = false; currentCase = nextCase;//case3
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,-1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,0,-.5f,-6.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,0,-.5f,-3.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,1.5f,-.5f,-4.5f);
						drawOrder_[0] = 2;
						drawOrder_[1] = 3;
						drawOrder_[2] = 0;
						drawOrder_[3] = 1;
				}
			}

		break;
		case 1:
				fslLoadIdentityMatrix4x4(modelMatrixcd0_);
				fslTranslateMatrix4x4(modelMatrixcd0_,1.5f,-.5f,-4.5f);
				fslLoadIdentityMatrix4x4(modelMatrixcd1_);
				fslTranslateMatrix4x4(modelMatrixcd1_,0,-.5f,-3.f);
				fslLoadIdentityMatrix4x4(modelMatrixcd2_);
				fslTranslateMatrix4x4(modelMatrixcd2_,0,-.5f,-6.f);
				fslLoadIdentityMatrix4x4(modelMatrixcd3_);
				fslTranslateMatrix4x4(modelMatrixcd3_,-1.5f,-.5f,-4.5f);
			if (right)
			{
				
				fslTranslateMatrix4x4(modelMatrixcd1_,sqrt((-(z*z))-(3.0f*z)),0,z);
				fslTranslateMatrix4x4(modelMatrixcd3_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd0_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,-sqrt((-(z*z))-(3.0f*z)),0,-z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {z = 0; w=-1.5f;; right = false; currentCase = nextCase; //case2
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,0,-.5f,-6.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,-1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,0,-.5f,-3.f);
						drawOrder_[0] = 3;
						drawOrder_[1] = 1;
						drawOrder_[2] = 2;
						drawOrder_[3] = 0;
				}
			}
			
			if (left)
			{

				fslTranslateMatrix4x4(modelMatrixcd1_,-sqrt((-(z*z))-(3.0f*z)),0,z);
				fslTranslateMatrix4x4(modelMatrixcd3_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd0_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,sqrt((-(z*z))-(3.0f*z)),0,-z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {
					z = 0; w=-1.5f; left = false; currentCase = nextCase;
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,-0.f,-.5f,-3.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,-1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,0.f,-.5f,-6.f);
						drawOrder_[0] = 0;
						drawOrder_[1] = 2;
						drawOrder_[2] = 1;
						drawOrder_[3] = 3;
				}
			}
		break;
		case 2:
			fslLoadIdentityMatrix4x4(modelMatrixcd0_);
			fslTranslateMatrix4x4(modelMatrixcd0_,0,-.5f,-6.f);
			fslLoadIdentityMatrix4x4(modelMatrixcd1_);
			fslTranslateMatrix4x4(modelMatrixcd1_,1.5f,-.5f,-4.5f);
			fslLoadIdentityMatrix4x4(modelMatrixcd2_);
			fslTranslateMatrix4x4(modelMatrixcd2_,-1.5f,-.5f,-4.5f);
			fslLoadIdentityMatrix4x4(modelMatrixcd3_);
			fslTranslateMatrix4x4(modelMatrixcd3_,0,-.5f,-3.f);
			if (right)
			{
				
				fslTranslateMatrix4x4(modelMatrixcd0_,-sqrt((-(z*z))-(3.0f*z)),0,-z);
				fslTranslateMatrix4x4(modelMatrixcd1_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd3_,sqrt((-(z*z))-(3.0f*z)),0,z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {z = 0; w=-1.5f; right = false; currentCase = nextCase; //case2
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,-1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,0.f,-.5f,-6.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,0.f,-.5f,-3.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,1.5f,-.5f,-4.5f);
					drawOrder_[0] = 2;
					drawOrder_[1] = 3;
					drawOrder_[2] = 0;
					drawOrder_[3] = 1;
				}
			}
			
			if (left)
			{

				fslTranslateMatrix4x4(modelMatrixcd0_,sqrt((-(z*z))-(3.0f*z)),0,-z);
				fslTranslateMatrix4x4(modelMatrixcd1_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd3_,-sqrt((-(z*z))-(3.0f*z)),0,z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {
					z = 0; w=-1.5f; left = false; currentCase = nextCase;
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,0,-.5f,-3.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,0,-.5f,-6.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,-1.5f,-.5f,-4.5f);
					drawOrder_[0] = 1;
					drawOrder_[1] = 0;
					drawOrder_[2] = 3;
					drawOrder_[3] = 2;
				}
			}
		break;
		case 3:
				fslLoadIdentityMatrix4x4(modelMatrixcd0_);
				fslTranslateMatrix4x4(modelMatrixcd0_,-1.5f,-.5f,-4.5f);
				fslLoadIdentityMatrix4x4(modelMatrixcd1_);
				fslTranslateMatrix4x4(modelMatrixcd1_,0,-.5f,-6.f);
				fslLoadIdentityMatrix4x4(modelMatrixcd2_);
				fslTranslateMatrix4x4(modelMatrixcd2_,0,-.5f,-3.f);
				fslLoadIdentityMatrix4x4(modelMatrixcd3_);
				fslTranslateMatrix4x4(modelMatrixcd3_,1.5f,-.5f,-4.5f);
			if (right)
			{
				
				fslTranslateMatrix4x4(modelMatrixcd1_,-sqrt((-(z*z))-(3.0f*z)),0,-z);
				fslTranslateMatrix4x4(modelMatrixcd3_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd0_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,-w-1.5f);
				fslTranslateMatrix4x4(modelMatrixcd2_,sqrt((-(z*z))-(3.0f*z)),0,z);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {z = 0; w=-1.5f; right = false; currentCase = nextCase; //case2
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,0,-.5f,-3.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,-1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,0,-.5f,-6.f);
					drawOrder_[0] = 0;
					drawOrder_[1] = 2;
					drawOrder_[2] = 1;
					drawOrder_[3] = 3;
				}
			}
			
			if (left)
			{

				fslTranslateMatrix4x4(modelMatrixcd0_,-sqrt((-(w*w))-(3.0f*w))+1.5f,0,w+1.5f);
				fslTranslateMatrix4x4(modelMatrixcd1_,sqrt((-(z*z))-(3.0f*z)),0,-z);
				fslTranslateMatrix4x4(modelMatrixcd2_,-sqrt((-(z*z))-(3.0f*z)),0,z);
				fslTranslateMatrix4x4(modelMatrixcd3_,sqrt((-(w*w))-(3.0f*w))-1.5f,0,-w-1.5f);
				if(z>-1.5)
				{
				z-=speed;
				w-= speed;
				} else {
					z = 0; w=-1.5f; left = false; currentCase = nextCase;
					fslLoadIdentityMatrix4x4(modelMatrixcd0_);
					fslTranslateMatrix4x4(modelMatrixcd0_,-0.f,-.5f,-6.f);
					fslLoadIdentityMatrix4x4(modelMatrixcd1_);
					fslTranslateMatrix4x4(modelMatrixcd1_,1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd2_);
					fslTranslateMatrix4x4(modelMatrixcd2_,-1.5f,-.5f,-4.5f);
					fslLoadIdentityMatrix4x4(modelMatrixcd3_);
					fslTranslateMatrix4x4(modelMatrixcd3_,0.f,-.5f,-3.f);
					drawOrder_[0] = 3;
					drawOrder_[1] = 1;
					drawOrder_[2] = 2;
					drawOrder_[3] = 0;
				}
			}
		break;
		default:
		break;
	}
	}
	visualizerCard_.animTick();
	return FSL_STATUS_SUCCESS;
}


//--------------------------------------------------------------------------------------
// Name: receiveCardEvent
// Desc: Assigns state attributes depending on the playbackControl's button clicked
//--------------------------------------------------------------------------------------
void fslMusicContainerScene::receiveCardEvent( unsigned int eventID, void* data )
{

	switch (eventID)
	{
		case 5000:
			printf("rew\n");
			startTimeOnScreen_ = fslGetTickCount();
			right = true;
			left = false;
			currentCase==3? nextCase=0: nextCase++;
			visualizerCard_.setPipelineState(0);
		break;
		case 5001:
			startTimeOnScreen_ = fslGetTickCount();
			visualizerCard_.setPipelineState(1);
			printf("pause\n");
		break;
		case 5002:
			startTimeOnScreen_ = fslGetTickCount();
			if(currentCase == 0)
			printf("play %i\n", cdCases_[0].getSongNumber());
			if(currentCase == 1)
			printf("play %i\n", cdCases_[1].getSongNumber());
			if(currentCase == 2)
			printf("play %i\n", cdCases_[3].getSongNumber());
			if(currentCase == 3)
			printf("play %i\n", cdCases_[2].getSongNumber());
			visualizerCard_.setPipelineState(2);
		break;
		case 5003:
			startTimeOnScreen_ = fslGetTickCount();
			printf("stop\n");
			visualizerCard_.setPipelineState(3);
		break;
		case 5004:
			startTimeOnScreen_ = fslGetTickCount();
			left = true;
			right = false;
			currentCase==0? nextCase=3: nextCase--;
			visualizerCard_.setPipelineState(4);
			printf("fwd\n");
		break;
		default: break;
	}


}


//--------------------------------------------------------------------------------------
// Name: handleInputEvent
// Desc: called whenever an event is detected on this scene, transforms the event's 
//	 (x,y) coordinates to the playbackControl card space, then sends it to the card
//	 if it landed within its limits
//--------------------------------------------------------------------------------------
fslStatus fslMusicContainerScene::handleInputEvent(fslInputEventType event)
{

	float vecA[4] = {0};
	float vecC[4] = {0};
	
	//Convert click coords to -1 to 1 X,Y 3D coords
	vecA[0] = ((float) event.signalA * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[1] = ((float) event.signalB * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[2] = -1.0f; 
	vecA[3] = 1.0f; 
	
	if( event.input == FSL_INPUT_POINTERDOWN )
	{
		showScene_=true;
		if(!left && !right && !goingAway_)
		{
			float controlMatrix[16];
			float controlMatrixInv[16];	
			
			fslMultMatrix4x4( controlMatrix, projMatrix_, modelMatrixControl_);
			fslInvertMatrix4x4( controlMatrix, controlMatrixInv);
			fslMultMatrix4x4Vec4x1 ( controlMatrixInv, vecA, vecC );
			printf("control out = %f, %f \n", vecC[0], vecC[1]);
			
			if( (vecC[0] <= 1.0) && (vecC[0] >= -1.0)
				&& (vecC[1] <= -0.45f) && (vecC[1] >= -0.83f))
			{
				printf("musiccontainerscene: Playback hit detected!\n");
				event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
				event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX); 
				myPlaybackCard_.handleInputEvent( event ); 
				return FSL_STATUS_SUCCESS;
			}
		}
	
	}

	else if(event.input == FSL_INPUT_POINTERUP)
	{
		myPlaybackCard_.handleInputEvent(event);
	}
 	if (event.input == FSL_INPUT_CLOSEWINDOW)
 	{
 		isRunning_ = false;
 	}

	return FSL_STATUS_SUCCESS;
}

