
/* File: cardsample.c
 * 
 * Desc: Mary had a little lamb.  Its fleece was scratchy and uncomfortable.
 * 			
 * Copyright  2009 Freescale Semiconductor
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <fcntl.h> 
#include <malloc.h> 
#include <math.h> 
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
 
#ifndef BOOL
#define BOOL int
#define TRUE 1
#define FALSE !TRUE
#endif

#include <GLES2/gl2.h>
#define GL_GLEXT_PROTOTYPES 1
#include <GLES2/gl2ext.h>
#include <GLES2/gl2amdext.h>
#include <EGL/egl.h>
#define EGL_EGLEXT_PROTOTYPES 1
#include <EGL/eglext.h>

#ifdef EGL_USE_X11

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/render.h>
#include <X11/extensions/Xrender.h>

#endif

#include <FSL/fslutil.h>
 
#include <CARD/fslcard.h>
#include <CARD/fslmusictilecard.h>

//--------------------------------------------------------------------------------------
// Types
//--------------------------------------------------------------------------------------

typedef struct XWindow_s {
	Display *display;
	Window window;
	int screen;
	Visual *visual;
	GC gc;
	Colormap colormap;
	int depth;
	int pixelsize;
	XFontStruct *font_info;
	XWindowAttributes attr;
	Picture picture;
	Pixmap pixmap;
} XWindow;


//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

EGLDisplay		g_egldisplay;
EGLConfig		g_eglconfig;
EGLSurface		g_eglsurface;
EGLContext		g_eglcontext;

unsigned int	g_currentFrame = 0;
unsigned int	g_shaderProgram[2]; 
unsigned int	g_examplePhase[2]; 
float 		g_animAngle = 0.0f;
int		g_projMatrixLoc[2];
fslMusicTileCard  g_myCard( NULL );


//--------------------------------------------------------------------------------------
// Shader definitions
//--------------------------------------------------------------------------------------
static char* g_strVSPre = "uniform   mat4 g_matProj;							\n";

static char* g_strVSPost = "    gl_Position  = g_matProj * cardPosition;		\n";

static char* g_strPSPre = "												 \n";

static char* g_strPSPost = "    gl_FragColor = cardColor;	\n";

//--------------------------------------------------------------------------------------
// Macro constants
//--------------------------------------------------------------------------------------

#define PRINT_FRAMERATE 0

//--------------------------------------------------------------------------------------
// Name: initGL
// Desc: Sets up GL variables and state before rendering.  Sets up EGLImage from X11 pixmap
//--------------------------------------------------------------------------------------
BOOL initGL( )
{
	printf("+Starting GL Initialization: \n");

	//There is no master scene in this example, so we are handling the init
	
	fslPhaseElementType phaseArray[MAX_PHASE_ARRAY_SIZE];

	g_myCard.getRenderPhases( phaseArray );

	//Start by testing only one phase
	g_examplePhase[0] = phaseArray[0].phaseID; 
	g_examplePhase[1] = phaseArray[1].phaseID; 

	//Set our shader contibution
	g_myCard.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	g_myCard.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	g_myCard.setTextureFileName("cdCase.CTES", 12);

        printf("Compile stage\n");
	//Compile
	if( g_myCard.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
	{
	        printf("Compile stage  -- PASS\n");

		return FSL_STATUS_GENERAL_ERROR;
	}

        printf("Compile stage  -- PASS\n");
	
	g_shaderProgram[0] = g_myCard.getShaderProgram(0);
	
	//Set any attribute handles needed
	// There are none for this scene
        printf("Link stage\n");
	
	//Link
	if( g_myCard.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	//Get uniform handles
	g_projMatrixLoc[0] = glGetUniformLocation( g_shaderProgram[0], "g_matProj" );
	
	printf("-Finished GL Initialization\n");

	return TRUE;
}

//--------------------------------------------------------------------------------------
// Name: cleanupGL()
// Desc: cleanup any objects created in initGL
//--------------------------------------------------------------------------------------
void cleanupGL( )
{
	printf("Cleaning Up GL\n");
	//Nothing yet to see... move along
}

//--------------------------------------------------------------------------------------
// Name: render
// Desc: Render the scene
//--------------------------------------------------------------------------------------
void render(int width, int height)
{
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	//printf("Clearing..\n");
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	
	
	float matProjMatrix[16] = {0};
	

		
		//Set Identity Matrix
	matProjMatrix[ 0] = 1.0f;
	matProjMatrix[ 5] = 1.0f;
	matProjMatrix[10] = 1.0f;
	matProjMatrix[15] = 1.0f;	
		
	//Rotation
	matProjMatrix[ 0] = +cos(g_animAngle );
        matProjMatrix[ 3] = +sin(g_animAngle );
        matProjMatrix[ 8] = -sin(g_animAngle );
        matProjMatrix[10] = +cos(g_animAngle );
		
	//Master Scene (or application) handles the clears
	//Scene handles other state setup such as:
	glDisable(GL_DEPTH_TEST);
	
	//set attribs and uniforms
	glUniformMatrix4fv( g_projMatrixLoc[0], 1, 0, matProjMatrix );
	
	//Render the card
	g_myCard.renderCard(g_examplePhase[0], 0);
	g_myCard.renderCard(g_examplePhase[1], 0);
	
	//printf("Swapping..\n");
	eglSwapBuffers(g_egldisplay, g_eglsurface);
}

//--------------------------------------------------------------------------------------
// Name: initEGL()
// Desc: Initialize the X window and EGL objects / state need draw to screen
//--------------------------------------------------------------------------------------

BOOL initEGL(XWindow* xwin)
{
	static const EGLint configAttribs16[] =
	{
			EGL_RED_SIZE,           5,
			EGL_GREEN_SIZE,         6,
			EGL_BLUE_SIZE,          5,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,
			EGL_SAMPLES,            0,
			EGL_NONE
	};

	static const EGLint configAttribs24[] =
	{
			EGL_RED_SIZE,           8,
			EGL_GREEN_SIZE,         8,
			EGL_BLUE_SIZE,          8,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,			
			EGL_SAMPLES,            0,
			EGL_NONE
	};

	EGLint	numconfigs 	= 0;
	unsigned int	width		= 640;
	unsigned int	height		= 480;

	unsigned int 	dummy, depth;
	int 		x;
	Window 		rootwindow;

	printf("+Starting Main Window X Initialization:\n");

	printf("\tXOpenDisplay [");
	xwin->display = XOpenDisplay(NULL);
	printf("%d]\n", (int)xwin->display);
	assert(xwin->display != NULL);

	printf("\tDefaultScreen [");
	xwin->screen = DefaultScreen(xwin->display);
	printf("%d]\n", xwin->screen);

	rootwindow = RootWindow(xwin->display,xwin->screen);

	printf("\tXCreateSimpleWindow \n");
	xwin->window = XCreateSimpleWindow(xwin->display, rootwindow, 0, 0, width, height, 0, 0, WhitePixel (xwin->display, xwin->screen));
	
	XMapWindow(xwin->display, xwin->window);
	
	XStoreName(xwin->display,xwin->window,"Xcomposite -> Native Pixmap 3D Demo");

	{
		XSizeHints Hints;
		Hints.flags=PSize|PMinSize|PMaxSize;
		Hints.min_width=Hints.max_width=Hints.base_width=width;
		Hints.min_height=Hints.max_height=Hints.base_height=height;
		XSetWMNormalHints(xwin->display,xwin->window,&Hints);
	}	
	
	if(!XGetGeometry(xwin->display, xwin->window, &rootwindow, &x, &x, &width, &height, &dummy, &depth) || (depth == 0))
	{
		printf("\tERROR: X11 Window Creation Error\n");
		return FALSE;
	}
	
	printf("-Finished Main Window X Initialization:\n");
	
	printf("+Starting EGL Initialization:\n");

	//Initialize EGL
	printf("\teglGetDisplay \n");
	g_egldisplay = eglGetDisplay ( xwin->display );
	fslEGLCheck(TRUE);

	printf("\teglInitialize \n");
	eglInitialize(g_egldisplay, NULL, NULL);
	fslEGLCheck(TRUE);

	printf("\teglBindAPI \n");
	eglBindAPI(EGL_OPENGL_ES_API);

	printf("\teglChooseConfig \n");
	if(depth == 16)
		eglChooseConfig(g_egldisplay, configAttribs16, &g_eglconfig, 1, &numconfigs);
	else if(depth == 24)
		eglChooseConfig(g_egldisplay, configAttribs24, &g_eglconfig, 1, &numconfigs);
	else
	{
		printf("\tERROR: Only 16b and 24b windows supported \n");
		return FALSE;
	}


	fslEGLCheck(TRUE);

	if(numconfigs != 1)
	{
		printf("\tERROR: No EGL suitable configurations found. \n");
		return FALSE;
	}

	printf("\teglCreateWindowSurface \n");
	g_eglsurface = eglCreateWindowSurface(g_egldisplay, g_eglconfig, xwin->window, NULL);
	fslEGLCheck(TRUE);

	printf("\tglCreateContext \n");
	g_eglcontext = eglCreateContext(g_egldisplay, g_eglconfig, NULL, NULL);
	fslEGLCheck(TRUE);

	printf("\teglMakeCurrent \n");
	eglMakeCurrent(g_egldisplay, g_eglsurface, g_eglsurface, g_eglcontext);
	fslEGLCheck(TRUE);
	
	printf("-Finished EGL Initialization:\n");
	
	return TRUE;
}

//--------------------------------------------------------------------------------------
// Name: cleanupEGL
// Desc: Cleanup any objects created in initEGL
//--------------------------------------------------------------------------------------

void cleanupEGL(XWindow* xwin)
{
	printf("Cleaning Up EGL and Main Window\n");
	eglMakeCurrent(g_egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglTerminate(g_egldisplay);
    XCloseDisplay(xwin->display);
	eglReleaseThread();
}

//--------------------------------------------------------------------------------------
// Name: handleEvents
// Desc: Handle events from X: Needs to use OpenKODE
//--------------------------------------------------------------------------------------
BOOL handleEvents(XWindow *xwin)
{
	//TBD
	return TRUE;
}



//--------------------------------------------------------------------------------------
// Name: main
// Desc: main
//--------------------------------------------------------------------------------------
int main (void)
{
	XWindow	xMainWindow;
	EGLint 	width = 0, height = 0;
#ifdef PRINT_FRAMERATE	
	unsigned int	startTime = 0, endTime = 0;
#endif	

	
#ifndef EGL_USE_X11
#error Must be compiled with X11 support EGL_USE_X11
#endif
	
	printf("Freescale CD Card Test harness\n");
	printf("++Starting Initialization\n");
	
	//Set up error handler here
	
	assert( initEGL( &xMainWindow ) );

	assert( initGL( ) );
	
	eglQuerySurface(g_egldisplay, g_eglsurface, EGL_WIDTH, &width);
	eglQuerySurface(g_egldisplay, g_eglsurface, EGL_HEIGHT, &height);
	printf("--Finished Initializing (%d x %d).  Now Rendering. \n",width, height);

	
	while (g_currentFrame < 100)
	{
		eglQuerySurface(g_egldisplay, g_eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(g_egldisplay, g_eglsurface, EGL_HEIGHT, &height);

                if(g_currentFrame == 50)
			g_myCard.EnableReflection();

		render(width, height);

                g_animAngle+=3.14159/20;
		g_currentFrame++;

#ifdef PRINT_FRAMERATE
		// Printing FPS metric
		if(g_currentFrame % 100 == 0)
		{
			endTime = fslGetTickCount();
			if(startTime && startTime < endTime)
			{
				
				printf("\t%f FPS, %f ms/frame [%d] (%d x %d)\n", 
					100 * 1000.f / (endTime - startTime),  
					(float)(endTime - startTime)/(float)100, 
					g_currentFrame, width, height);
			
			}
			startTime = fslGetTickCount();
		}
#endif		
							
	}
	cleanupGL( );
	cleanupEGL( &xMainWindow );

	return 0;
}





