
#include <CARD/fslmusicvisualizercard.h>

/*MAX character length of shaders for memory reservation*/
#define MAX_SHADER_LENGTH 1000 

/*Thread for gstreamer loop that would contain the audio pipeline*/
GThread *gstLoop; 

/*Error for gstreamer Thread*/
GError	*err1 = NULL;

/*static variables from audioPipeline */
extern float speed; //contains audio speed
extern GstElement *audioPipeline;//containes audioPipeline gstreamer element
extern int goomEvent; //contains goomEvent
extern bool isAudioPlaying; //variable to determine if audio isplaying or not

/*array of songs to be played by the visualizer*/
gchar *playList[] = {"media/music/Kaleidoscope.mp3","media/music/James.mp3","media/music/Polichinelle.mp3","media/music/Lampi_e_Tuoni_a_Ciel_Sereno.mp3"};
int songNumber = 0;

/*strings cointaining shader programs code*/
static const char* g_strEnd =
    "}											\n";

/*variables used by Vertex Shader*/
static const char* g_strVSProgramA = 
    "uniform   mat4 gc_matModelView;							\n"
    "											\n"
    "attribute vec4  gc_vVertex;							\n"
    "attribute vec4  gc_vTexcoord;							\n"
    "attribute vec3  gc_vColor;								\n"
    "attribute float gc_vAlpha;								\n"
    "											\n"
    "varying   vec4  gc_vVSTexcoord;							\n"
    "varying   vec3  gc_vVSColor;							\n"
    "varying   float gc_vVSAlpha;							\n"
    "											\n";

/*Vertex shader main method*/
static const char* g_strVSProgramB = 
    "void main()									\n"
    "{											\n"	
    "    vec4 cardPosition = gc_matModelView * gc_vVertex;				\n"
    "    gc_vVSTexcoord = gc_vTexcoord;							\n"
    "    gc_vVSColor.xyz = gc_vColor.xyz;						\n"
    "    gc_vVSAlpha = gc_vAlpha;							\n";

/*Fragment shader variables*/
static const char* g_strFSProgramA = 
    "#ifdef GL_FRAGMENT_PRECISION_HIGH							\n"
    "   precision highp float;								\n"
    "#else										\n"
    "   precision mediump float;							\n"
    "#endif										\n"
    "											\n"
    "uniform   sampler2D gc_sImageTexture;						\n"
    "varying   vec4      gc_vVSTexcoord;						\n"
    "varying   vec3 	 gc_vVSColor;							\n"
    "varying   float 	 gc_vVSAlpha;							\n"
    "											\n";

/*Fragment shader main method*/
static const char* g_strFSProgramB = 
    "void main()									\n"
    "{											\n"
    "    vec4 cardColor = texture2D(gc_sImageTexture, gc_vVSTexcoord.xy);		\n"
    "    float colorFactor = (cardColor.r + cardColor.g +cardColor.b)/3.0; 		\n"
    "    float textureAlpha = 1.0; 							\n"
    "											\n"
    "	 cardColor.r = (gc_vVSColor.r*(colorFactor) + cardColor.r*(1.0-colorFactor));	\n"
    "	 cardColor.g = (gc_vVSColor.g*(colorFactor) + cardColor.r*(1.0-colorFactor));	\n"
    "	 cardColor.b = (gc_vVSColor.b*(colorFactor) + cardColor.r*(1.0-colorFactor));	\n"
    "	 cardColor.a = (gc_vVSAlpha + colorFactor)/2.0;					\n";

/*Constructor Methods for MusicVisualizer*/
fslMusicVisualizerCard::fslMusicVisualizerCard()
:vertexLoc_(0),
texcoordLoc_(1),
colorsLoc_(2),
alphaLoc_(3),
isAnimated_(FALSE),
morphing_(FALSE)
{
	currentTime_ = 0.0f;
	tickTime_ = 5.0f;
	currentEffect = rand()%5;
}

fslMusicVisualizerCard::fslMusicVisualizerCard(void* parentscene)
:vertexLoc_(0),
texcoordLoc_(1),
colorsLoc_(2),
alphaLoc_(3),
isAnimated_(FALSE),
morphing_(FALSE)
{
	currentTime_ = 0.0f;
	tickTime_ = 5.0f;
	parentScene_ = parentscene;
	currentEffect = rand()%5;
}

fslMusicVisualizerCard::~fslMusicVisualizerCard()
{
}


fslStatus fslMusicVisualizerCard::initCard( fslInitStageType init, unsigned int config)
{
	
	static GLuint hVertexShader; //static handler for vertex shader
	static GLuint hFragmentShader; //static handler for fragment shader
        fslStatus retVal = FSL_STATUS_SUCCESS;

	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
			{

				/***************init gstreamer***************/
				gst_init (NULL, NULL);

				/***************init struct sound info**************/
				soundInfo_init();

				/***********card initialization*************/
				/*init textures to be used by the accelerator*/
				if(accelerator_.initTextures("media/textures/Particle.CTES","media/textures/Particle2.CTES","media/textures/Particle3.CTES") != FSL_STATUS_SUCCESS)
				{
					retVal = FSL_STATUS_GENERAL_ERROR;
					break;
				}
			 	/*init particles with the shape of a freescale logo*/
				if (accelerator_.initParticles("media/fslLogoLow.txt") != FSL_STATUS_SUCCESS)
				{
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}			
				GLint nCompileResult;
				/*handler for vectoshader code*/
				char vs[MAX_SHADER_LENGTH]; 
				/*handler for fragmentshader code*/
                                char ps[MAX_SHADER_LENGTH]; 
				const char* pvs = (char*)(vs); 
				const char* pps = (char*)(ps); 

				/*concatenation of the fragment shader parts*/
				sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
				/*concatenation of the vector shader parts*/
				sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				printf("Vertex Shader = \n %s \n", vs);
				printf("Pixel Shader = \n %s \n", ps);

				// Compile the shaders
				printf("Compiling Shaders... \n");
				printf("...VERTEX \n");

				//Create a Vertex Shader and get the handler
				hVertexShader = glCreateShader( GL_VERTEX_SHADER );

				//point the hander to the string containin the shader code and compile the shader
				glShaderSource( hVertexShader, 1, &pvs, NULL );
				glCompileShader( hVertexShader );

				//get the compile resut from the parameter GL_COMPILE_STATUS and save it in nComileResult
				glGetShaderiv(hVertexShader, GL_COMPILE_STATUS, &nCompileResult);

				//if vertex shader compilation was not succesful
				if (!nCompileResult)
				{
					GLint nLength = 0; //length of the shader log
					printf("Error compiling shader! \n");
					//get shader log length
					glGetShaderiv(hVertexShader, GL_INFO_LOG_LENGTH, &nLength);
					
					if(nLength > 1)
					{
						//get the info log
						char* infoLog = (char *)malloc(sizeof(char) * nLength);
						glGetShaderInfoLog(hVertexShader, nLength, NULL, infoLog);
						printf("%s\n", infoLog);
					}
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				printf("...FRAGMENT \n");
				//Create a Fragment Shader and get the handler
				hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
				//point the hander to the string containin the shader code and compile the shader
				glShaderSource( hFragmentShader, 1, &pps, NULL );
				glCompileShader( hFragmentShader );

				//reset nCompileResult
				nCompileResult = 0;
				//get the compile resut from the parameter GL_COMPILE_STATUS and save it in nComileResult
				glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
				//if fragment shader compilation was not succesful
				if (!nCompileResult)
				{
					GLint nLength = 0; //length of the shader log
					printf("Error compiling shader! \n");
					//get shader log length
					glGetShaderiv(hFragmentShader, GL_INFO_LOG_LENGTH, &nLength);
					if(nLength > 1)
					{
						//get the info log
						char* infoLog = (char *)malloc(sizeof(char) * nLength);
						glGetShaderInfoLog(hFragmentShader, nLength, NULL, infoLog);
						printf("%s\n", infoLog);
					}
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}
				
				// Attach the individual shaders to the common shader program
				//create a shader program and get the handler
				shaderProgram_[config] = glCreateProgram();
				//attach the vertex shader
				glAttachShader( shaderProgram_[config], hVertexShader );
				//attach the fragment shader
				glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
				//bind vertex position handler and shader attribute
				glBindAttribLocation(shaderProgram_[config], vertexLoc_, "gc_vVertex");
				//bind texture position handler and shader attribute
				glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "gc_vTexcoord");
				//bind color handler and shader attribute
				glBindAttribLocation(shaderProgram_[config], colorsLoc_, "gc_vColor");
				//bind alpha value handler and shader attribute
				glBindAttribLocation(shaderProgram_[config], alphaLoc_, "gc_vAlpha");
			}
 		
			break;
		case FSL_INIT_LINK:
			{
				//variable to store the result of shader linking
				GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
				printf("Linking Shaders... \n");
				glLinkProgram( shaderProgram_[config] );

				// Check for link success
				nLinkResult = 0;

				//get the link result from parameter GL_LINK_STATUS and save it in nLinkResult
				glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);

				if (!nLinkResult)//If linking was not succesful
				{
					GLint nLength = 0; //length of the shader log
					printf("Error compiling shader! \n");
					//get shader log length
					glGetShaderiv(shaderProgram_[config], GL_INFO_LOG_LENGTH, &nLength);
					if(nLength > 1)
					{
						//get the info log
						char* infoLog = (char *)malloc(sizeof(char) * nLength);
						glGetShaderInfoLog(shaderProgram_[config], nLength, NULL, infoLog);
						printf("%s\n", infoLog);
					}
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				//delete the vertex shader
				glDeleteShader( hVertexShader );
				//delete the fragment shader
				glDeleteShader( hFragmentShader );
				//build the auido pipeline with the first song on the array
				buildPipeline(playList[songNumber]);
				//create a thread containing the audio pipeline
				gstLoop = g_thread_create((GThreadFunc)play,FALSE, TRUE, &err1);	
			}			
			break;
		default:
				retVal = FSL_STATUS_BAD_PARAMETER;
                                break;
	}
	
	return retVal;
}

/*method to reset MusicVisualizer*/
void fslMusicVisualizerCard::resetVisualizer()
{
	//reset particle accelerator
	accelerator_.resetParticles();
	//reset pipeline and make it ready to play
	setPipelineState(REBUILD);
}

fslStatus fslMusicVisualizerCard::shutdownCard() //Releases dynamic resources without destroying card
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerCard::resumeCard()   //Reallocates resources for card; probably the same as initCard for most cards
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerCard::handleInputEvent(fslInputEventType)
{
        return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerCard::animTick() //adjusts state by the time tick set in resolution
{
	//This card currently has no animation
	isAnimated_ = TRUE;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerCard::setTickRes(float time)
{
	//This card currently has no animation
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerCard::setTime(float time)
{
	//This card currently has no animation
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

float fslMusicVisualizerCard::getTime()
{
	return currentTime_;
}

//method that set the pipeline state
fslStatus fslMusicVisualizerCard::setPipelineState(int state)
{
	if(state == BWD) //play the previous song
	{
		if ((songNumber <= 4) && (songNumber >= 1)) //decrese the song number
		{
			songNumber--;
			printf("song number = %d \n",songNumber);
		}else //if we are in the first song play the last song
			songNumber = 3;
		//set the pipeline to null
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_NULL);
		//unreference the pipeline
		gst_object_unref (GST_OBJECT (audioPipeline));
		//rebuild pipeline with the NOW current song
		buildPipeline(playList[songNumber]);
		//play the song
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_PLAYING);
		//tell the visualizer that a song is playing
		isAudioPlaying=TRUE;
		return FSL_STATUS_SUCCESS;
	}else if (state == PAUSE) //pause the current song
	{
		//set the pipeline to pause
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_PAUSED);
		//tell the visualizer the song is no longer played
		isAudioPlaying=FALSE;
		return FSL_STATUS_SUCCESS;
	}else if (state == PLAY) //play the current song
	{
		//set the pipeline to play
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_PLAYING);
		//tell the visualizer that a song is playing
		isAudioPlaying=TRUE;
		return FSL_STATUS_SUCCESS;
	}else if (state == STOP) //stop the current song
	{	
		//set the pipeline to ready
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_READY);
		//tell the visualizer the song is no longer played
		isAudioPlaying=FALSE;
		return FSL_STATUS_SUCCESS;
	}else if (state == FWD) //play the next song
	{
		//increase the song number 
		if ((songNumber <= 2) && (songNumber >= 0))
		{
			songNumber++;
			printf("song number = %d \n",songNumber);
		}else //if the last song is already been played, play the first song
			songNumber = 0;
		//set the pipeline to null
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_NULL);
		//unreference the pipeline
		gst_object_unref (GST_OBJECT (audioPipeline));
		//rebuild pipeline with the NOW current song
		buildPipeline(playList[songNumber]);
		//play the song
		gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_PLAYING);
		//tell the visualizer that a song is playing
		isAudioPlaying=TRUE;
	}else if (state == DESTROY) //destroy the pipeline
	{
		if (GST_IS_ELEMENT(audioPipeline))
		{
			//set the pipeline to null
			gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_NULL);
			//unreference the pipeline
			gst_object_unref (GST_OBJECT (audioPipeline));
		}
		//tell the visualizer the song is no longer played
		isAudioPlaying=FALSE;
	}else if (state == REBUILD) //re-build pipeline
	{
		buildPipeline(playList[songNumber]);
	}else
		return FSL_STATUS_BAD_PARAMETER;
}
fslStatus fslMusicVisualizerCard::renderCard(unsigned int phaseID, unsigned int config)
{
	
	switch(phaseID)
	{
		case 1:
                {       
		if(isAnimated_) 
		{
		
			glClearColor(0.0f,0.0f,0.0f,0.0f);  //set clear color as black
			glClear(GL_COLOR_BUFFER_BIT); //clear buffer
			//install shader program as part of the rendering state
			glUseProgram( shaderProgram_[config] );
			//clear the depth buffer and disable it, since it is not needed for this card
			glClearDepthf(1.0f);
			glDisable(GL_DEPTH_TEST);
			//enable blending
			glEnable(GL_BLEND);
			//set blending function as dependant of alpha
			glBlendFunc(GL_SRC_ALPHA,GL_ONE);

			if(goomEvent>0) //if we have an audio event
			{
				//switch particle accelerator efect
				currentEffect = rand()%5; 
			}

			//set efect speed
			accelerator_.setEffectVelocity(speed, currentEffect);

			if(!isAudioPlaying && !morphing_) //prepare for the next morph
			{
				// set current particle accelerator position as morphing source
				accelerator_.setMorphSource();
				morphing_=TRUE; //turn on morphing flag
			}
			
			if (morphing_) // if the particle acceleration is morfing
			{
				//set the current morphing delta value
				accelerator_.setMorphDelta();
			}

			// cicle trought all the particles in the accelerator
			for (int i = 0; i < MAX_PARTICLES; i++)
			{

				if(!isAudioPlaying)
				{
					//set freescale logo colors
					if (i<60)
						accelerator_.setParticleColor(i,FSL_LOGO_RED);
					else if (i<90)
						accelerator_.setParticleColor(i,FSL_LOGO_YELLOW);
					else if (i<150) 
						accelerator_.setParticleColor(i,FSL_LOGO_RED);
					else if (i<180)	
						accelerator_.setParticleColor(i,FSL_LOGO_YELLOW);
					else if (i<210)	
						accelerator_.setParticleColor(i,FSL_LOGO_RED);
					else if (i<240)	
						accelerator_.setParticleColor(i,FSL_LOGO_YELLOW);
					//if morphing is not done yet, update particle positions
					if(accelerator_.updateMorphing(i) != TRUE)
					{
						morphing_=FALSE;
					}	
				}		

				if (isAudioPlaying) //if a song is playing
				{
					//update accelerator logic
					accelerator_.updateAccelerator(i);
					//update effect  logic
					accelerator_.updateEffect(i, currentEffect); 				
				}
			}

			//get vertex positions from accelerator
			float *VertexPositions = accelerator_.getAcceleratorVertex();
			/*printf("VertexPositions\n");
			printf("%f,%f,%f,%f \n", VertexPositions[0],VertexPositions[1],
                                                 VertexPositions[2],VertexPositions[3]);
			printf("%f,%f,%f,%f \n", VertexPositions[4],VertexPositions[5],
						 VertexPositions[6],VertexPositions[7]);
			printf("%f,%f,%f,%f \n", VertexPositions[8],VertexPositions[9],
					         VertexPositions[10],VertexPositions[11]);
			printf("%f,%f,%f,%f \n", VertexPositions[12],VertexPositions[13],
					         VertexPositions[14],VertexPositions[15]);
			printf("%f,%f,%f,%f \n", VertexPositions[16],VertexPositions[17],
					         VertexPositions[18],VertexPositions[19]);
			printf("%f,%f,%f,%f \n", VertexPositions[20],VertexPositions[21],
					         VertexPositions[22],VertexPositions[23]);*/
			//get texture positions from accelerator
			float *VertexTexcoord = accelerator_.getAcceleratorTextCoord();
			/*printf("VertexTexcoord\n");
			printf("%f,%f \n", VertexTexcoord[0], VertexTexcoord[1]);
			printf("%f,%f \n", VertexTexcoord[2], VertexTexcoord[3]);
			printf("%f,%f \n", VertexTexcoord[4], VertexTexcoord[5]);
			printf("%f,%f \n", VertexTexcoord[6], VertexTexcoord[7]);
			printf("%f,%f \n", VertexTexcoord[8], VertexTexcoord[9]);
			printf("%f,%f \n", VertexTexcoord[10], VertexTexcoord[11]);*/

			//get particle colors from accelerator
			float *Color = accelerator_.getAcceleratorColor();
			/*printf("Color\n");
			printf("%f,%f,%f \n", Color[0],Color[1],
                                                 Color[2]);
			printf("%f,%f,%f \n", Color[3],Color[4],
						 Color[5]);
			printf("%f,%f,%f \n", Color[6],Color[7],
					         Color[8]);
			printf("%f,%f,%f \n", Color[9],Color[10],
					         Color[11]);
			printf("%f,%f,%f \n", Color[12],Color[13],
					         Color[14]);
			printf("%f,%f,%f \n", Color[15],Color[16],
					         Color[17]);*/

			//get particle alpha valures from accelerator
			float *Alpha = accelerator_.getAcceleratorAlpha();
			//printf("%f,%f,%f \n", Alpha[0], Alpha[1], Alpha[2]);

			//Bind texture for particles
			glBindTexture( GL_TEXTURE_2D, accelerator_.textureHandleParticle3_ );
			//point vertex location handler to VertexPositions
			glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
			//enable vertex array with vertex location handler for draw arrays
			glEnableVertexAttribArray( vertexLoc_ );
			//point texture coordinates handler to VertexTexcoord
			glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexcoord);
			//enable vertex array with the texture coordinates handler for draw arrays
			glEnableVertexAttribArray( texcoordLoc_ );
			//point colors handler to Color
			glVertexAttribPointer( colorsLoc_, 3, GL_FLOAT, 0, 0, Color);
			//enable vertex array with colors for draw arrays
			glEnableVertexAttribArray( colorsLoc_ );
			//point alpha value to Alpha
			glVertexAttribPointer( alphaLoc_, 1, GL_FLOAT, 0, 0, Alpha);
			//enable vertex array with alpha values for draw arrays
			glEnableVertexAttribArray( alphaLoc_ );

			glDrawArrays( GL_TRIANGLES, 0, MAX_PARTICLES*6 ); //draw			
			/******************test************************************************/
			/*float VertexPositions[] =
							{
								 0.5f,  0.5f, 0.0f, 1.0f,
								-0.5f,  0.5f, 0.0f, 1.0f,
								-0.5f, -0.5f, 0.0f, 1.0f,

								 0.5f,  0.5f, 0.0f, 1.0f,
								-0.5f, -0.5f, 0.0f, 1.0f,
								 0.5f, -0.5f, 0.0f, 1.0f,
								
							};

			float VertexTexcoord[] =
							{
								1.0f,	1.0f,
							        0.0f,	1.0f,
							        0.0f,	0.0f,

								1.0f,	1.0f,
								0.0f,	0.0f,
								1.0f,	0.0f,
							};

			float particleColor [] = {1.0f,0.0f,0.0f,1.0f,0.0f,0.0f,1.0f,0.0f,0.0f,1.0f,0.0f,0.0f,
						  1.0f,0.0f,0.0f,1.0f,0.0f,0.0f};

			float particleAlpha [] = {1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f,1.0f};			

			glBindTexture( GL_TEXTURE_2D, accelerator_.textureHandleParticle3_ );
			glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
			glEnableVertexAttribArray( vertexLoc_ );

			glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexcoord);
			glEnableVertexAttribArray( texcoordLoc_ );
		
			glVertexAttribPointer( colorsLoc_, 3, GL_FLOAT, 0, 0, particleColor);
			glEnableVertexAttribArray( colorsLoc_ );

			glVertexAttribPointer( alphaLoc_, 1, GL_FLOAT, 0, 0, particleAlpha);
			glEnableVertexAttribArray( alphaLoc_ );

			glDrawArrays( GL_TRIANGLES, 0, 6 );*/
			/******************testend************************/

						
	
			/*undo setup so other cards drawn have no trouble*/
			glDisableVertexAttribArray( vertexLoc_ );
			glDisableVertexAttribArray( texcoordLoc_ );
			glDisableVertexAttribArray( colorsLoc_ );
			glEnable(GL_DEPTH_TEST);
			glDisable(GL_BLEND);
			isAnimated_ = FALSE;
                }
		}
			break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}

	return FSL_STATUS_SUCCESS;
}

void fslMusicVisualizerCard::getRenderPhases(fslPhaseElementType* phaseArray) //assumed fixed size of MAX_PHASE_ARRAY_SIZE
{
	if(phaseArray == NULL)
		return;
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_WRITEONLY;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_RENDER_READWRITE;
	phaseArray[1].phaseID = 2;
	phaseArray[2].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[2].phaseID = 0;

	return;
}

float fslMusicVisualizerCard::getAspectRatio()
{
	return 4.0f/3.0f;
}
