//
// C++ Implementation: Music Visualizer Scene
//
// Description: 
//
//
// Author: FCSZ <root@ubuntu-desktop>, (C) 2009
//
// Copyright: See COPYING file that comes with this distribution
//
//

#include "testscene_fslmusicvisualizercard.h"

static char* g_strVSPre =  "	uniform   mat4 g_matProj;				\n";
static char* g_strVSPost = "    gl_Position  = g_matProj * cardPosition;		\n";
static char* g_strPSPre =  "								\n";
static char* g_strPSPost = "    gl_FragColor = cardColor;				\n";

fslMusicVisualizerScene::fslMusicVisualizerScene( fslScene* parentScene )
{
	parentScene_ = parentScene;

	for(int i=0; i < MAX_CARDS_IN_SCENE; i++)
		cardList_[i] = NULL;

	for(int i=0; i < MAX_SCENES_IN_SCENE; i++)
		sceneList_[i] = NULL;

	numCards_ = 1;
	numScenes_ = 0;

	newCardID_ = 0;
	orientation_ = FSL_LANDSCAPE;
	isVisible_ = FSL_FALSE;

	myMusicVisualizerCard.setParentScene(this);
	//base value to keep track of events from card
	// i.e if between 1000 - 1999, then it is from this card
	//Only really valuable when tracking multiple cards
	myMusicVisualizerCard.setBaseEventID( 1000 );
	isRunning_ = true;
	return;
}

fslMusicVisualizerScene::~fslMusicVisualizerScene()
{
	return;
}
	
fslStatus fslMusicVisualizerScene::setAspectRatio( float aspect )
{
	//There is no big impact to this card for changing the aspect ratio
	aspectRatio_ = aspect;
	return FSL_STATUS_SUCCESS;
}
	
fslStatus fslMusicVisualizerScene::setOrientation( fslOrientationType orientation )
{
	//There is no big impact to this card for changing the orientation
	orientation_ = orientation;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerScene::transitionOnStage( fslTransitionType transition)
{
	//Nothing fancy for the example
	//This should probably trigger an animation state change in a real scene
	isVisible_ = FSL_TRUE;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerScene::transitionOffStage( fslTransitionType transition)
{
	isVisible_ = FSL_FALSE;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerScene::renderScene( fslPhaseType onlyThese)
{
		
	if(isVisible_ != FSL_TRUE)
		return FSL_STATUS_SUCCESS;

	if(onlyThese == FSL_PHASE_RENDER_WRITEONLY || onlyThese == FSL_PHASE_UNSPECIFIED)
	{
		fslLoadIdentityMatrix4x4(visualizerModViewMatrix_);
		//memcpy(currentProjMatrix_, currentProjMatrix_, (sizeof(float) * 16));
		fslInvertMatrix4x4( currentProjMatrix_, currentInvProjMatrix_);
		fslTranslateMatrix4x4(visualizerModViewMatrix_,0.5f,1.0f,-9.0f);
		//fslTranslateMatrix4x4(visualizerModViewMatrix_,0.0f,0.0f,-3.0f); //test
		//fslTranslateMatrix4x4(visualizerModViewMatrix_,0.0f,0.0f,-9.0f);
		glUniformMatrix4fv( modelMatrixLoc_,1, GL_FALSE, visualizerModViewMatrix_ );

		//Master Scene (or application) handles the clears
		//Scene handles other state setup such as:
		//glDisable(GL_DEPTH_TEST); //because we'll start using it :)
	
		//set attribs and uniforms
		//myMusicTileCard_[0].preRenderCard(phases_[0], 0);
		
		glUniformMatrix4fv( projMatrixLoc, 1, 0, currentProjMatrix_ );//pass matrix to shader uniform
		
		myMusicVisualizerCard.renderCard(phases_[0], 0);

		return FSL_STATUS_SUCCESS;
		
	}
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerScene::handleInputEvent( fslInputEventType event)
{


	float vecA[4] = {0};
	float vecC[4] = {0};
	float vecA3[4] = {0};
	float vecC3[4] = {0};
	
	//This switch is for catching other event types
	/*
	printf("Scene Event:");
	switch(event.input)
	{
		case FSL_INPUT_KEYPRESS: printf("Key Press"); break;
		case FSL_INPUT_KEYRELEASE: printf("Key Release"); break;
		case FSL_INPUT_POINTERDOWN: printf("Pointer Down"); break;
		case FSL_INPUT_POINTERUP: printf("Pointer Up"); break;
		case FSL_INPUT_POINTERMOVE: printf("Pointer Move"); break;
		default: printf("UNHANDLED\n");
	}
	*/
	
	//Convert click coords to -1 to 1 X,Y 3D coords
	vecA[0] = vecA3[0] = ((float) event.signalA * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[1] = vecA3[1] = ((float) event.signalB * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	
	vecA3[1] -= 0.8f;

	//convert from screen space to card space
	fslMultMatrix4x4Vec4x1 ( currentInvProjMatrix_, vecA, vecC );

	
	//Debug Info
	//printf(" [%u,%u] \n",event.signalA,event.signalB);
//	printf(" pretransformed vec = [%f,%f,%f,%f] \n",vecA3[0],vecA3[1],vecA3[2],vecA3[3]);		
//	printf(" invtransformed vec = [%f,%f,%f,%f] \n",vecC3[0],vecC3[1],vecC3[2],vecC3[3]);	
	
	//Make sure the click falls in the bounds of the card
	if( (event.input == FSL_INPUT_POINTERDOWN) 
		&& (vecC[0] <= 1.0) && (vecC[0] >= -1.0)
		&& (vecC[1] <= 1.0) && (vecC[1] >= -1.0))
	{
		//Convert from -1 to 1 float to 0 to 1 uint
		event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
		event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX);  
		
		//printf( "Passing [%u,%u] to Card\n", event.signalA, event.signalB);
		
// 		myExampleCard_.handleInputEvent( event );
	}

	//Make sure the click falls in the bounds of the card
	if( ((event.input == FSL_INPUT_POINTERDOWN) || 
		(event.input == FSL_INPUT_POINTERUP)) 
		&& (vecC3[0] <= 1.0) && (vecC3[0] >= -1.0)
		&& (vecC3[1] <= 1.0) && (vecC3[1] >= -1.0))
	{
		//Convert from -1 to 1 float to 0 to 1 uint
		event.signalA = (unsigned int)(((vecC3[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
		event.signalB = (unsigned int)(((vecC3[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX);  


	}	

	if (event.input == FSL_INPUT_CLOSEWINDOW)
	{
		isRunning_ = false;
	}

	return FSL_STATUS_SUCCESS;
}

fslStatus fslMusicVisualizerScene::initScene()  //GL state inititization
{
	glClearColor(1.0f,1.0f,1.0f,0.0f);
	glClearDepthf(1.0f);
	glDisable(GL_DEPTH_TEST);
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA,GL_ONE);
	glEnable(GL_TEXTURE_2D);

	fslPhaseElementType phaseArray[MAX_PHASE_ARRAY_SIZE];

	myMusicVisualizerCard.getRenderPhases( phaseArray );

 	//Set projection matrix to Identity Matrix
	fslLoadIdentityMatrix4x4(currentProjMatrix_);

	//set perspective projection matrix
	//should be aspectRatio_, not sure if that works, so hard coded in the meantime
	fslPerspectiveMatrix4x4(currentProjMatrix_, 45.0f, 1.3333f, 0.01f, 200.0f);

	//I know there is only one phase for this card
	//More dynamic scenes would need to parse the whole array
	phases_[0] = phaseArray[0].phaseID;

	//Set our shader contibution

	myMusicVisualizerCard.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myMusicVisualizerCard.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	printf("about to compile musicVisualizerCard shader \n");
	if( myMusicVisualizerCard.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	shaderProgram = myMusicVisualizerCard.getShaderProgram(0);
	//Set any attribute handles needed
	// There are none for this scene
	

	if( myMusicVisualizerCard.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	//Get uniform handles
	projMatrixLoc = glGetUniformLocation( shaderProgram, "g_matProj" );
	modelMatrixLoc_ = glGetUniformLocation( shaderProgram, "gc_matModelView" );
	if((modelMatrixLoc_ == -1) || (projMatrixLoc == -1))
        {
		return FSL_STATUS_GENERAL_ERROR;
                
        }
	
	//Rinse, repeat for extra stages, reflections, etc.  Make sure and increase the config parameter for init
	
	return FSL_STATUS_SUCCESS;
}

fslBool fslMusicVisualizerScene::isOnStage()
{
	return isVisible_;
}

fslStatus fslMusicVisualizerScene::animResolve( float deltaTime )
{
	//he scene calls the animTick functions of the cards

	myMusicVisualizerCard.animTick();

	//fslTranslateMatrix4x4(myMusicVisualizerCard.getMVMatrix(),-1.f,0,-3); ???

	return FSL_STATUS_SUCCESS;
}

void fslMusicVisualizerScene::receiveCardEvent( unsigned int eventID, void* data )
{
	//Child Cards call this function on events

	fslInputEventType event;
	if (eventID == FSL_INPUT_CLOSEWINDOW)
	{
		
		event.input = FSL_INPUT_CLOSEWINDOW;
// 		myExampleCard_.handleInputEvent( event );
		isRunning_ = false;

	}

}