/* File: fslplaybackcontrolcard.cpp
 * 
 * Desc: This is the card for playback control card.  This card
 * draws the playback control and accepts click events localizes
 * the click and passes the appropriate event back to the parent.
 * 
 *                      
 * Copyright © 2010 Freescale Semiconductor
 */

//The shaders are very simple for this card.  The textures are rendered with a color added
// to every pixel.  This allows transparency and highlighting to be applied per draw call.


static const char* g_strEnd =
    "}													\n";

static const char* g_strVSProgramA = 
    "uniform   mat4 _matMVLocal;							\n"
    "													\n"
    "attribute vec4 gc_vVertex;							\n"
    "attribute vec4 gc_vTexcoord;						\n"
    "													\n"
    "varying   vec4 gc_vVSTexcoord;						\n"
    "													\n";

static const char* g_strVSProgramB = 
    "void main()										\n"
    "{													\n"	
	"    vec4 cardPosition = _matMVLocal * gc_vVertex;\n"
    "    gc_vVSTexcoord = gc_vTexcoord;					\n";
		


static const char* g_strFSProgramA = 
    "#ifdef GL_FRAGMENT_PRECISION_HIGH									\n"
    "   precision highp float;											\n"
    "#else																\n"
    "   precision mediump float;										\n"
    "#endif																\n"
    "																	\n"
    "uniform   sampler2D gc_sImageTexture;								\n"
    "uniform   vec4      gc_vAdderColor;								\n"
    "varying   vec4      gc_vVSTexcoord;									\n"
    "																	\n";

static const char* g_strFSProgramB = 
    "void main()														\n"
    "{																	\n"
	"    vec4 cardColor = texture2D(gc_sImageTexture, gc_vVSTexcoord.xy);	\n"	
    	"    cardColor.r = clamp(0.0, 1.0, gc_vAdderColor.r + cardColor.r);                  \n" 
	"    cardColor.g = clamp(0.0, 1.0, gc_vAdderColor.g + cardColor.g);                  \n" 
	"    cardColor.b = clamp(0.0, 1.0, gc_vAdderColor.b + cardColor.b);                  \n" 
	"    cardColor.a = clamp(0.0, 1.0, gc_vAdderColor.a + cardColor.a);                  \n" 

	"    //Make the control semi-transparent clamping to 0.0	\n"	
	"    cardColor.a = max(cardColor.a - 0.5,  0.0) ;	\n"	;
    



#include <iostream>
#include <FSL/fslutil.h>
#include <CARD/fslcard.h>
#include <CARD/fslscene.h>
#include <CARD/fslplaybackcontrolcard.h>

#ifdef DBGPRINT
#undef DBGPRINT
#endif

#if 1
#define PREPRINT std::cout << "PLAYBACK_CTL: ";
#define DBGPRINT(...) PREPRINT printf(__VA_ARGS__);
#else
#define DBGPRINT(fmt,...)
#endif
#define MAX_SHADER_LENGTH 2000

//Basic constructor.  By default texture is highlighted in blue
fslPlaybackControlCard::fslPlaybackControlCard()
: vertexLoc_(0),
texcoordLoc_(1),
adderColorLoc_(2)
{
    adderColor_[0]=0;
    adderColor_[1]=0;
    adderColor_[2]=0.5f;
    adderColor_[3]=0;
    sprintf(textureFileName_, "media/textures/player-ctrl.CTES");
    initButtons();
}

//Basic constructor + parent set.  By default texture is highlighted in blue
fslPlaybackControlCard::fslPlaybackControlCard(void* parentscene)
: vertexLoc_(0),
texcoordLoc_(1),
adderColorLoc_(2)
{
    	adderColor_[0]=0;
    	adderColor_[1]=0;
    	adderColor_[2]=0.5f;
    	adderColor_[3]=0;
        sprintf(textureFileName_, "media/textures/player-ctrl.CTES");
	parentScene_ = parentscene;
	initButtons();
}

//Destructor
fslPlaybackControlCard::~fslPlaybackControlCard()
{
}

//Setup initial button information
void fslPlaybackControlCard::initButtons()
{
	buttons_[REW].xTexCoord = 1.0f;
        buttons_[REW].yTexCoord = 0.0f;
        buttons_[REW].texWidth  = -0.5f;
        buttons_[REW].texHeight = 0.5f;
        buttons_[REW].isPressed = false;

	buttons_[PAUSE].xTexCoord = 0.0f;
        buttons_[PAUSE].yTexCoord = 0.0f;
        buttons_[PAUSE].texWidth  = 0.5f;
        buttons_[PAUSE].texHeight = 0.5f;
        buttons_[PAUSE].isPressed = false;

	buttons_[PLAY].xTexCoord = 0.0f;
        buttons_[PLAY].yTexCoord = 0.5f;
        buttons_[PLAY].texWidth  = 0.5f;
        buttons_[PLAY].texHeight = 0.5f;
        buttons_[PLAY].isPressed = false;

	buttons_[STOP].xTexCoord = 0.5f;
        buttons_[STOP].yTexCoord = 0.5f;
        buttons_[STOP].texWidth  = 0.5f;
        buttons_[STOP].texHeight = 0.5f;
        buttons_[STOP].isPressed = false;

	buttons_[FFWD].xTexCoord = 0.5f;
        buttons_[FFWD].yTexCoord = 0.0f;
        buttons_[FFWD].texWidth  = 0.5f;
        buttons_[FFWD].texHeight = 0.5f;
        buttons_[FFWD].isPressed = false;
}


//This function will be called to initialize the card the initialization
// is broken into phases so the parent can add any necessary additional
// initialization between each phase.
fslStatus fslPlaybackControlCard::initCard( fslInitStageType init, unsigned int config) 
{
	static GLuint hVertexShader;
	static GLuint hFragmentShader;
	
        fslStatus retVal = FSL_STATUS_SUCCESS;
	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
			{
				GLint nCompileResult;
				char vs[MAX_SHADER_LENGTH];
                                char ps[MAX_SHADER_LENGTH];
				const char* pvs = (char*)(vs); // Needed for C++ compiler
				const char* pps = (char*)(ps); // Needed for C++ compiler

				DBGPRINT("Starting Initialization \n");

				//Concatenate all of the shader strings into a single string
				sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
				sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				//Load the texture image
				if(!fslInit2DCTESTextureGL( textureFileName_, &textureHandle_))
                                {
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
                                }
				DBGPRINT("Vertex Shader = \n %s \n", vs);
				DBGPRINT("Pixel Shader = \n %s \n", ps);

					// Compile the shaders
				DBGPRINT("Compiling Shaders... \n");
				DBGPRINT("...VERTEX \n");
				hVertexShader = glCreateShader( GL_VERTEX_SHADER );
				glShaderSource( hVertexShader, 1, &pvs, NULL );
				glCompileShader( hVertexShader );

				//Print compiler errors if any
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					DBGPRINT("%s\n", Log);
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				DBGPRINT("...FRAGMENT \n");
				hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
				glShaderSource( hFragmentShader, 1, &pps, NULL );
				glCompileShader( hFragmentShader );

				// Check for compile success
				nCompileResult = 0;
				glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
				//Print compiler errors if any
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					DBGPRINT("%s\n", Log);
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				// Attach the individual shaders to the common shader program
				shaderProgram_[config] = glCreateProgram();
				glAttachShader( shaderProgram_[config], hVertexShader );
				glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
				glBindAttribLocation(shaderProgram_[config], vertexLoc_, "gc_vVertex");
				glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "gc_vTexcoord");
				glBindAttribLocation(shaderProgram_[config], matMVLocalLoc_, "_matMVLocal");

				
				numShaderAttributes_ = 3; //The scene can use attributes higher than this

			}
 		
			break;
		case FSL_INIT_LINK:
			{
				GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
				DBGPRINT("Linking Shaders... \n");
				glLinkProgram( shaderProgram_[config] );

				// Check for link success
				nLinkResult = 0;
				glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);

				//Print link errors if any
				if (!nLinkResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error linking shader! \n");
					glGetProgramInfoLog(shaderProgram_[config], 1024, &nLength, Log);
					DBGPRINT("%s\n", Log);
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				adderColorLoc_ = glGetUniformLocation( shaderProgram_[config], "gc_vAdderColor" );

				glDeleteShader( hVertexShader );
				glDeleteShader( hFragmentShader );
				DBGPRINT("Finished Initialization... \n");	
			}			
			break;
		default:
				retVal = FSL_STATUS_BAD_PARAMETER;
                                break;
	}
	
	return retVal;
}

//Nothing to be done for this card
fslStatus fslPlaybackControlCard::shutdownCard() 
{
	return FSL_STATUS_SUCCESS;
}

//Nothing to be done for this card
fslStatus fslPlaybackControlCard::resumeCard()   
{
	return FSL_STATUS_SUCCESS;
}

//adjusts state by the time tick set in resolution
fslStatus fslPlaybackControlCard::animTick(void) 
{
	//This card currently has no animation
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslPlaybackControlCard::setTickRes(float time)
{
	//This card currently has no animation
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslPlaybackControlCard::setTime(float time)
{
	//This card currently has no animation
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

float fslPlaybackControlCard::getTime()
{
	return currentTime_;
}

//This function is needed so the parent can load the shader for the  phase that's
// going to be rendered so it can set any necessary data that will be used by the shader
// before actually calling render
fslStatus fslPlaybackControlCard::preRenderCard(unsigned int phaseID, unsigned int config)  
{
	switch(phaseID)
	{
	case 1:
			// Set the shader program and the texture
			glUseProgram( shaderProgram_[config] );
			break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;

}

//This function is where all of the work is actually done for rendering the card
// It may be called multiple times with different phase IDs by the parent
// However, this card only supports a single render phase
fslStatus fslPlaybackControlCard::renderCard(unsigned int phaseID, unsigned int config)  
{
	switch(phaseID)
	{
		case 1:
                {        
 			float matModelView[16] = {0}; 
			float left = -1.0f;


			//Set the initial vertex positions for the first button
			float VertexPositions[] =
								{
									left ,		-.20f , 0.0f, 1.0f,
									left + .4f,	-.20f, 0.0f, 1.0f,
									left,		.20f, 0.0f, 1.0f,
									left + .4f,	.20f, 0.0f, 1.0f
								};

			//Rewind button
			float VertexTexcoord[8]; 

			fslLoadIdentityMatrix4x4(matModelView);

			glBindTexture( GL_TEXTURE_2D, textureHandle_ );

			glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
			glEnableVertexAttribArray( vertexLoc_ );

			glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexcoord);
			glEnableVertexAttribArray( texcoordLoc_ );

			//Setup alpha blending.  We can't assume the state hasn't been changed by
			// other cards or scenes
			glEnable(GL_BLEND);
                        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			//Go through each button and draw it.  The buttons structure defines where the 
			//button sub-textures are located in the global texture and the location of the
			//button in the button bar are determined by the order in which they are rendered
			// from left to right 
			for (int i = REW; i < LAST_BUTTON; i++)
			{
				// set buttons coordinates
				VertexTexcoord[0] = buttons_[i].xTexCoord;
				VertexTexcoord[1] = buttons_[i].yTexCoord + buttons_[i].texHeight;
				VertexTexcoord[2] = buttons_[i].xTexCoord + buttons_[i].texWidth;
				VertexTexcoord[3] = buttons_[i].yTexCoord + buttons_[i].texHeight;
				VertexTexcoord[4] = buttons_[i].xTexCoord;
				VertexTexcoord[5] = buttons_[i].yTexCoord;
				VertexTexcoord[6] = buttons_[i].xTexCoord + buttons_[i].texWidth;
				VertexTexcoord[7] = buttons_[i].yTexCoord;
				//Adder color is used to denote that a button is pressed
				if(buttons_[i].isPressed)
				{
					glUniform4f( adderColorLoc_ , adderColor_[0], adderColor_[1],adderColor_[2], adderColor_[3]);
				}
				else
				{
					glUniform4f( adderColorLoc_ , 0.0f, 0.0f, 0.0f, 0.0f);
				}
				glUniformMatrix4fv(matMVLocalLoc_,1, GL_FALSE, matModelView ); 
				glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
				fslTranslateMatrix4x4(matModelView, 0.4f, 0, 0);
			}
				
			glDisable(GL_BLEND);

			glDisableVertexAttribArray( vertexLoc_ );
			glDisableVertexAttribArray( texcoordLoc_ );
                }
			break;

		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}

//This function allows the card to tell the parent that only one phase is supported
void fslPlaybackControlCard::getRenderPhases(fslPhaseElementType* phaseArray) 
{
	if(phaseArray == NULL)
		return;
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_READWRITE;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[1].phaseID = 0;

	return;
}

//This is a hint to the parent of what the ideal aspect ratio
// is for this card
float fslPlaybackControlCard::getAspectRatio()
{
	return 6.0f/1.0f ; 
}


// This function allows the parent to set the adder that should be 
// applied to buttons that are in the pushed state
fslStatus fslPlaybackControlCard::setAdderColor(float adderColor[4])
{
        adderColor_[0] = adderColor[0];
        adderColor_[1] = adderColor[1];
        adderColor_[2] = adderColor[2];
        adderColor_[3] = adderColor[3];
	return FSL_STATUS_SUCCESS;
}

//This function allows the card to reacted to input events from the user.  The button touched
// is detected and an appropriate event is raised to the parent as a result.
//These events are ONLY card relevent;  All coords are in 0-1, 0-1, card space (lower left origin)
fslStatus fslPlaybackControlCard::handleInputEvent( fslInputEventType event)
{
	//On pointer down, mark the button as pressed 
	if(event.input == FSL_INPUT_POINTERDOWN) 
	{
		int index = ((float)event.signalA/(float)FSL_POINTER_MAX) * (float)LAST_BUTTON;
		buttons_[index].isPressed = true;
                if (parentScene_)
		{
			((fslScene*)parentScene_)->receiveCardEvent(index + baseEventID_, NULL);
		} 
	}
	//On pointer up mark the button as not pressed
	else if(event.input == FSL_INPUT_POINTERUP) 
	{
		for (int i = REW; i < LAST_BUTTON; i++) 
		{
			buttons_[i].isPressed = false;
		}
	} else if (event.input == FSL_INPUT_CLOSEWINDOW)
	{
                if (parentScene_)
		{
			((fslScene*)parentScene_)->receiveCardEvent(event.input, NULL);
		} 
		shutdownCard();
	}
	
	return FSL_STATUS_SUCCESS;
}




