#include <GLES2/gl2.h>
#include <EGL/egl.h>
#include <CARD/fslvideochapterscard.h>
#include <CARD/plane.h>
#include <FSL/fslutil.h>
#include <gst/gst.h>
#include <stdbool.h>
#include <stdio.h>
#include <pthread.h>

int			g_iWidth = 320;
int			g_iHeight = 240;
GstElement*		g_pPipeline = NULL;
GstElement*		pVideo = NULL;
GstElement*		g_pPipelineH264 = NULL;
GstElement*		pVideoH264 = NULL;
gboolean		g_bInitialized = FALSE;
gchar*			g_pcFrameBuffer = NULL;
GMutex*			g_pFrameBufferMutex = NULL;
GLfloat			g_afAlpha = 1.0f;
GLfloat			g_fMovieAspect = 1.0f;
GLuint			g_uiDisplayList = 0;
GLfloat			g_fOffset = 0.5f;
gdouble			g_fLastTimeStamp = 0.0f;
gdouble			g_fCurrentTimeStamp = 0.0f;
gdouble			g_fLastFullSecond = 0.0f;

GTimer*			g_pTimerId = NULL;

GThread myThread;

static gboolean bus_call (GstBus* pBus, GstMessage* pMessage,
									 gpointer data);
static void on_pad_added (GstElement* pDemuxer,
						  GstPad* pPad,
						  GstElement* pDecoder);
static void on_handoff (GstElement* pFakeSink,
						GstBuffer* pBuffer,
						GstPad* pPad);
gboolean build_pipeline (gchar* pcFilename,
						 GMainLoop** pLoopH264,
						 GstElement** pPipelineH264,
						 GstBusFunc busCallbackFunc,
						 GCallback padAddedFunc,
						 GstPad** pVideoPadH264,
						 GCallback handoffHandlerFunc);
gboolean spit_out_clip_stats (GstPad* pVideoPad);
struct thread_data{
	GMainLoop* mainloop;
	void* databuff;
};


static gpointer loop_func(gpointer  data){

		GMainLoop* pLoopH264 = NULL;
				GstPad* pVideoPadH264 = NULL;
				gst_init (NULL, NULL);
				g_pFrameBufferMutex = g_mutex_new ();
				g_pTimerId = g_timer_new ();
			
				printf("about to build pipeline \n" );
				build_pipeline ("media/Kaleidoscope_QVGA.mp4",
					&pLoopH264,
					&g_pPipelineH264,
					bus_call,
					(GCallback) on_pad_added,
					&pVideoPadH264,
					(GCallback) on_handoff);
			  printf("pipeline built \n" );
				g_timeout_add (1000, (GSourceFunc) spit_out_clip_stats, pVideoPadH264);

				g_pcFrameBuffer = (gchar*)malloc(g_iWidth*g_iHeight*2);
				/* Now set to playing and iterate. */
				gst_element_set_state (g_pPipelineH264, GST_STATE_PLAYING);
				printf("State set to PLAYING \n" );
	printf("about to run %08x\n",pLoopH264);


	g_main_loop_run (pLoopH264);



				if (g_pPipelineH264)
				{
					gst_element_set_state (g_pPipelineH264, GST_STATE_NULL);
					gst_object_unref (GST_OBJECT (g_pPipelineH264));
				}
return NULL;
}

gint get_pad_width (GstPad* pVideoPad)
{
	gint iWidth = 0;
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_int (pStructure, "width", &iWidth);
	}
	else
		g_print ("gst_pad_width() - Could not get caps for the pad!\n");

	return iWidth;
}

gint get_pad_height (GstPad* pVideoPad)
{
	gint iHeight = 0;
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_int (pStructure, "height", &iHeight);
	}
	else
		g_print ("gst_pad_height() - Could not get caps for the pad!\n");

	return iHeight;
}


gdouble get_pad_fps (GstPad* pVideoPad)
{
	gint aiFps[2] = {0, 0};
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;
	gdouble fFpsRate = 0.0f;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_fraction (pStructure,
									"framerate",
									&aiFps[0],
									&aiFps[1]);
		fFpsRate = (gdouble) aiFps[0] / (gdouble) aiFps[1];
	}
	else
		g_print ("gst_pad_fps() - Could not get caps for the pad!\n");

	return fFpsRate;
}

gdouble get_pad_movie_aspect (GstPad* pVideoPad)
{
	gint iWidth = 0;
	gint iHeight = 0;
	gint aiPixelAspect[2] = {0, 0};
	gdouble fAspectRatio = 0.0f;
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_int (pStructure, "width", &iWidth);
		gst_structure_get_int (pStructure, "height", &iHeight);
		gst_structure_get_fraction (pStructure,
									"pixel-aspect-ratio",
									&aiPixelAspect[0],
									&aiPixelAspect[1]);

		if (aiPixelAspect[0] == 0)
			fAspectRatio = (gdouble) iWidth / (gdouble) g_iHeight;
		else
		{
			fAspectRatio = (gdouble) aiPixelAspect[0] /
						   (gdouble) aiPixelAspect[1] *
						   (gdouble) iWidth /
						   (gdouble) iHeight;
		}
	}
	else
		g_print ("get_pad_movie_aspect() - Could not get caps for the pad!\n");

	return fAspectRatio;
}
gboolean spit_out_clip_stats (GstPad* pVideoPad)
{
	GLfloat afTexCoords[2] = {(GLfloat) g_iWidth, (GLfloat) g_iHeight};

	//g_print ("The video is %dx%d at %6.3f fps, movie-aspect is %3.2f:1",
			 //get_pad_width (pVideoPad),
			 //get_pad_height (pVideoPad),
			 //get_pad_fps (pVideoPad),
			 //get_pad_movie_aspect (pVideoPad));

	g_mutex_lock (g_pFrameBufferMutex);
	g_pcFrameBuffer = (gchar*)g_malloc0(240* 320* 3);
	if (g_pcFrameBuffer)
		g_bInitialized = TRUE;
	g_mutex_unlock (g_pFrameBufferMutex);

/*
	if (g_uiDisplayList)
	{
		glNewList (g_uiDisplayList, GL_COMPILE);
		glBegin (GL_QUADS);
		glNormal3f (0.0f, 0.0f, 1.0f);
		glTexCoord2f (afTexCoords[0], 0.0f);
		glVertex3f (g_fMovieAspect, 1.0f, 0.0f);
		glTexCoord2f (0.0f, 0.0f);
		glVertex3f (-g_fMovieAspect, 1.0f, 0.0f);
		glTexCoord2f (0.0f, afTexCoords[1]);
		glVertex3f (-g_fMovieAspect, -1.0f, 0.0f);
		glTexCoord2f (afTexCoords[0], afTexCoords[1]);
		glVertex3f (g_fMovieAspect, -1.0f, 0.0f);
		glEnd ();
		glEndList ();
	}
	else
		g_print ("configure_event() - Crap, no display-list was generated!\n");
*/
	/* remove callback after the first call */
	return FALSE;
}
gint64 get_pipeline_current_position (GstElement* pPipeline)
{
	GstFormat fmt = GST_FORMAT_TIME;
	gint64 iPosition = 0L;

	gst_element_query_position (pPipeline, &fmt, &iPosition);

	return iPosition;
}

gint64 get_pipeline_total_duration (GstElement* pPipeline)
{
	GstFormat fmt = GST_FORMAT_TIME;
	gint64 iLength = 0L;

	gst_element_query_duration (pPipeline, &fmt, &iLength);

	return iLength;
}

void seek_to_time (GstElement* pPipeline,
				   gint64 time_nanoseconds)
{
	if (!gst_element_seek (pPipeline,
						   1.0,
						   GST_FORMAT_TIME,
						   GST_SEEK_FLAG_FLUSH,
						   GST_SEEK_TYPE_SET,
						   time_nanoseconds,
						   GST_SEEK_TYPE_NONE,
						   GST_CLOCK_TIME_NONE))
	{
		g_print ("Seek failed!\n");
	}
}

static void on_pad_added (GstElement* pDemuxer,
						  GstPad* pPad,
						  GstElement* pDecoder)
{
	gchar* pcName = NULL;

	pcName = gst_pad_get_name (pPad);

	if (!g_ascii_strncasecmp (pcName, "video", 5))
		if (!gst_element_link_pads (pDemuxer, pcName, pDecoder, "sink"))
			g_print ("could not link demuxer with decoder\n");

	g_free (pcName);
}
static void on_handoff (GstElement* pFakeSink,
						GstBuffer* pBuffer,
						GstPad* pPad)
{
	g_iWidth = get_pad_width (pPad);
	g_iHeight = get_pad_height (pPad);
	g_fMovieAspect = get_pad_movie_aspect (pPad);

	if (g_bInitialized)
	{
		gst_buffer_ref (pBuffer);
		g_mutex_lock (g_pFrameBufferMutex);
		g_memmove (g_pcFrameBuffer,
				   GST_BUFFER_DATA (pBuffer),
				   g_iWidth * g_iHeight * 3);
		//printf("pbuffer copied %08x \n", g_pcFrameBuffer);
		g_mutex_unlock (g_pFrameBufferMutex);
		gst_buffer_unref (pBuffer);
	}
}



gboolean build_pipeline (gchar* pcFilename,
						 GMainLoop** pLoopH264,
						 GstElement** pPipelineH264,
						 GstBusFunc busCallbackFunc,
						 GCallback padAddedFunc,
						 GstPad** pVideoPadH264,
						 GCallback handoffHandlerFunc)
{
	GstElement* pSourceH264 = NULL;
	GstElement* pDemuxerH264 = NULL;
	GstElement* pDecoderH264 = NULL;
	GstElement* pVideoConverterH264  = NULL;
	GstElement* pVideoSinkH264 = NULL;

	*pLoopH264 = g_main_loop_new (NULL, FALSE);
	*pPipelineH264 = gst_pipeline_new ("pipeline-h264");
	gst_bus_add_watch (gst_pipeline_get_bus (GST_PIPELINE (*pPipelineH264)),
					   busCallbackFunc,
					   *pLoopH264);
	pSourceH264 = gst_element_factory_make ("filesrc", "source-h264");
	g_object_set (G_OBJECT (pSourceH264), "location", pcFilename, NULL);
	pDemuxerH264 = gst_element_factory_make ("qtdemux", "demuxer-h264");
	pDecoderH264 = gst_element_factory_make ("mfw_vpudecoder", "decoder-h264");
	gst_bin_add_many (GST_BIN (*pPipelineH264), pSourceH264, pDemuxerH264, pDecoderH264, NULL);
	if (!gst_element_link_pads (pSourceH264, "src", pDemuxerH264, "sink"))
		g_print ("build_pipeline() - could not link filesrc with qtdemux\n");
	g_signal_connect (pDemuxerH264, "pad-added", padAddedFunc, pDecoderH264);
	pVideoH264 = gst_bin_new ("videobin-h264");
	pVideoConverterH264 = gst_element_factory_make ("ffmpegcolorspace", "vconv");
	*pVideoPadH264 = gst_element_get_pad (pVideoConverterH264, "sink");
	pVideoSinkH264 = gst_element_factory_make ("fakesink", "sink-h264");
	g_object_set (G_OBJECT (pVideoSinkH264), "sync", TRUE, "signal-handoffs", TRUE, NULL);
	g_signal_connect (pVideoSinkH264, "handoff", handoffHandlerFunc, NULL);
	gst_bin_add_many (GST_BIN (pVideoH264), pVideoConverterH264, pVideoSinkH264, NULL);
	gst_element_link_filtered (pVideoConverterH264, pVideoSinkH264, gst_caps_new_simple ("video/x-raw-rgb","bpp",G_TYPE_INT,16, NULL));
	gst_element_add_pad (pVideoH264, gst_ghost_pad_new ("sink", *pVideoPadH264));
	gst_object_unref (*pVideoPadH264);
	gst_bin_add (GST_BIN (*pPipelineH264), pVideoH264);
	if (!gst_element_link_pads (pDecoderH264, "src", pVideoH264, "sink"))
		g_print ("build_pipeline() - could not link ffdec_h264 with ffmpegcolorspace\n");
	printf("full pipeline built \n");
	return TRUE;
}
static const char* g_strEnd =
    "}													\n";

static const char* g_strVSProgramA = 
	"uniform   mat4 g_matModelView;						\n"
    "													\n"
    "attribute vec4 g_vVertex;							\n"
    "attribute vec4 g_vTexcoord;						\n"
    "varying   vec4 g_vVSTexcoord;						\n"
    "													\n";

static const char* g_strVSProgramB = 
    "void main()										\n"
    "{													\n"	

    //"    vec4 vPositionES = g_matModelView * g_vVertex; \n"
	"    vec4 cardPosition = g_matModelView * g_vVertex;   \n"

    "    g_vVSTexcoord = g_vTexcoord;					\n";
		

//What vars should scene control?
//g_strVSProgram = g_strVSProgramA + SceneA + g_strVSProgramB + SceneB + g_strEnd;
//SceneB =   "    gl_Position  = cardPosition;		\n"

static const char* g_strFSProgramA = 
    "#ifdef GL_FRAGMENT_PRECISION_HIGH									\n"
    "   precision highp float;											\n"
    "#else																\n"
    "   precision mediump float;										\n"
    "#endif																\n"
    "																	\n"
    "uniform   sampler2D g_sImageTexture;								\n"
    "varying   vec4      g_vVSTexcoord;									\n"
    "																	\n";

static const char* g_strFSProgramB = 
    "void main()														\n"
    "{																	\n"
	"    vec4 cardColor;	\n"		
	"    cardColor.r = 0.0;	\n"		
	"    cardColor.g = 0.5;	\n"		
	"    cardColor.b = 0.3;	\n"
	"    cardColor.a = 1.0;	\n"		
	"    cardColor = texture2D(g_sImageTexture, g_vVSTexcoord.xy);	\n";
 
    

//g_strFSProgram = g_strFSProgramA + SceneA + g_strFSProgramB + SceneB + g_strEnd;
//SceneB =   "    gl_FragColor = cardColor;	\n";





Plane *bgPlane = 0;
Plane *filmPlane = 0;



static GMainLoop *loop;

static gboolean bus_call (GstBus* pBus,
						  GstMessage* pMessage,
						  gpointer data)
{
	GMainLoop* pLoop = (GMainLoop*)data;

	switch (GST_MESSAGE_TYPE (pMessage))
	{
		case GST_MESSAGE_EOS:
			g_print ("End-of-stream\n");
			g_main_loop_quit (pLoop);
		break;

		case GST_MESSAGE_ERROR:
		{
			gchar* pcDebug = NULL;
			GError* pError = NULL;

			gst_message_parse_error (pMessage, &pError, &pcDebug);
			g_free (pcDebug);

			g_print ("Error: %s\n", pError->message);
			g_error_free (pError);

			g_main_loop_quit (pLoop);
			break;
		}

		default:
		break;
	}

	return TRUE;
}


/*
static gboolean bus_call(GstBus *bus,GstMessage *msg, gpointer user_data)
{
	switch(GST_MESSAGE_TYPE(msg))
	{
	   case GST_MESSAGE_EOS:
	    {
		g_message("End-of-stream");
		g_main_loop_quit(loop);
	      break;
             }
           case GST_MESSAGE_ERROR:
	      {	
		gchar *debug;
		GError *err;
		gst_message_parse_error(msg, &err,&debug);
		g_free(debug);
		g_error("%s",err->message);
		g_error_free(err);
		g_main_loop_quit(loop);
	       break;
	      }
	    default:
	       break;
            }
   return true;	

}
*/
static void play_uri(gchar *uri){

	GstElement *pipeline;
        GstBus *bus;  //create  bus
	
	
	loop = g_main_loop_new(NULL,FALSE);
	pipeline = gst_element_factory_make("playbin","player");
	
	
	bus = gst_pipeline_get_bus(GST_PIPELINE(pipeline)); //get the bus from our pipeline
	gst_bus_add_watch(bus,bus_call,NULL);
	gst_object_unref(bus);

	
	if(uri){
	   g_object_set(G_OBJECT(pipeline),"uri",uri,NULL);	
	
	}
	gst_element_set_state(GST_ELEMENT(pipeline),GST_STATE_PLAYING); //play the uri

	g_main_loop_run (loop);

	gst_element_set_state (GST_ELEMENT (pipeline), GST_STATE_NULL); //stop gst
        gst_object_unref (GST_OBJECT (pipeline)); //unreference from the main loop


}
fslExampleCard::fslExampleCard( )
{
	parentscene_ = NULL;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	return;
}

fslExampleCard::fslExampleCard( void* parentscene )
{
	parentscene_ = parentscene;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	return;
}

fslExampleCard::~fslExampleCard()
{
		if (g_bInitialized)
				{
					g_mutex_lock (g_pFrameBufferMutex);
					g_free (g_pcFrameBuffer);
					g_mutex_unlock (g_pFrameBufferMutex);
					g_bInitialized = FALSE;

				}

				g_mutex_free (g_pFrameBufferMutex);
	return;
}

//This should probably be made to work with 'n' shaders
fslStatus fslExampleCard::initCard( fslInitStageType init, unsigned int config) 
{
	static GLuint hVertexShader;
	static GLuint hFragmentShader;
	bgPlane = new Plane();
	filmPlane = new Plane();
	bgPlane->createPlane(0.0,0.0,1.0,1.0);
	filmPlane->createPlane(0.2,0.75,0.8,0.2);


	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
			{
				GLint nCompileResult;
				char vs[MAX_SHADER_LENGTH];
				char ps[MAX_SHADER_LENGTH];
				const char* pvs = (char*)(vs); // This is kind of silly and I'm not sure why this is being required in C++ and not C
				const char* pps = (char*)(ps); //  may also be solved with &(const_cast<char*>(vs))

				//vs = new char[MAX_SHADER_LENGTH];
				//ps = new char[MAX_SHADER_LENGTH];

				printf("Starting Initialization \n");

				//This would probably be better handled with string objects
				sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
				sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				//Could probably add more code to make this less intensive on subsequent inits 
	      printf("About to load texture! \n");
				if(!fslInit2DCTESTextureGL( "../../media/textures/Background.CTES", &backgroundTextureHandle_[config] ))
					return FSL_STATUS_GENERAL_ERROR;

				if(!fslInit2DCTESTextureGL( "../../media/textures/frame256x256.CTES", &backgroundTextureHandle_[1] ))
					return FSL_STATUS_GENERAL_ERROR;
		
				printf("Vertex Shader = \n %s \n", vs);
				printf("Pixel Shader = \n %s \n", ps);

					// Compile the shaders
				printf("Compiling Shaders... \n");
				hVertexShader = glCreateShader( GL_VERTEX_SHADER );
				glShaderSource( hVertexShader, 1, &pvs, NULL );
				glCompileShader( hVertexShader );

				hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
				glShaderSource( hFragmentShader, 1, &pps, NULL );
				glCompileShader( hFragmentShader );

				// Check for compile success
				nCompileResult = 0;
				glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					printf("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					return FSL_STATUS_GENERAL_ERROR;
				}

				// Attach the individual shaders to the common shader program
				shaderProgram_[config] = glCreateProgram();
				glAttachShader( shaderProgram_[config], hVertexShader );
				glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
				glBindAttribLocation(shaderProgram_[config], vertexLoc_,   "g_vVertex");
				glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "g_vTexcoord");
				
				numShaderAttributes_ = 2; //The scene can use attributes higher than this

			}
 		
			break;
		case FSL_INIT_LINK:
			{
				GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
				printf("Linking Shaders... \n");
				glLinkProgram( shaderProgram_[config] );

				// Check for link success
				nLinkResult = 0;
				glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);
				if (!nLinkResult)
				{
					char Log[1024];
					GLint nLength;
					printf("Error linking shader! \n");
					glGetProgramInfoLog(shaderProgram_[config], 1024, &nLength, Log);
					return FSL_STATUS_GENERAL_ERROR;
				}

				//projMatrixLoc = glGetUniformLocation( g_hshaderProgram_, "g_matProj" );
				modelMatrixLoc_[config] = glGetUniformLocation( shaderProgram_[config], "g_matModelView" );

				glDeleteShader( hVertexShader );
				glDeleteShader( hFragmentShader );
				//printf("Finished Initialization... \n");	
				
				
				g_thread_init (NULL);
				thread_data myLoopData;
			
				g_thread_create(loop_func,&myLoopData,FALSE,NULL);
				

				

				


				//gst_init (NULL, NULL); //initialize gst
        //play_uri ("file:///home/ubuntu/Kaleidoscope_mp4-mpeg4+mp3-d1.mp4");
			}			
			break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::shutdownCard() //Releases dynamic resources without destroying card
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::resumeCard()   //Reallocates resources for card; probably the same as initCard for most cards
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::animTick(void) //adjusts state by the time tick set in resolution
{

	float speed = .001f;

	if(filmPlane){
		
		filmPlane->vertexTexCoords[0]+=speed;
		filmPlane->vertexTexCoords[2]+=speed;
		filmPlane->vertexTexCoords[4]+=speed;
		filmPlane->vertexTexCoords[6]+=speed;
		filmPlane->vertexTexCoords[8]+=speed;
		filmPlane->vertexTexCoords[10]+=speed;


}

	
/*
	FrameVertexTexCoords[0]+=speed;
	FrameVertexTexCoords[2]+=speed;
	FrameVertexTexCoords[4]+=speed;
	FrameVertexTexCoords[6]+=speed;
	FrameVertexTexCoords[8]+=speed;
  FrameVertexTexCoords[10]+=speed;
	*/
return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::setTickRes(float time)
{
	//This card currently has no animation
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslExampleCard::setTime(float time)
{
	//This card currently has no animation
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

float fslExampleCard::getTime()
{
	return currentTime_;
}

fslStatus fslExampleCard::renderCard( unsigned int phaseID, unsigned int config )  
{
	//Broken into phases so master render function doesn't get things out of order (alpha, etc.)
	//PhaseID are set by the card in the getRenderPhases
	//Scenes call getRenderPhases to get stages needed by card
	//Scenes then call renderCard with the IDs they got from getRenderPhases grouped with other
	//  cards by the type
	int textureID = 0;
	
	switch(phaseID)
	{
		case 1:
			{

				float matModelView[16] = {0};
				float width_offset = 1.0f;
				float height_offset = 1.0f;
				//background Vertex positions
			
				

				float VertexPositions[] =
									{  //background
										-1.0*width_offset,	-1.0*height_offset, 0.0f, 1.0f,//lower left
										width_offset,	-1.0*height_offset, 0.0f, 1.0f,//lower right
										-1.0*width_offset,	height_offset, 0.0f, 1.0f,//upper left
										-1.0*width_offset,	height_offset, 0.0f, 1.0f,//upper left
										width_offset,	-1.0*height_offset, 0.0f, 1.0f,//lower right
										width_offset,	height_offset, 0.0f, 1.0f//upper right
		
									};
				//Background coords
				
				//movie frame
				float FrameVertexPositions[] =
				{
					-.8*width_offset,	-0.9*height_offset, 0.0f, 1.0f,//lower left
					width_offset*.8,	-0.9*height_offset, 0.0f, 1.0f,//lower right
					-0.8*width_offset,	-0.5*height_offset, 0.0f, 1.0f, //upper left
					-0.8*width_offset,	-0.5*height_offset, 0.0f, 1.0f, //upper left
					width_offset*.8,	-0.9*height_offset, 0.0f, 1.0f,//lower right
					0.8*width_offset,	-0.5*height_offset, 0.0f, 1.0f//upper right
				};

				matModelView[ 0] = 1.0f;
				matModelView[ 5] = 1.0f;
				matModelView[10] = 1.0f;
				matModelView[15] = 1.0f;


	     /*  //background
				// Set the shader program and the texture
				glUseProgram( shaderProgram_[config] );
				
				glUniformMatrix4fv( modelMatrixLoc_[config],1, 0, matModelView );
				glBindTexture( GL_TEXTURE_2D, backgroundTextureHandle_[config] );

				glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
				glEnableVertexAttribArray( vertexLoc_ );

				glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexCoord);
				glEnableVertexAttribArray( texcoordLoc_ );

				//printf("Drawing...\n");
				glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );

				glDisableVertexAttribArray( vertexLoc_ );
				glDisableVertexAttribArray( texcoordLoc_ );
*/
				//chapter frame
			glUseProgram( shaderProgram_[config] );
			glUniformMatrix4fv( modelMatrixLoc_[config],1, 0, matModelView );
			
			if (g_bInitialized && g_mutex_trylock (g_pFrameBufferMutex))
			{
					//printf("binding frame texture \n");
					glBindTexture (GL_TEXTURE_2D, backgroundTextureHandle_[0]);
					glTexImage2D (GL_TEXTURE_2D,
					  0,
					  GL_RGB,
					  g_iWidth,
					  g_iHeight,
					  0,
					  GL_RGB,
					  GL_UNSIGNED_SHORT_5_6_5,
					  g_pcFrameBuffer);
		g_mutex_unlock (g_pFrameBufferMutex);
		
	}
		  drawPlane(bgPlane->vertex,bgPlane->vertexTexCoords,backgroundTextureHandle_[0]);
			drawPlane(filmPlane->vertex,filmPlane->vertexTexCoords,backgroundTextureHandle_[1]);
/*
			g_fCurrentTimeStamp = g_timer_elapsed (g_pTimerId, &ulMilliSeconds);
			fFrameTimeDelta = g_fCurrentTimeStamp - g_fLastTimeStamp;
			fFullSecond = g_fCurrentTimeStamp - g_fLastFullSecond;
			g_fLastTimeStamp = g_fCurrentTimeStamp;
*/
			}

			break;

		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}


void fslExampleCard::getRenderPhases(fslPhaseElementType* phaseArray) 
{
	if(phaseArray == NULL)
		return;
		
	//phaseArray is assumed fixed size of MAX_PHASE_ARRAY_SIZE
	//This card only has one stage
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_WRITEONLY;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[1].phaseID = 0;

	return;
}

float fslExampleCard::getAspectRatio()
{
	//This is only a hard-coded suggested value as the scene can really use whatever it wants
	return 4.0f/3.0f; 
}

//These events are ONLY card relevent;  All coords are in 0-1, 0-1, card space (lower left origin)
fslStatus fslExampleCard::handleInputEvent( fslInputEventType event)
{
	//No inputs for this card
	printf("input!!!");
	return FSL_STATUS_SUCCESS;
}


fslStatus fslExampleCard::drawPlane( float VertexPos[] , float TexCoords[], GLuint texture)
{
			
				
				glBindTexture( GL_TEXTURE_2D, texture );

				glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPos );
				glEnableVertexAttribArray( vertexLoc_ );

				glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, TexCoords);
				glEnableVertexAttribArray( texcoordLoc_ );

				//printf("Drawing...\n");
				glDrawArrays( GL_TRIANGLES, 0, 6 );

				glDisableVertexAttribArray( vertexLoc_ );
				glDisableVertexAttribArray( texcoordLoc_ );
	return FSL_STATUS_SUCCESS;
}











