//This define is to get eglImage working
#if USE_EGL_IMAGE
#define EGL_EGLEXT_PROTOTYPES 1
#define GL_GLEXT_PROTOTYPES 1
#endif

#include <GLES2/gl2.h>
#include <GLES2/gl2ext.h>
#include <EGL/egl.h>
#include <EGL/eglext.h>
#include <EGL/eglfslext.h>
#include <CARD/fslvideorendercard.h>
#include <CARD/plane.h>
#include <FSL/fslutil.h>
#include <stdbool.h>
#include <stdio.h>
#include <pthread.h>

int			g_iWidth = 320;
int			g_iHeight = 240;
GstElement*		g_pPipeline = NULL;
GstElement*		pVideo = NULL;
GstElement*		g_pPipelineH264 = NULL;
GstElement*		pVideoH264 = NULL;
gboolean		g_bInitialized = FALSE;
gboolean		g_bPlaying = FALSE;
gchar*			g_pcFrameBuffer = NULL;
GMutex*			g_pFrameBufferMutex = NULL;
GLfloat			g_afAlpha = 1.0f;
GLfloat			g_fMovieAspect = 1.0f;
GLuint			g_uiDisplayList = 0;
GLfloat			g_fOffset = 0.5f;
gdouble			g_fLastTimeStamp = 0.0f;
gdouble			g_fCurrentTimeStamp = 0.0f;
gdouble			g_fLastFullSecond = 0.0f;

GTimer*			g_pTimerId = NULL;

GLeglImageOES		g_imgHandle;
GThread myThread; //declares a Gobject thread
float g_matMV[16] = {0};

//function prototypes for all the gstreamer stuff
static gboolean bus_call (GstBus* pBus, GstMessage* pMessage,
			  gpointer data);
static void on_pad_added (GstElement* pDemuxer,
			  GstPad* pPad,
     GstElement* pDecoder);
static void on_handoff (GstElement* pFakeSink,
			GstBuffer* pBuffer,
   GstPad* pPad);
gboolean build_pipeline (gchar* pcFilename,
			 GMainLoop** pLoopH264,
    GstElement** pPipelineH264,
    GstBusFunc busCallbackFunc,
    GCallback padAddedFunc,
    GstPad** pVideoPadH264,
    GCallback handoffHandlerFunc);
gboolean spit_out_clip_stats (GstPad* pVideoPad);
void seek_to_time (GstElement* pPipeline, gint64 time_nanoseconds);

//starts the main loop
static gpointer loop_func(gpointer  data){

	GMainLoop* pLoopH264 = NULL;
	GstPad* pVideoPadH264 = NULL;
	gst_init (NULL, NULL);
	g_pFrameBufferMutex = g_mutex_new ();
	g_pTimerId = g_timer_new ();
        thread_data* loopData = (thread_data*)data;
			
	printf("about to build pipeline \n" );
	build_pipeline (loopData->fileName ,
			&pLoopH264,
   &g_pPipelineH264,
   bus_call,
   (GCallback) on_pad_added,
    &pVideoPadH264,
    (GCallback) on_handoff);
        //share the loop with the external world so it can be stopped
	loopData->mainloop = pLoopH264; 
	printf("pipeline built \n" );
	printf("g_pcFrameBuffer: 0x%08x \n", g_pcFrameBuffer );
	g_timeout_add (1000, (GSourceFunc) spit_out_clip_stats, pVideoPadH264);

#if USE_EGL_IMAGE
#else
	g_pcFrameBuffer = (gchar*)malloc(g_iWidth*g_iHeight*2);
#endif
	g_bInitialized = TRUE;
	
	/* Now set to playing and iterate. */
	gst_element_set_state (g_pPipelineH264, GST_STATE_PLAYING);
	printf("State set to PLAYING \n" );
	printf("about to run %08x\n",pLoopH264);

loop_run:
		g_main_loop_run (pLoopH264);

		printf("State set to NULL \n" );
		seek_to_time(g_pPipelineH264, 0);
		if(g_bPlaying == TRUE )
		{
			gst_element_set_state (g_pPipelineH264, GST_STATE_PLAYING);
			goto loop_run;
		}

		gst_object_unref (GST_OBJECT (g_pPipelineH264));
		g_pPipelineH264 = 0;
		g_main_loop_unref (pLoopH264);
		pLoopH264 = loopData->mainloop = NULL;

				return NULL;
}
//gets the pad width
gint get_pad_width (GstPad* pVideoPad)
{
	gint iWidth = 0;
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_int (pStructure, "width", &iWidth);
	}
	else
		g_print ("gst_pad_width() - Could not get caps for the pad!\n");

	return iWidth;
}
//gets the pad height
gint get_pad_height (GstPad* pVideoPad)
{
	gint iHeight = 0;
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_int (pStructure, "height", &iHeight);
	}
	else
		g_print ("gst_pad_height() - Could not get caps for the pad!\n");

	return iHeight;
}

//GETS THE FRAMES PER SECOND
gdouble get_pad_fps (GstPad* pVideoPad)
{
	gint aiFps[2] = {0, 0};
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;
	gdouble fFpsRate = 0.0f;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_fraction (pStructure,
					    "framerate",
	 &aiFps[0],
  &aiFps[1]);
		fFpsRate = (gdouble) aiFps[0] / (gdouble) aiFps[1];
	}
	else
		g_print ("gst_pad_fps() - Could not get caps for the pad!\n");

	return fFpsRate;
}
//gets the movie aspect
gdouble get_pad_movie_aspect (GstPad* pVideoPad)
{
	gint iWidth = 0;
	gint iHeight = 0;
	gint aiPixelAspect[2] = {0, 0};
	gdouble fAspectRatio = 0.0f;
	GstCaps* pCaps = NULL;
	GstStructure* pStructure = NULL;

	pCaps = gst_pad_get_negotiated_caps (pVideoPad);
	if (pCaps)
	{
		pStructure = gst_caps_get_structure (pCaps, 0);
		gst_structure_get_int (pStructure, "width", &iWidth);
		gst_structure_get_int (pStructure, "height", &iHeight);
		gst_structure_get_fraction (pStructure,
					    "pixel-aspect-ratio",
	 &aiPixelAspect[0],
  &aiPixelAspect[1]);

		if (aiPixelAspect[0] == 0)
			fAspectRatio = (gdouble) iWidth / (gdouble) g_iHeight;
		else
		{
			fAspectRatio = (gdouble) aiPixelAspect[0] /
					(gdouble) aiPixelAspect[1] *
					(gdouble) iWidth /
					(gdouble) iHeight;
		}
	}
	else
		g_print ("get_pad_movie_aspect() - Could not get caps for the pad!\n");

	return fAspectRatio;
}

//currently not working
gboolean spit_out_clip_stats (GstPad* pVideoPad)
{
	GLfloat afTexCoords[2] = {(GLfloat) g_iWidth, (GLfloat) g_iHeight};

	//g_print ("The video is %dx%d at %6.3f fps, movie-aspect is %3.2f:1",
			 //get_pad_width (pVideoPad),
			 //get_pad_height (pVideoPad),
			 //get_pad_fps (pVideoPad),
			 //get_pad_movie_aspect (pVideoPad));

	//g_mutex_lock (g_pFrameBufferMutex);
	//g_pcFrameBuffer = (gchar*)g_malloc0(240* 320* 3);
	//if (g_pcFrameBuffer)
	//	g_bInitialized = TRUE;
	//g_mutex_unlock (g_pFrameBufferMutex);

/*
	if (g_uiDisplayList)
	{
	glNewList (g_uiDisplayList, GL_COMPILE);
	glBegin (GL_QUADS);
	glNormal3f (0.0f, 0.0f, 1.0f);
	glTexCoord2f (afTexCoords[0], 0.0f);
	glVertex3f (g_fMovieAspect, 1.0f, 0.0f);
	glTexCoord2f (0.0f, 0.0f);
	glVertex3f (-g_fMovieAspect, 1.0f, 0.0f);
	glTexCoord2f (0.0f, afTexCoords[1]);
	glVertex3f (-g_fMovieAspect, -1.0f, 0.0f);
	glTexCoord2f (afTexCoords[0], afTexCoords[1]);
	glVertex3f (g_fMovieAspect, -1.0f, 0.0f);
	glEnd ();
	glEndList ();
}
	else
	g_print ("configure_event() - Crap, no display-list was generated!\n");
*/
	/* remove callback after the first call */
	return FALSE;
}

//get current position in the pipeline
gint64 get_pipeline_current_position (GstElement* pPipeline)
{
	GstFormat fmt = GST_FORMAT_TIME;
	gint64 iPosition = 0L;

	gst_element_query_position (pPipeline, &fmt, &iPosition);

	return iPosition;
}
//get the duration of the pipeline
gint64 get_pipeline_total_duration (GstElement* pPipeline)
{
	GstFormat fmt = GST_FORMAT_TIME;
	gint64 iLength = 0L;

	gst_element_query_duration (pPipeline, &fmt, &iLength);

	return iLength;
}
//seeks the pipeline to a given time
void seek_to_time (GstElement* pPipeline,
		   gint64 time_nanoseconds)
{
	GstSeekFlags seekFlags = (GstSeekFlags)(GST_SEEK_FLAG_FLUSH | GST_SEEK_FLAG_KEY_UNIT); 
	if (!gst_element_seek (pPipeline,
	     1.0,
      GST_FORMAT_TIME,
      seekFlags ,
      GST_SEEK_TYPE_SET,
      time_nanoseconds,
      GST_SEEK_TYPE_NONE,
      GST_CLOCK_TIME_NONE))
	{
		g_print ("Seek failed!\n");
	}
}

static void on_pad_added (GstElement* pDemuxer,
			  GstPad* pPad,
     GstElement* pDecoder)
{
	gchar* pcName = NULL;

	pcName = gst_pad_get_name (pPad);

	if (!g_ascii_strncasecmp (pcName, "video", 5))
		if (!gst_element_link_pads (pDemuxer, pcName, pDecoder, "sink"))
			g_print ("could not link demuxer with decoder\n");

	g_free (pcName);
}

//handoff function, called every frame
static void on_handoff (GstElement* pFakeSink,
			GstBuffer* pBuffer,
   GstPad* pPad)
{
	g_iWidth = get_pad_width (pPad);
	g_iHeight = get_pad_height (pPad);
	g_fMovieAspect = get_pad_movie_aspect (pPad);

	if (g_bInitialized)
	{
		gst_buffer_ref (pBuffer);
		g_mutex_lock (g_pFrameBufferMutex);
		g_memmove (g_pcFrameBuffer,
			   GST_BUFFER_DATA (pBuffer),
					    g_iWidth * g_iHeight * 2);
		//	printf("pbuffer copied %08x \n", g_pcFrameBuffer);
		g_mutex_unlock (g_pFrameBufferMutex);
		gst_buffer_unref (pBuffer);
	}
}


//given a filename and a gstreamer loop, builds the pipeline
gboolean build_pipeline (gchar* pcFilename,
			 GMainLoop** pLoopH264,
    GstElement** pPipelineH264,
    GstBusFunc busCallbackFunc,
    GCallback padAddedFunc,
    GstPad** pVideoPadH264,
    GCallback handoffHandlerFunc)
{
	GstElement* pPlayBin = NULL;
#if 0
	GstElement* pCapsFilter  = NULL;
#endif
	GstElement* pFakeSink = NULL;
	GstElement* pSinkBin = NULL;
	GstElement* pFfConv = NULL;
	GstPad*     pSinkPad = NULL;
	GstPad*     pFfConvSinkPad = NULL;
	char        aCharCwd[1024];
	char        aCharFileName[1024];


	if(!pcFilename)
	{
		printf("filename invalid\n");
                return FALSE;
 
	}
	
       //We need the absolute path to the file, so we will grab the cwd and combine it with the file name. 
	sprintf(aCharFileName, "file://%s/%s", getcwd(aCharCwd, sizeof(aCharCwd)), pcFilename);
	printf("%s\n", aCharFileName);

	//this is the loop where the pipeline will run
	*pLoopH264 = g_main_loop_new (NULL, FALSE);
	if (!pLoopH264)
	{
		printf("Couldn't create Loop\n");
		return FALSE;
	}
 
	//Instantiate the pipeline
	*pPipelineH264 = gst_pipeline_new ("pipeline-h264");
	if (!*pPipelineH264)
	{
		printf("Couldn't create pipeline\n");
		return FALSE;
	}

	gst_bus_add_watch (gst_pipeline_get_bus (GST_PIPELINE (*pPipelineH264)),
			   busCallbackFunc, *pLoopH264);


	//Instantiate the Playbin2 super element
	pPlayBin = gst_element_factory_make ("playbin2", "PlayBin");
	if (!pPlayBin)
	{
		printf("Couldn't create pPlayBin\n");
		return FALSE;
	}
	g_object_set (G_OBJECT (pPlayBin), "uri", aCharFileName, NULL);


	//Add forcing to use ffmpegconvert to help playbin out with the decision making
	pFfConv = gst_element_factory_make ("ffmpegcolorspace", "ffconv");
	if (!pFfConv)
	{
		printf("Couldn't create pFfConv\n");
		return FALSE;
	}

#if 0
	//Setup the filter to force RGB output
        pCapsFilter = gst_element_factory_make ("capsfilter", "CapsFilter");
	if (!pCapsFilter)
	{
		printf("Couldn't create pCapsFilter\n");
		return FALSE;
	}
	g_object_set (G_OBJECT (pCapsFilter), "caps", gst_caps_new_simple ("video/x-raw-rgb","bpp",G_TYPE_INT,16, NULL), NULL);
#endif

	//Setup the FakeSink plugin
	pFakeSink = gst_element_factory_make ("fakesink", "FakeSink");
	if (!pFakeSink)
	{
		printf("Couldn't create pFakeSink\n");
		return FALSE;
	}
	g_object_set (G_OBJECT (pFakeSink), "sync", TRUE, "signal-handoffs", TRUE, NULL);
	g_signal_connect (pFakeSink, "handoff", handoffHandlerFunc, NULL);

	//Put the fake sink and caps fiter into a single bin
	pSinkBin = gst_bin_new("SinkBin");
	if (!pSinkBin)
	{
		printf("Couldn't create pSinkBin\n");
		return FALSE;
	}
	gst_bin_add_many (GST_BIN (pSinkBin), pFfConv, pFakeSink, NULL);
 	gst_element_link_filtered (pFfConv, pFakeSink, gst_caps_new_simple ("video/x-raw-rgb","bpp",G_TYPE_INT,16, NULL));

	//In order to link the sink bin to the play been we have to create
	//a ghost pad that points to the capsfilter sink pad
	pFfConvSinkPad = gst_element_get_static_pad(pFfConv, "sink");
	if (!pFfConvSinkPad)
	{
		printf("Couldn't create pFfCovSinkPad\n");
		return FALSE;
	}

	pSinkPad = gst_ghost_pad_new( "sink", pFfConvSinkPad );
	if (!pSinkPad)
	{
		printf("Couldn't create pSinkPad\n");
		return FALSE;
	}
	gst_element_add_pad(pSinkBin, pSinkPad);
        gst_object_unref( pFfConvSinkPad);
 
	// force the SinkBin to be used as the video sink
	g_object_set (G_OBJECT (pPlayBin), "video-sink", pSinkBin, NULL);

	//Link the filesrc and PlayBin together to make the pipeline
	gst_bin_add (GST_BIN (*pPipelineH264), pPlayBin);

	printf("full pipeline built \n");
	return TRUE;
}
static const char* g_strEnd =
		"}													\n";

static const char* g_strVSProgramA = 
		//"uniform   mat4 g_matModelView;						\n"
		"													\n"
		"attribute vec4 g_vVertex;							\n"
		"attribute vec4 g_vTexcoord;						\n"
		"varying   vec4 g_vVSTexcoord;						\n"
		"													\n";

static const char* g_strVSProgramB = 
		"void main()										\n"
		"{													\n"	

    //"    vec4 vPositionES = g_matModelView * g_vVertex; \n"
		"    vec4 cardPosition = g_vVertex;   \n"

		"    g_vVSTexcoord = g_vTexcoord;					\n";
		

//What vars should scene control?
//g_strVSProgram = g_strVSProgramA + SceneA + g_strVSProgramB + SceneB + g_strEnd;
//SceneB =   "    gl_Position  = cardPosition;		\n"
static const char* g_strFSProgramA = 
		"#ifdef GL_FRAGMENT_PRECISION_HIGH									\n"
		"   precision highp float;											\n"
		"#else																\n"
		"   precision mediump float;										\n"
		"#endif																\n"
		"																	\n"
		"uniform   sampler2D g_sImageTexture;								\n"
		"varying   vec4      g_vVSTexcoord;									\n"
		"																	\n";

static const char* g_strFSProgramB = 
		"void main()														\n"
		"{																	\n"
		"    vec4 cardColor;	\n"		
//		"    cardColor.r = 0.0;	\n"		
//		"    cardColor.g = 0.5;	\n"		
//		"    cardColor.b = 0.3;	\n"
//		"    cardColor.a = 1.0;	\n"		
		"    cardColor = texture2D(g_sImageTexture, g_vVSTexcoord.xy);	\n";

 
    

//g_strFSProgram = g_strFSProgramA + SceneA + g_strFSProgramB + SceneB + g_strEnd;
//SceneB =   "    gl_FragColor = cardColor;	\n";





Plane *bgPlane = 0; //creates a plane object for the background, this is where  the video will play



static GMainLoop *loop;

static gboolean bus_call (GstBus* pBus,
			  GstMessage* pMessage,
     gpointer data)
{
	//this is where g streamer gets the events, or calls
	GMainLoop* pLoop = (GMainLoop*)data;

	switch (GST_MESSAGE_TYPE (pMessage))
	{
		case GST_MESSAGE_EOS:
			g_print ("End-of-stream\n");
			g_main_loop_quit (pLoop);
			break;

		case GST_MESSAGE_ERROR:
		{
			gchar* pcDebug = NULL;
			GError* pError = NULL;

			gst_message_parse_error (pMessage, &pError, &pcDebug);
			g_free (pcDebug);

			g_print ("Error: %s\n", pError->message);
			g_error_free (pError);

			g_main_loop_quit (pLoop);
			break;
		}

		default:
			break;
	}

	return TRUE;
}



fslVideoRenderCard::fslVideoRenderCard( )
{
	parentScene_ = NULL;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	return;
}

fslVideoRenderCard::fslVideoRenderCard( void* parentscene )
{
	parentScene_ = parentscene;
	vertexLoc_ = 0;
	texcoordLoc_ = 1;
	return;
}

fslVideoRenderCard::~fslVideoRenderCard()
{
	if (g_bInitialized)
	{
		g_mutex_lock (g_pFrameBufferMutex);
		free (g_pcFrameBuffer);
		g_mutex_unlock (g_pFrameBufferMutex);
		g_bInitialized = FALSE;

	}

	g_mutex_free (g_pFrameBufferMutex);
	return;
}

//This should probably be made to work with 'n' shaders
fslStatus fslVideoRenderCard::initCard( fslInitStageType init, unsigned int config) 
{
	static GLuint hVertexShader;
	static GLuint hFragmentShader;
	selectedChapter = 0;
	bgPlane = new Plane();//new plane, a plane consists of two triangles
	
	bgPlane->createPlane(1.3,1.0); //creates a plane that is 1.3width and 1.0 height

	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
		{
			GLint nCompileResult;
			char vs[MAX_SHADER_LENGTH];
			char ps[MAX_SHADER_LENGTH];
			const char* pvs = (char*)(vs); // This is kind of silly and I'm not sure why this is being required in C++ and not C
			const char* pps = (char*)(ps); //  may also be solved with &(const_cast<char*>(vs))

				//vs = new char[MAX_SHADER_LENGTH];
				//ps = new char[MAX_SHADER_LENGTH];

			printf("Starting Initialization \n");

				//This would probably be better handled with string objects
				//creating fragment and vertex shader strings
			sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
			sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				//Could probably add more code to make this less intensive on subsequent inits 
			printf("About to load texture! \n");

#if USE_EGL_IMAGE
			if(config == 0)  // This is a hack.  The object should detect if it has been already inited and try to make a new thread
			{
				EGLint attribs[] = { EGL_WIDTH, 320,
              						EGL_HEIGHT, 240,
							EGL_IMAGE_FORMAT_FSL, EGL_FORMAT_RGB_565_FSL,
							EGL_NONE};	

				glGenTextures(1, &backgroundTextureHandle_[config]);
				printf ("init --> dpy_: 0x%08x\n", dpy_); 
                        	g_imgHandle = eglCreateImageKHR(dpy_, EGL_NO_CONTEXT, EGL_NEW_IMAGE_FSL, NULL, attribs);
				printf ("init --> g_imgHandle: 0x%08x\n", g_imgHandle); 
                                eglQueryImageFSL(dpy_, g_imgHandle, EGL_CLIENTBUFFER_TYPE_FSL, (EGLint*)&g_pcFrameBuffer);
				glBindTexture (GL_TEXTURE_2D, backgroundTextureHandle_[0]);
				glEGLImageTargetTexture2DOES(GL_TEXTURE_2D, g_imgHandle);
				
				printf ("init --> g_pcFrameBuffer: 0x%08x\n", g_pcFrameBuffer); 

			}
#else
			if(!fslInit2DCTESTextureGL( "media/textures/Background.CTES", &backgroundTextureHandle_[config] ))
				return FSL_STATUS_GENERAL_ERROR;

			if(!fslInit2DCTESTextureGL( "media/textures/frame256x256.CTES", &backgroundTextureHandle_[1] ))
				return FSL_STATUS_GENERAL_ERROR;
		
#endif
			printf("Vertex Shader = \n %s \n", vs);
			printf("Pixel Shader = \n %s \n", ps);

					// Compile the shaders
			printf("Compiling Shaders... \n");
			hVertexShader = glCreateShader( GL_VERTEX_SHADER );
			glShaderSource( hVertexShader, 1, &pvs, NULL );
			glCompileShader( hVertexShader );

			hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
			glShaderSource( hFragmentShader, 1, &pps, NULL );
			glCompileShader( hFragmentShader );

				// Check for compile success
			nCompileResult = 0;
			glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
			if (!nCompileResult)
			{
				char Log[1024];
				GLint nLength;
				printf("Error compiling shader! \n");
				glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
				return FSL_STATUS_GENERAL_ERROR;
			}

				// Attach the individual shaders to the common shader program
			shaderProgram_[config] = glCreateProgram();
			glAttachShader( shaderProgram_[config], hVertexShader );
			glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
			glBindAttribLocation(shaderProgram_[config], vertexLoc_,   "g_vVertex");
			glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "g_vTexcoord");
				
			numShaderAttributes_ = 2; //The scene can use attributes higher than this

		}
 		
		break;
		case FSL_INIT_LINK:
		{
			GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
			printf("Linking videoRender Shaders... \n");
			glLinkProgram( shaderProgram_[config] );

				// Check for link success
			nLinkResult = 0;
			glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);
			if (!nLinkResult)
			{
				char Log[1024];
				GLint nLength;
				printf("Error linking shader! \n");
				glGetProgramInfoLog(shaderProgram_[config], 1024, &nLength, Log);
				return FSL_STATUS_GENERAL_ERROR;
			}

				
			modelMatrixLoc_[config] = glGetUniformLocation( shaderProgram_[config], "g_matModelView" );

			glDeleteShader( hVertexShader );
			glDeleteShader( hFragmentShader );
				//printf("Finished Initialization... \n");	
				
			if(config == 0)  // [Brian] This is a hack.  The object should detect if it has been already inited and try to make a new thread
			{
				g_thread_init (NULL);
				
			}

				
		}			
		break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	
	return FSL_STATUS_SUCCESS;
}

//this is in case you want to implement 3D picking in the scene, the demo only uses 2D 
//coords for now, but the structure is ready for ray-casting picking

/*I know this is not correct, but I am having a problem if I declare/initialize this
 arrays in the function, they do not seem to initialize properly and I get garbage, Paco had this problem too
-guillermo	 */
float p[3] = {0.0, 0.0, 0.0};
float d[3] = {0.0, 0.0, 0.0};
float v0[3]= {0.0, 0.0, 0.0};
float v1[3] = {0.0, 0.0, 0.0};
float v2[3]={0.0, 0.0, 0.0};

float tmpV0[3]={0.0, 0.0, 0.0};
float tmpV1[3]={0.0, 0.0, 0.0};
float tmpV2[3] = {0.0, 0.0, 0.0};
float tmpV3[3]={0.0, 0.0, 0.0};
float tmpV4[3]={0.0, 0.0, 0.0};
float tmpV5[3] = {0.0, 0.0, 0.0};

//this method is for an intersection test, given a ray and a plane
bool fslVideoRenderCard::intersects( float nearX, float nearY,float nearZ,float farX, float farY,float farZ)
{

	
	p[0]=nearX;
	p[1]=nearY;
	p[2]=nearZ;
	
	
	d[0]=(farX-nearX);
	d[1]=(farY-nearY);
	d[2]=(farZ-nearZ);
	
	fslNormalize(d);
		
	tmpV0[0]=bgPlane->vertex[0];//get the Plane coordinates
	tmpV0[1]=bgPlane->vertex[1];
	tmpV0[2]=bgPlane->vertex[2];
	
	tmpV1[0]=bgPlane->vertex[4];
	tmpV1[1]=bgPlane->vertex[5];
	tmpV1[2]=bgPlane->vertex[6];
	
	tmpV2[0]=bgPlane->vertex[8];
	tmpV2[1]=bgPlane->vertex[9];
	tmpV2[2]=bgPlane->vertex[10];
	
	tmpV3[0]=bgPlane->vertex[12];
	tmpV3[1]=bgPlane->vertex[13];
	tmpV3[2]=bgPlane->vertex[14];
	
	tmpV4[0]=bgPlane->vertex[16];
	tmpV4[1]=bgPlane->vertex[17];
	tmpV4[2]=bgPlane->vertex[18];
	
	tmpV5[0]=bgPlane->vertex[20];
	tmpV5[1]=bgPlane->vertex[21];
	tmpV5[2]=bgPlane->vertex[22];

	
	//printf("about to check intersection p(%f,%f,%f) d(%f,%f,%f) v0(%f,%f,%f) v1(%f,%f,%f) v2(%f,%f,%f) \n",p[0],p[1],p[2],d[0],d[1],d[2],tmpV0[0],tmpV0[1],tmpV0[2],tmpV1[0],tmpV1[1],tmpV1[2],tmpV2[0],tmpV2[1],tmpV2[2]);
	
	//checks intersection with the first triangle of the plane, then tests the second
	if(fslRayIntersectsTriangle(p,d,tmpV0,tmpV1,tmpV2)||fslRayIntersectsTriangle(p,d,tmpV3,tmpV4,tmpV5))
		return true;
	else	
		return false;
}
fslStatus fslVideoRenderCard::shutdownCard() //Releases dynamic resources without destroying card
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoRenderCard::resumeCard()   //Reallocates resources for card; probably the same as initCard for most cards
{
	//TBD
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoRenderCard::animTick(void) //adjusts state by the time tick set in resolution
{
//no animation here
	

	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoRenderCard::setTickRes(float time)
{
	//This card currently has no animation
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoRenderCard::setTime(float time)
{
	//This card currently has no animation
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

float fslVideoRenderCard::getTime()
{
	return currentTime_;
}

fslStatus fslVideoRenderCard::preRenderCard(unsigned int phaseID, unsigned int config)
{
	switch(phaseID)
	{
	case 1:
            //printf(" Set the shader program and the texture \n");

			glUseProgram( shaderProgram_[config] );
			break;
		default:
			return FSL_STATUS_BAD_PARAMETER;
	}

	return FSL_STATUS_SUCCESS;

}

fslStatus fslVideoRenderCard::renderCard( unsigned int phaseID, unsigned int config )  
{
	//Broken into phases so master render function doesn't get things out of order (alpha, etc.)
	//PhaseID are set by the card in the getRenderPhases
	//Scenes call getRenderPhases to get stages needed by card
	//Scenes then call renderCard with the IDs they got from getRenderPhases grouped with other
	//  cards by the type
	int textureID = 0;
	
	switch(phaseID)
	{
		case 1:
		{




			//chapter frame
			glUseProgram( shaderProgram_[config] );
			
			// if everything is right, bind video texture to plane
			if (g_bInitialized && g_mutex_trylock (g_pFrameBufferMutex))
			{
#if USE_EGL_IMAGE
				glBindTexture (GL_TEXTURE_2D, backgroundTextureHandle_[0]);

#else
					//printf("binding frame texture \n");
				glBindTexture (GL_TEXTURE_2D, backgroundTextureHandle_[0]);

				glTexImage2D (GL_TEXTURE_2D, 0,
	   				GL_RGB,
    					g_iWidth,
    					g_iHeight,
    					0,
    					GL_RGB,
    					GL_UNSIGNED_SHORT_5_6_5,
    					g_pcFrameBuffer);
#endif
				g_mutex_unlock (g_pFrameBufferMutex);
			}
			drawPlane(bgPlane->vertex,bgPlane->vertexTexCoords,backgroundTextureHandle_[0]);
		}

		break;

		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}

void fslVideoRenderCard::getRenderPhases(fslPhaseElementType* phaseArray) //assumed fixed size of MAX_PHASE_ARRAY_SIZE
{
	if(phaseArray == NULL)
		return;
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_READWRITE;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[1].phaseID = 0;

	return;
}

float fslVideoRenderCard::getAspectRatio()
{
	//This is only a hard-coded suggested value as the scene can really use whatever it wants
	return 4.0f/3.0f; 
}

//These events are ONLY card relevent;  All coords are in 0-1, 0-1, card space (lower left origin)
fslStatus fslVideoRenderCard::handleInputEvent( fslInputEventType event)
{
	
		

	

	
	switch (event.input){
		case FSL_INPUT_PAUSE:
			//pauses the gstreamer pipeline
			gst_element_set_state (g_pPipelineH264, GST_STATE_PAUSED);
			break;
		case FSL_INPUT_PLAY:
			//sets gstreamer pipeline to play
			if(g_bPlaying == TRUE)
				gst_element_set_state (g_pPipelineH264, GST_STATE_PLAYING);
			else
			{
				//Test if a file name has been set yet
				//otherwise don't start the player
				if(videoFileName_[0])
				{
			                g_bPlaying = TRUE;
					strcpy(playerThreadData_.fileName, videoFileName_);
					g_thread_create(loop_func,&playerThreadData_,FALSE,NULL);
				}
			}
				
			break;	
		case FSL_INPUT_STOP:
			//stops the gstreamer pipeline
			printf("Stopping\n");
			if(g_pPipelineH264)
			{
				g_bPlaying = FALSE;
				gst_element_set_state (g_pPipelineH264, GST_STATE_NULL);
			}
			if(playerThreadData_.mainloop)
			{		
				g_main_loop_quit(playerThreadData_.mainloop);
			}
			break;	
		case FSL_INPUT_CLOSEWINDOW:
			printf("Shutting down video\n");
			if(g_pPipelineH264)
			{
				gst_element_set_state (g_pPipelineH264, GST_STATE_NULL);
				gst_object_unref (GST_OBJECT (g_pPipelineH264));
				g_pPipelineH264 = 0;
			}
			break;
		case FSL_INPUT_SEEK:
			//seeks to a given chapter
			printf("seek to chapter %i \n",selectedChapter);
			seekToChapter(selectedChapter);	
			break;	
		default:
			break;
	}

	return FSL_STATUS_SUCCESS;
}

int num_chapters = 4;
gint64 duration, position;
GstFormat format;

fslStatus fslVideoRenderCard::setChapter(int c)
{
	selectedChapter = c;	
	
	return FSL_STATUS_SUCCESS;
}
fslStatus fslVideoRenderCard::seekToChapter(int chapter)
{
	float seek = 0;
	//this will break the video in a given set of chapters
	
	seek = (100/num_chapters)*chapter; //divides the chapters with even duration
	/* get the duration */
	format = GST_FORMAT_TIME;
	position = 1 * GST_SECOND; //gstreamer seconds
	
	gst_element_query_duration (g_pPipelineH264, &format, &duration);
	position = duration * seek / 100; //asks the duration and converts to a position
	
	gst_element_seek_simple (g_pPipelineH264, GST_FORMAT_TIME, GST_SEEK_FLAG_FLUSH,
				 position); //finally seeks to the position
	return FSL_STATUS_SUCCESS;
}


//this method recieves a vertexarray (a plane) and a handler to a texture
fslStatus fslVideoRenderCard::drawPlane( float VertexPos[] , float TexCoords[], GLuint texture)
{
			
				
	glBindTexture( GL_TEXTURE_2D, texture ); //binds the txture
	
	//points to the vertexshader
	glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPos );
	glEnableVertexAttribArray( vertexLoc_ );
	//texture coordinates
	glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, TexCoords);
	glEnableVertexAttribArray( texcoordLoc_ );

	//printf("Drawing...\n");
	//draws the plane, which consists of 2 triangles
	glDrawArrays( GL_TRIANGLES, 0, 6 );
	//disable client state
	glDisableVertexAttribArray( vertexLoc_ );
	glDisableVertexAttribArray( texcoordLoc_ );
	return FSL_STATUS_SUCCESS;
}

//recieves a filenename of the current video
fslStatus fslVideoRenderCard::setVideoFile( char* filename )
{
	fslInputEventType inputEvent;
	printf("Setting Video Render to %s \n", filename);
	sprintf(videoFileName_, "%s", filename);
	inputEvent.input = FSL_INPUT_STOP;
	handleInputEvent( inputEvent );
	inputEvent.input = FSL_INPUT_PLAY;
	handleInputEvent( inputEvent );
	
	//May want to fail on bad filename
	return FSL_STATUS_SUCCESS;	
}








