#include <FSL/c2d_api.h>
#include "EGL/egl.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/fb.h>
#include <unistd.h>


#define BYTESPP(f) ((f == C2D_COLOR_0565) ? 2 : 4)
#define WHITE_565 0xFFFF

typedef int (*COLORCHECKFN)(unsigned char*, C2D_COLORFORMAT);

void loadDrawStateDefaults(C2D_CONTEXT ctx)
{

	c2dSetBrushSurface(ctx,0,0);
	c2dSetMaskSurface(ctx,0,0);
	c2dSetDstRectangle(ctx,NULL);
	c2dSetSrcRectangle(ctx,NULL);
	c2dSetDstClipRect(ctx,0);

	c2dSetBlendMode(ctx,C2D_ALPHA_BLEND_NONE);
	c2dSetGlobalAlpha(ctx,0xFF);

	c2dSetSrcColorkey(ctx,0,0);
	c2dSetDstColorkey(ctx,0,0);

	c2dSetSrcRotate(ctx,0);
	c2dSetDstRotate(ctx,0);

	c2dSetRop(ctx,0xCCCC);

	c2dSetFgColor(ctx,0x00000000);
	c2dSetBgColor(ctx,0x00000000);

	c2dSetGradientDirection(ctx,C2D_GD_LEFT_RIGHT);
	c2dSetStretchMode(ctx,C2D_STRETCH_POINT_SAMPLING);


}

#define C2DR(ARG) if(ARG != C2D_STATUS_OK ){ printf("C2D Status error (0x%x) line %d\n",ARG,__LINE__); return 1; }

int main(int argc, char **argv)
{
       	int fb_handler = open("/dev/fb0",O_RDWR);
	void* physical_address;
	struct fb_fix_screeninfo fixinfo;
	struct fb_var_screeninfo varinfo;
	int rectWidth = 266; // one third of screen size
	int rectHeight = 160;
	void* pBuff;
	//unsigned int source_surface [rectWidth * rectHeight];

    	C2D_CONTEXT DrawContext;
   	C2D_RECT Rect = {0,0,rectWidth,rectHeight};
	C2D_SURFACE_DEF SurfaceSrcDef = {C2D_COLOR_8888, rectWidth, rectHeight,0 ,0 ,0 ,0 };
	C2D_SURFACE_DEF SurfaceDstDef;
	C2D_SURFACE m_srcSurf = NULL;
	C2D_SURFACE m_destSurf = NULL;

	ioctl(fb_handler,FBIOGET_FSCREENINFO,&fixinfo);
	ioctl(fb_handler,FBIOGET_VSCREENINFO,&varinfo);
	physical_address = mmap(0,fixinfo.smem_len,PROT_WRITE,MAP_SHARED,fb_handler,0);

	//define destination surface (screen)
	SurfaceDstDef.width = varinfo.xres;
	SurfaceDstDef.height = varinfo.yres;
	SurfaceDstDef.stride = fixinfo.line_length;
	SurfaceDstDef.buffer = (void*)fixinfo.smem_start;
	SurfaceDstDef.host = (void*) physical_address; 
	SurfaceDstDef.flags = C2D_SURFACE_NO_BUFFER_ALLOC;

	switch( varinfo.bits_per_pixel )
	{
		case 16:  SurfaceDstDef.format =  C2D_COLOR_0565; break;
		case 24:  SurfaceDstDef.format =  C2D_COLOR_888; break;
		case 32:  SurfaceDstDef.format =  C2D_COLOR_8888; break;
		default: printf("Framebuffer format not supported\n"); return 1;
	}

	printf("creating contexts... \n");
	C2DR( c2dCreateContext(&DrawContext) )
	C2DR( c2dSurfAlloc(DrawContext, &m_srcSurf, &SurfaceSrcDef) )
	C2DR( c2dSurfAlloc(DrawContext, &m_destSurf, &SurfaceDstDef) )

	printf("load initial defaults...\n");
	loadDrawStateDefaults(DrawContext);

	//clear source surface to white
	//this can also be done with C2D commands, but is done with memset
	//here to show buffer mapping for memory use by user applications
	pBuff = NULL;
	C2DR( c2dSurfLock(DrawContext, m_srcSurf, &pBuff) ) 
	if( pBuff )
		//The stride gets filled in by the c2dSurfaceAlloc 
		memset(pBuff,0xFFFFFFFF,rectHeight*SurfaceSrcDef.stride);
	else
	{
		printf("Error Locking Source Surface!\n");
		return 1;
	}
	C2DR( c2dSurfUnlock(DrawContext, m_srcSurf) )

	//clear destination (screen) to black
	pBuff = NULL;
	C2DR( c2dSurfLock(DrawContext, m_destSurf, &pBuff) )
	if( pBuff )
		memset(pBuff,0x00000000,varinfo.xres*varinfo.yres*(varinfo.bits_per_pixel/8));
	else
	{
		printf("Error Locking Destination Surface!\n");
		return 1;
	}
	C2DR( c2dSurfUnlock(DrawContext, m_destSurf) )

	printf("set raster operations \n");
	C2DR( c2dSetRop(DrawContext,0xF0F0) ) //Raster operation

	//set destination and source surfaces
	C2DR( c2dSetSrcSurface(DrawContext, m_srcSurf) )
	C2DR( c2dSetDstSurface(DrawContext, m_destSurf) )
	C2DR( c2dSetDstRectangle(DrawContext,&Rect) )
	C2DR( c2dSetSrcRectangle(DrawContext,&Rect) )

	printf("flush drawing operations \n");
	C2DR( c2dDrawBlit(DrawContext) )  //blit surfaces
	C2DR( c2dFinish(DrawContext) )

	sleep(1);

	printf("destroying contexts...\n");

	c2dSurfFree(DrawContext,m_destSurf);
     	c2dDestroyContext(DrawContext);

	return 1;

}




