#include <FSL/c2d_api.h>
#include "EGL/egl.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <fcntl.h>
#include <sys/mman.h>
#include <linux/fb.h>
#include <unistd.h>


#define BYTESPP(f) ((f == C2D_COLOR_0565) ? 2 : 4)
#define WHITE_565 0xFFFF

typedef int (*COLORCHECKFN)(unsigned char*, C2D_COLORFORMAT);

#define SCREEN_HEIGHT 800
#define SCREEN_WIDTH	480


void loadDrawStateDefaults(C2D_CONTEXT ctx)
{

	c2dSetBrushSurface(ctx,0,0);
	c2dSetMaskSurface(ctx,0,0);
	c2dSetDstRectangle(ctx,NULL);
	c2dSetSrcRectangle(ctx,NULL);
	c2dSetDstClipRect(ctx,0);

	c2dSetBlendMode(ctx,C2D_ALPHA_BLEND_NONE);
	c2dSetGlobalAlpha(ctx,0xFF);

	c2dSetSrcColorkey(ctx,0,0);
	c2dSetDstColorkey(ctx,0,0);

	c2dSetSrcRotate(ctx,0);
	c2dSetDstRotate(ctx,0);

	c2dSetRop(ctx,0xCCCC);

	c2dSetFgColor(ctx,0x00000000);
	c2dSetBgColor(ctx,0x00000000);

	c2dSetGradientDirection(ctx,C2D_GD_LEFT_RIGHT);
	c2dSetStretchMode(ctx,C2D_STRETCH_POINT_SAMPLING);


}


int main(int argc, char **argv)
{

 	int fb_handler = open("/dev/fb0",O_RDWR);
	void *pa;
	struct fb_fix_screeninfo fixinfo;
	struct fb_var_screeninfo varinfo;
	int rectWidth = 800; // one third of screen size
	int rectHeight = 480;


	C2D_CONTEXT DrawContext;
	C2D_RECT Rect = {0,0,rectWidth,rectHeight};

	ioctl(fb_handler,FBIOGET_FSCREENINFO,&fixinfo);
	ioctl(fb_handler,FBIOGET_VSCREENINFO,&varinfo);
	pa=mmap(0,fixinfo.smem_len,PROT_WRITE,MAP_SHARED,fb_handler,0);


	C2D_SURFACE_DEF SurfaceDstDef = \
		{C2D_COLOR_8888, varinfo.xres,varinfo.yres,fixinfo.line_length, (void*) fixinfo.smem_start, (void*)pa, C2D_SURFACE_NO_BUFFER_ALLOC};
	C2D_SURFACE m_destSurf = NULL;

	void* values_src;
	unsigned long rgba3[] ={0x00FF0000,0x0000FF00,0x000000FF};
	int i, colorIndex = 0;

	for(i=0 ; i < 3 ; i++)
	{
		printf("creating contexts... \n");
		c2dCreateContext(&DrawContext);

		c2dSurfAlloc(DrawContext, &m_destSurf, &SurfaceDstDef);
		printf("load initial defautls...\n");

		//initial set
		loadDrawStateDefaults(DrawContext);
		c2dSurfLock(DrawContext, m_destSurf,&values_src);

		//black dest surface
		memset(values_src,0x000000,varinfo.xres*varinfo.yres*4);
		c2dSurfUnlock(DrawContext, m_destSurf);

		//sets raster operations and source and destination surfaces
		printf("set raster operations \n");
		c2dSetRop(DrawContext,0xF0F0);
		c2dSetDstSurface(DrawContext, m_destSurf);
		c2dSetDstRectangle(DrawContext,&Rect);

		//draw rectangle
		printf("drawing gradient... color 0x%8lx:\n",rgba3[colorIndex]);
		c2dSetFgColor(DrawContext,rgba3[colorIndex]);
		if(  c2dDrawRect(DrawContext,C2D_PARAM_GRADIENT_BIT) != C2D_STATUS_OK)
			printf("error drawing rectangle \n");

		//flush all operations
		printf("flush drawing operations \n");

		c2dFinish(DrawContext);

		colorIndex++;
		sleep(1);

		printf("destroying contexts...\n");

		c2dSurfFree(DrawContext,m_destSurf);
		c2dDestroyContext(DrawContext);

 	}

	return 1;
}




