#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <assert.h>
#include <math.h>
#ifndef USE_MX31

#include "GLES2/gl2.h"
#include "GLES2/gl2ext.h"
#include "EGL/egl.h"
#else
#include <GLES/gl.h>
#include <GLES/egl.h>
#endif
#ifdef EGL_USE_X11
#include <X11/X.h>
#include <X11/Xlib.h>
#endif 


#define TRUE 1
#define FALSE !TRUE
EGLDisplay			egldisplay;
EGLConfig			eglconfig;
EGLSurface			eglsurface;
EGLContext			eglcontext;

#ifdef EGL_USE_X11
Display 			*display;
#endif

int				currentFrame = 0;

GLuint       g_hShaderProgram      = 0;
GLuint       g_hModelViewMatrixLoc = 0;
GLuint       g_hProjMatrixLoc      = 0;
GLuint       g_hVertexLoc          = 0;
GLuint       g_hColorLoc           = 1;


//--------------------------------------------------------------------------------------
// Name: g_strVertexShader / g_strFragmentShader
// Desc: The vertex and fragment shader programs
//--------------------------------------------------------------------------------------
const char* g_strVertexShader = 
    "uniform   mat4 g_matModelView;				\n"
    "uniform   mat4 g_matProj;					\n"
    "								\n"
    "attribute vec4 g_vPosition;				\n"
    "attribute vec3 g_vColor;					\n"
    "								\n"
    "varying   vec3 g_vVSColor;					\n"
    "								\n"
    "void main()						\n"
    "{								\n"
    "    vec4 vPositionES = g_matModelView * g_vPosition;	\n"
    "    gl_Position  = g_matProj * vPositionES;		\n"
    "    g_vVSColor = g_vColor;					\n"
    "}								\n";


const char* g_strFragmentShader = 
    "#ifdef GL_FRAGMENT_PRECISION_HIGH				\n"
    "   precision highp float;					\n"
    "#else							\n"
    "   precision mediump float;				\n"
    "#endif							\n"
    "								\n"
    "varying   vec3      g_vVSColor;				\n"
    "								\n"
    "void main()						\n"
    "{								\n"
    "    gl_FragColor = vec4( g_vVSColor, 1.0 );		\n"
    "}								\n";


    // Define vertice for a 4-sided pyramid
    float VertexPositions[] =
    {
         0.0f,1.0f, 0.0f,     
	-1.0f,-1.0f, 0.0f,     
	+1.0f,-1.0f, 0.0f,
	 1.0f, 1.0f, 0.0f,
        -1.0f,1.0f, 0.0f,
         1.0f, -1.0, 0.0f		
    };

void render(float w, float h)
{
  static float fAngle = 0.0f;
    fAngle += 0.01f;
	
 

    // Rotate and translate the model view matrix
    float matModelView[16] = {0};
    matModelView[ 0] = 1.0;
    matModelView[ 2] = 0.0f;
    matModelView[ 5] = 1.0f;
    matModelView[ 8] = 0.0f;
    matModelView[10] = 0.0f;
    matModelView[14] = -4.0f;//translation Z
    matModelView[15] = 1.0f;

    // Build a perspective projection matrix
    float matProj[16] = {0};
    matProj[ 0] = cosf( 0.5f ) / sinf( 0.5f );
    matProj[ 5] = matProj[0] * (w/h) ;
    matProj[10] = -( 10.0f ) / ( 9.0f );
    matProj[11] = -1.0f;
    matProj[14] = -( 10.0f ) / ( 9.0f );


   // Clear the colorbuffer and depth-buffer
    glClearColor( 0.0f, 0.0f, 0.5f, 1.0f );
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );

    // Set some state
    glEnable( GL_DEPTH_TEST );
  
    // Set the shader program
    glUseProgram( g_hShaderProgram );
    glUniformMatrix4fv( g_hModelViewMatrixLoc, 1, 0, matModelView );
    glUniformMatrix4fv( g_hProjMatrixLoc,      1, 0, matProj );

    // Bind the vertex attributes
    glVertexAttribPointer( g_hVertexLoc, 3, GL_FLOAT, 0, 0, VertexPositions );
    glEnableVertexAttribArray( g_hVertexLoc );

   

   glDrawArrays( GL_TRIANGLES, 0, 3 );

    // Cleanup
    glDisableVertexAttribArray( g_hVertexLoc );
 

}


int init(void)
{

	/*static const EGLint gl_context_attribs[] =
	{
      		EGL_CONTEXT_CLIENT_VERSION, 2,
      		EGL_NONE
    	};*/

	static const EGLint s_configAttribs[] =
	{
		EGL_RED_SIZE,		5,
		EGL_GREEN_SIZE, 	6,
		EGL_BLUE_SIZE,		5,
		EGL_ALPHA_SIZE, 	0,
		EGL_SAMPLES,		0,
		EGL_NONE
	};

	EGLint numconfigs;

	#ifdef EGL_USE_X11
	printf("XOpenDisplay \n");
	display = XOpenDisplay(NULL);
	assert(display != NULL);	

	int screen;
	printf("DefaultScreen [");
	screen = DefaultScreen(display);
	printf("%d]\n", screen);

	Window window, rootwindow; 
	printf("RootWindow \n");
	rootwindow = RootWindow(display,screen); 


	printf("eglGetDisplay (%d)\n", (int)display);
	egldisplay = eglGetDisplay ( display);
	assert(eglGetError() == EGL_SUCCESS);

	printf("Disp = %d \n",(int) egldisplay);

	printf("eglInitialize \n");
	eglInitialize(egldisplay, NULL, NULL);
	assert(eglGetError() == EGL_SUCCESS);

	printf("eglBindAPI \n");
	eglBindAPI(EGL_OPENGL_ES_API);

	printf("eglChooseConfig \n");
	eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
	assert(eglGetError() == EGL_SUCCESS);

	assert(numconfigs == 1);

	printf("XCreateSimpleWindow \n");
	window = XCreateSimpleWindow(display, rootwindow, 0, 0, 400, 533, 0, 0, WhitePixel (display, screen));

	XMapWindow(display, window);

	printf("*eglCreateWindowSurface \n");
	eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, window, NULL);
	assert(eglGetError() == EGL_SUCCESS);
  EGLint ContextAttribList[] = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE };

	#else

	egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
	eglInitialize(egldisplay, NULL, NULL);
	assert(eglGetError() == EGL_SUCCESS);
	eglBindAPI(EGL_OPENGL_ES_API);

	eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
	assert(eglGetError() == EGL_SUCCESS);
	assert(numconfigs == 1);

	//You must pass in the file system handle to the linux framebuffer when creating a window
	eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, open("/dev/fb0", O_RDWR), NULL);
	assert(eglGetError() == EGL_SUCCESS);
	EGLint ContextAttribList[] = { EGL_CONTEXT_CLIENT_VERSION, 2, EGL_NONE };

	

	#endif
   
	 eglcontext = eglCreateContext( egldisplay, eglconfig, EGL_NO_CONTEXT, ContextAttribList );
	assert(eglGetError() == EGL_SUCCESS);
	eglMakeCurrent(egldisplay, eglsurface, eglsurface, eglcontext);
	assert(eglGetError() == EGL_SUCCESS);


	GLfloat xmin, xmax, ymin, ymax;
	EGLint h, w;	
	eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &w);
	eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &h);
    	
    	ymax = 0.1f * tan(45.0f * M_PI/360.0f);
    	ymin = -ymax;
	
    	xmin = ymin * ((GLfloat)w/(GLfloat)h);
    	xmax = ymax * ((GLfloat)w/(GLfloat)h);




	 {
        // Compile the shaders
        GLuint hVertexShader = glCreateShader( GL_VERTEX_SHADER );
        glShaderSource( hVertexShader, 1, &g_strVertexShader, NULL );
        glCompileShader( hVertexShader );

        // Check for compile success
        GLint nCompileResult = 0;
        glGetShaderiv( hVertexShader, GL_COMPILE_STATUS, &nCompileResult );
        if( 0 == nCompileResult )
        {
            char  strLog[1024];
            GLint nLength;
            glGetShaderInfoLog( hVertexShader, 1024, &nLength, strLog );
            return FALSE;
        }

        GLuint hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
        glShaderSource( hFragmentShader, 1, &g_strFragmentShader, NULL );
        glCompileShader( hFragmentShader );

        // Check for compile success
        glGetShaderiv( hFragmentShader, GL_COMPILE_STATUS, &nCompileResult );
        if( 0 == nCompileResult )
        {
            char  strLog[1024];
            GLint nLength;
            glGetShaderInfoLog( hFragmentShader, 1024, &nLength, strLog );
            return FALSE;
        }

        // Attach the individual shaders to the common shader program
        g_hShaderProgram = glCreateProgram();
        glAttachShader( g_hShaderProgram, hVertexShader );
        glAttachShader( g_hShaderProgram, hFragmentShader );

        // Init attributes BEFORE linking
        glBindAttribLocation( g_hShaderProgram, g_hVertexLoc,   "g_vPosition" );
       
        // Link the vertex shader and fragment shader together
        glLinkProgram( g_hShaderProgram );

        // Check for link success
        GLint nLinkResult = 0;
        glGetProgramiv( g_hShaderProgram, GL_LINK_STATUS, &nLinkResult );
        if( 0 == nLinkResult )
        {
            char strLog[1024];
            GLint nLength;
            glGetProgramInfoLog( g_hShaderProgram, 1024, &nLength, strLog );
            return FALSE;
        }

        // Get uniform locations
        g_hModelViewMatrixLoc = glGetUniformLocation( g_hShaderProgram, "g_matModelView" );
        g_hProjMatrixLoc      = glGetUniformLocation( g_hShaderProgram, "g_matProj" );

        glDeleteShader( hVertexShader );
        glDeleteShader( hFragmentShader );


	//gen textures
	
    }

    	
    	return 1;
}

void Cleanup()
{
    
}

void deinit(void)
{
	Cleanup();
	eglMakeCurrent(egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	assert(eglGetError() == EGL_SUCCESS);
	eglTerminate(egldisplay);
	assert(eglGetError() == EGL_SUCCESS);
	eglReleaseThread();
	#ifdef EGL_USE_X11	
	XCloseDisplay(display);
	#endif
}

int main (void)
{
	assert( init() );

	while (currentFrame < 1000)
	{
		EGLint width = 0;
		EGLint height = 0;
		eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &height);
		render(width, height);
		currentFrame++;
		eglSwapBuffers(egldisplay, eglsurface);
	}
	deinit();
return 0;
}

